/************************************************************************/
/*      $Id: d70tif.cpp,v 1.11 2006/01/12 23:05:42 joman Exp joman $    */
/*                                                                      */
/*      Source file name:   d70tif.cpp                                  */
/*                                                                      */
/************************************************************************/
/*      Revisions:                                                      */
/*                                                                      */
/*      Version 0.01 - Tue May 10 14:47:38 EDT 2005:                    */
/*              First coding.                                           */
/************************************************************************/
//----------------------------------------------------------------------//
//                                                                      //
//   File:   d70tif.cpp                                                 //
//                                                                      //
//   Desc:   Structure and Class definitions for the                    //
//           Tagged Image File Format Revision 6.0                      //
//                                                                      //
//----------------------------------------------------------------------//

#ifdef WIN_GUI
#include "stdafx.h"
#endif

#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <list>

#include "d70revbyte.hpp"
#include "d70tif.hpp"
#include "d70claifd.hpp"
#include "d70inc2.h"

//..................various constant/name sets

//static int reverse_byte_order = 0;

int ReverseByteOrder( void ) { return reverse_byte_order; }

//..................various constant/name sets

static struct TAGNAMES TagNames[] =
{
  { 0, 65515u, "VRMode_Type0                     " },
  { 0, 65516u, "NearFocusLimitDOF_Type0          " },
  { 0, 65517u, "FarFocusLimitDOF_Type0           " },
  { 0, 65518u, "HyperfocalDistance_Type0         " },
  { 0, 65519u, "TotalDOF_Type0                   " },
  { 0, 65520u, "CircleOfConfusion_Type0          " },

  { 0, 65521u, "AFAperture_Type0                 " },
  { 0, 65522u, "FocusPosition_Type0              " },
  { 0, 65523u, "MinFocalLength_Type0             " },
  { 0, 65524u, "MaxFocalLength_Type0             " },
  { 0, 65525u, "MaxApertureAtMinFocal_Type0      " },
  { 0, 65526u, "MaxApertureAtMaxFocal_Type0      " },
  { 0, 65527u, "MCUVersion_Type0                 " },
  { 0, 65528u, "EffectiveMaxAperture_Type0       " },
  { 0, 65529u, "LensDataVersion_Type0            " },
  { 0, 65530u, "FocusDistance_Type0              " },
  { 0, 65531u, "LensID_Type0                     " },
  { 0, 65532u, "FileSize_Type0                   " },
  { 0, 65533u, "FileName_Type0                   " },
  { 0, 65534u, "D70ReaderProgramVersion_Type0    " },
  { 0, 65535u, "IgnoredTag_Type0                 " },

  { 1, 254,    "NewSubfileType_Type1             " },
  { 1, 256,    "ImageWidth_Type1                 " },
  { 1, 257,    "ImageHeight_Type1                " },
  { 1, 258,    "BitsPerSample_Type1              " },
  { 1, 259,    "Compression_Type1                " },
  { 1, 262,    "PhotometricInterpretation_Type1  " },
  { 1, 270,    "ImageDescription_Type1           " },
  { 1, 271,    "Make_Type1                       " },
  { 1, 272,    "Model_Type1                      " },
  { 1, 273,    "StripOffsets_Type1               " },
  { 1, 274,    "Orientation_Type1                " },
  { 1, 277,    "SamplesPerPixel_Type1            " },
  { 1, 278,    "RowsPerStrip_Type1               " },
  { 1, 279,    "StripByteCounts_Type1            " },
  { 1, 282,    "XResolution_Type1                " },
  { 1, 283,    "YResolution_Type1                " },
  { 1, 284,    "PlanarConfiguration_Type1        " },
  { 1, 296,    "ResolutionUnit_Type1             " },
  { 1, 305,    "Software_Type1                   " },
  { 1, 306,    "ModifyDate_Type1                 " },
  { 1, 315,    "Artist_Type1                     " },
  { 1, 318,    "WhitePoint_Type1                 " },
  { 1, 319,    "PrimaryChromaticities_Type1      " },
  { 1, 330,    "SubIFDOffsets_Type1              " },
  { 1, 529,    "YCbCrCoefficients_Type1          " },
  { 1, 531,    "YCbCrPositioning_Type1           " },
  { 1, 532,    "ReferenceBlackWhite_Type1        " },
  { 1, 33432u, "Copyright_Type1                  " },
  { 1, 34665u, "ExifOffset_Type1                 " },
  { 1, 34675u, "InterColorProfile_Type1          " },
  { 1, 34853u, "GPSInfoOffset_Type1              " },
  { 1, 36867u, "DateTimeOriginal_Type1           " },
  { 1, 37398u, "TIFF_EPStandardId_Type1          " },
  { 1, 41985u, "CustomRendered_Type1             " },
  { 1, 41986u, "ExposureMode_Type1               " },
  { 1, 41987u, "WhiteBalanceEXIF_Type1           " },
  { 1, 41988u, "DigitalZoomRatio_Type1           " },
  { 1, 41989u, "FocalLengthIn35mmFilm_Type1      " },
  { 1, 41990u, "SceneCaptureType_Type1           " },
  { 1, 41991u, "GainControl_Type1                " },
  { 1, 41992u, "Contrast_Type1                   " },
  { 1, 41993u, "SaturationEXIF_Type1             " },
  { 1, 41994u, "Sharpness_Type1                  " },
  { 1, 41996u, "SubjectDistanceRange_Type1       " },

/*
  { 1, 254,    "NewSubfileType_Type1             " },
  { 1, 256,    "ImageWidth_Type1                 " },
  { 1, 257,    "ImageHeight_Type1                " },
  { 1, 258,    "BitsPerSample_Type1              " },
  { 1, 259,    "Compression_Type1                " },
  { 1, 262,    "PhotometricInterpretation_Type1  " },
  { 1, 266,    "FillOrder_Type1                  " },
  { 1, 269,    "DocumentName_Type1               " },
  { 1, 270,    "ImageDescription_Type1           " },
  { 1, 271,    "Make_Type1                       " },
  { 1, 272,    "Model_Type1                      " },
  { 1, 273,    "StripOffsets_Type1               " },
  { 1, 274,    "Orientation_Type1                " },
  { 1, 277,    "SamplesPerPixel_Type1            " },
  { 1, 278,    "RowsPerStrip_Type1               " },
  { 1, 279,    "StripByteCounts_Type1            " },
  { 1, 282,    "XResolution_Type1                " },
  { 1, 283,    "YResolution_Type1                " },
  { 1, 284,    "PlanarConfiguration_Type1        " },
  { 1, 296,    "ResolutionUnit_Type1             " },
  { 1, 301,    "TransferFunction_Type1           " },
  { 1, 305,    "Software_Type1                   " },
  { 1, 306,    "ModifyDate_Type1                 " },
  { 1, 315,    "Artist_Type1                     " },
  { 1, 318,    "WhitePoint_Type1                 " },
  { 1, 319,    "PrimaryChromaticities_Type1      " },
  { 1, 330,    "SubIFDOffsets_Type1              " },
  { 1, 342,    "TransferRange_Type1              " },
  { 1, 512,    "JPEGProc_Type1                   " },
  { 1, 513,    "ThumbnailOffset_Type1            " },
  { 1, 514,    "ThumbnailLength_Type1            " },
  { 1, 529,    "YCbCrCoefficients_Type1          " },
  { 1, 530,    "YCbCrSubSampling_Type1           " },
  { 1, 531,    "YCbCrPositioning_Type1           " },
  { 1, 532,    "ReferenceBlackWhite_Type1        " },
  { 1, 3585,   "Unknown_Type1                    " },
  { 1, 3589,   "Unknown_Type1                    " },
  { 1, 3592,   "Unknown_Type1                    " },
  { 1, 3593,   "SoftwareNC_Type1                 " },
  { 1, 3598,   "NikonCaptureOffsets_Type1        " },
  { 1, 3600,   "Unknown_Type1                    " },
  { 1, 3609,   "Unknown_Type1                    " },
  { 1, 4097,   "RelatedImageWidth_Type1          " },
  { 1, 4098,   "RelatedImageLength_Type1         " },
  { 1, 34665u, "ExifOffset_Type1                 " },
  { 1, 34675u, "InterColorProfile_Type1          " },
  { 1, 34853u, "GPSInfoOffset_Type1              " },
  { 1, 36867u, "DateTimeOriginal_Type1           " },
  { 1, 36868u, "DateTimeDigitized_Type1          " },
  { 1, 37398u, "TIFF_EPStandardId_Type1          " },
  { 1, 41985u, "CustomRendered_Type1             " },
  { 1, 41986u, "ExposureMode_Type1               " },
  { 1, 41987u, "WhiteBalanceEXIF_Type1           " },
  { 1, 41988u, "DigitalZoomRatio_Type1           " },
  { 1, 41989u, "FocalLengthIn35mmFilm_Type1      " },
  { 1, 41990u, "SceneCaptureType_Type1           " },
  { 1, 41991u, "GainControl_Type1                " },
  { 1, 41992u, "Contrast_Type1                   " },
  { 1, 41993u, "SaturationEXIF_Type1             " },
  { 1, 41994u, "Sharpness_Type1                  " },
  { 1, 41996u, "SubjectDistanceRange_Type1       " },
*/
  { 2, 254,    "NewSubfileType_Type2             " },
  { 2, 256,    "ImageWidth_Type2                 " },
  { 2, 257,    "ImageHeight_Type2                " },
  { 2, 258,    "BitsPerSample_Type2              " },
  { 2, 259,    "Compression_Type2                " },
  { 2, 262,    "PhotometricInterpretation_Type2  " },
  { 2, 273,    "StripOffsets_Type2               " },
  { 2, 274,    "Orientation_Type2                " },
  { 2, 277,    "SamplesPerPixel_Type2            " },
  { 2, 278,    "RowsPerStrip_Type2               " },
  { 2, 279,    "StripByteCounts_Type2            " },
  { 2, 282,    "XResolution_Type2                " },
  { 2, 283,    "YResolution_Type2                " },
  { 2, 284,    "PlanarConfiguration_Type2        " },
  { 2, 296,    "ResolutionUnit_Type2             " },
  { 2, 513,    "ThumbnailOffset_Type2            " },
  { 2, 514,    "ThumbnailLength_Type2            " },
  { 2, 531,    "YCbCrPositioning_Type2           " },
  { 2, 33421u, "CFARepeatPatternDim_Type2        " },
  { 2, 33422u, "CFAPattern_Type2                 " },
  { 2, 37399u, "SensingMethod_Type2              " },

/*
  { 2, 254,    "NewSubfileType_Type2             " },
  { 2, 256,    "ImageWidth_Type2                 " },
  { 2, 257,    "ImageHeight_Type2                " },
  { 2, 258,    "BitsPerSample_Type2              " },
  { 2, 259,    "Compression_Type2                " },
  { 2, 262,    "PhotometricInterpretation_Type2  " },
  { 2, 266,    "FillOrder_Type2                  " },
  { 2, 269,    "DocumentName_Type2               " },
  { 2, 270,    "ImageDescription_Type2           " },
  { 2, 271,    "Make_Type2                       " },
  { 2, 272,    "Model_Type2                      " },
  { 2, 273,    "StripOffsets_Type2               " },
  { 2, 274,    "Orientation_Type2                " },
  { 2, 277,    "SamplesPerPixel_Type2            " },
  { 2, 278,    "RowsPerStrip_Type2               " },
  { 2, 279,    "StripByteCounts_Type2            " },
  { 2, 282,    "XResolution_Type2                " },
  { 2, 283,    "YResolution_Type2                " },
  { 2, 284,    "PlanarConfiguration_Type2        " },
  { 2, 296,    "ResolutionUnit_Type2             " },
  { 2, 301,    "TransferFunction_Type2           " },
  { 2, 305,    "Software_Type2                   " },
  { 2, 306,    "ModifyDate_Type2                 " },
  { 2, 315,    "Artist_Type2                     " },
  { 2, 318,    "WhitePoint_Type2                 " },
  { 2, 319,    "PrimaryChromaticities_Type2      " },
  { 2, 330,    "SubIFDOffsets_Type2              " },
  { 2, 342,    "TransferRange_Type2              " },
  { 2, 512,    "JPEGProc_Type2                   " },
  { 2, 513,    "ThumbnailOffset_Type2            " },
  { 2, 514,    "ThumbnailLength_Type2            " },
  { 2, 529,    "YCbCrCoefficients_Type2          " },
  { 2, 530,    "YCbCrSubSampling_Type2           " },
  { 2, 531,    "YCbCrPositioning_Type2           " },
  { 2, 532,    "ReferenceBlackWhite_Type2        " },
*/

  { 3, 254,    "NewSubfileType_Type3             " },
  { 3, 256,    "ImageWidth_Type3                 " },
  { 3, 257,    "ImageHeight_Type3                " },
  { 3, 258,    "BitsPerSample_Type3              " },
  { 3, 259,    "Compression_Type3                " },
  { 3, 262,    "PhotometricInterpretation_Type3  " },
  { 3, 273,    "StripOffsets_Type3               " },
  { 3, 274,    "Orientation_Type3                " },
  { 3, 277,    "SamplesPerPixel_Type3            " },
  { 3, 278,    "RowsPerStrip_Type3               " },
  { 3, 279,    "StripByteCounts_Type3            " },
  { 3, 282,    "XResolution_Type3                " },
  { 3, 283,    "YResolution_Type3                " },
  { 3, 284,    "PlanarConfiguration_Type3        " },
  { 3, 296,    "ResolutionUnit_Type3             " },
  { 3, 513,    "ThumbnailOffset_Type3            " },
  { 3, 514,    "ThumbnailLength_Type3            " },
  { 3, 33421u, "CFARepeatPatternDim_Type3        " },
  { 3, 33422u, "CFAPattern_Type3                 " },
  { 3, 37399u, "SensingMethod_Type3              " },

/*
  { 3, 254,    "NewSubfileType_Type3             " },
  { 3, 256,    "ImageWidth_Type3                 " },
  { 3, 257,    "ImageHeight_Type3                " },
  { 3, 258,    "BitsPerSample_Type3              " },
  { 3, 259,    "Compression_Type3                " },
  { 3, 262,    "PhotometricInterpretation_Type3  " },
  { 3, 266,    "FillOrder_Type3                  " },
  { 3, 269,    "DocumentName_Type3               " },
  { 3, 270,    "ImageDescription_Type3           " },
  { 3, 271,    "Make_Type3                       " },
  { 3, 272,    "Model_Type3                      " },
  { 3, 273,    "StripOffsets_Type3               " },
  { 3, 274,    "Orientation_Type3                " },
  { 3, 277,    "SamplesPerPixel_Type3            " },
  { 3, 278,    "RowsPerStrip_Type3               " },
  { 3, 279,    "StripByteCounts_Type3            " },
  { 3, 282,    "XResolution_Type3                " },
  { 3, 283,    "YResolution_Type3                " },
  { 3, 284,    "PlanarConfiguration_Type3        " },
  { 3, 296,    "ResolutionUnit_Type3             " },
  { 3, 301,    "TransferFunction_Type3           " },
  { 3, 305,    "Software_Type3                   " },
  { 3, 306,    "ModifyDate_Type3                 " },
  { 3, 315,    "Artist_Type3                     " },
  { 3, 318,    "WhitePoint_Type3                 " },
  { 3, 319,    "PrimaryChromaticities_Type3      " },
  { 3, 330,    "SubIFDOffsets_Type3              " },
  { 3, 342,    "TransferRange_Type3              " },
  { 3, 512,    "JPEGProc_Type3                   " },
  { 3, 513,    "ThumbnailOffset_Type3            " },
  { 3, 514,    "ThumbnailLength_Type3            " },
  { 3, 529,    "YCbCrCoefficients_Type3          " },
  { 3, 530,    "YCbCrSubSampling_Type3           " },
  { 3, 531,    "YCbCrPositioning_Type3           " },
  { 3, 532,    "ReferenceBlackWhite_Type3        " },
  { 3, 33421u, "CFARepeatPatternDim_Type3        " },
  { 3, 33422u, "CFAPattern_Type3                 " },
  { 3, 37399u, "SensingMethod_Type3              " },
*/

  { 4, 254,    "NewSubfileType_Type4             " },
  { 4, 256,    "ImageWidth_Type4                 " },
  { 4, 257,    "ImageHeight_Type4                " },
  { 4, 258,    "BitsPerSample_Type4              " },
  { 4, 259,    "Compression_Type4                " },
  { 4, 262,    "PhotometricInterpretation_Type4  " },
  { 4, 273,    "StripOffsets_Type4               " },
  { 4, 274,    "Orientation_Type4                " },
  { 4, 277,    "SamplesPerPixel_Type4            " },
  { 4, 278,    "RowsPerStrip_Type4               " },
  { 4, 279,    "StripByteCounts_Type4            " },
  { 4, 282,    "XResolution_Type4                " },
  { 4, 283,    "YResolution_Type4                " },
  { 4, 284,    "PlanarConfiguration_Type4        " },
  { 4, 296,    "ResolutionUnit_Type4             " },
  { 4, 33421u, "CFARepeatPatternDim_Type4        " },
  { 4, 33422u, "CFAPattern_Type4                 " },
/*
  { 4, 254,    "NewSubfileType_Type4             " },
  { 4, 256,    "ImageWidth_Type4                 " },
  { 4, 257,    "ImageHeight_Type4                " },
  { 4, 258,    "BitsPerSample_Type4              " },
  { 4, 259,    "Compression_Type4                " },
  { 4, 262,    "PhotometricInterpretation_Type4  " },
  { 4, 266,    "FillOrder_Type4                  " },
  { 4, 269,    "DocumentName_Type4               " },
  { 4, 270,    "ImageDescription_Type4           " },
  { 4, 271,    "Make_Type4                       " },
  { 4, 272,    "Model_Type4                      " },
  { 4, 273,    "StripOffsets_Type4               " },
  { 4, 274,    "Orientation_Type4                " },
  { 4, 277,    "SamplesPerPixel_Type4            " },
  { 4, 278,    "RowsPerStrip_Type4               " },
  { 4, 279,    "StripByteCounts_Type4            " },
  { 4, 282,    "XResolution_Type4                " },
  { 4, 283,    "YResolution_Type4                " },
  { 4, 284,    "PlanarConfiguration_Type4        " },
  { 4, 296,    "ResolutionUnit_Type4             " },
  { 4, 301,    "TransferFunction_Type4           " },
  { 4, 305,    "Software_Type4                   " },
  { 4, 306,    "ModifyDate_Type4                 " },
  { 4, 315,    "Artist_Type4                     " },
  { 4, 318,    "WhitePoint_Type4                 " },
  { 4, 319,    "PrimaryChromaticities_Type4      " },
  { 4, 330,    "SubIFDOffsets_Type4              " },
  { 4, 342,    "TransferRange_Type4              " },
  { 4, 512,    "JPEGProc_Type4                   " },
  { 4, 513,    "ThumbnailOffset_Type4            " },
  { 4, 514,    "ThumbnailLength_Type4            " },
  { 4, 529,    "YCbCrCoefficients_Type4          " },
  { 4, 530,    "YCbCrSubSampling_Type4           " },
  { 4, 531,    "YCbCrPositioning_Type4           " },
  { 4, 532,    "ReferenceBlackWhite_Type4        " },
  { 4, 3585,   "Unknown_Type4                    " },
  { 4, 3589,   "Unknown_Type4                    " },
  { 4, 3592,   "Unknown_Type4                    " },
  { 4, 3593,   "SoftwareNC_Type4                 " },
  { 4, 3598,   "NikonCaptureOffsets_Type4        " },
  { 4, 3600,   "Unknown_Type4                    " },
  { 4, 3609,   "Unknown_Type4                    " },
  { 4, 4097,   "RelatedImageWidth_Type4          " },
  { 4, 4098,   "RelatedImageLength_Type4         " },
  { 4, 33421u, "CFARepeatPatternDim_Type4        " },
  { 4, 33422u, "CFAPattern_Type4                 " },
  { 4, 33423u, "BatteryLevel_Type4               " },
  { 4, 33432u, "Copyright_Type4                  " },
  { 4, 33434u, "ExposureTime_Type4               " },
  { 4, 33437u, "FNumber_Type4                    " },
  { 4, 33723u, "IPTC/NAA_Type4                   " },
  { 4, 34665u, "ExifOffset_Type4                 " },
  { 4, 34675u, "InterColorProfile_Type4          " },
  { 4, 34850u, "ExposureProgram_Type4            " },
  { 4, 34852u, "SpectralSensitivity_Type4        " },
  { 4, 34853u, "GPSInfoOffset_Type4              " },
  { 4, 34855u, "ISOSpeedRatings_Type4            " },
  { 4, 34856u, "OECF_Type4                       " },
  { 4, 36864u, "ExifVersion_Type4                " },
  { 4, 36867u, "DateTimeOriginal_Type4           " },
  { 4, 36868u, "DateTimeDigitized_Type4          " },
  { 4, 37121u, "ComponentsConfiguration_Type4    " },
  { 4, 37122u, "CompressedBitsPerPixel_Type4     " },
  { 4, 37377u, "ShutterSpeedValue_Type4          " },
  { 4, 37378u, "ApertureValue_Type4              " },
  { 4, 37379u, "BrightnessValue_Type4            " },
  { 4, 37380u, "ExposureBiasValue_Type4          " },
  { 4, 37381u, "MaxApertureValue_Type4           " },
  { 4, 37382u, "SubjectDistance_Type4            " },
  { 4, 37383u, "MeteringMode_Type4               " },
  { 4, 37384u, "LightSourceEXIF_Type4            " },
  { 4, 37385u, "Flash_Type4                      " },
  { 4, 37386u, "FocalLength_Type4                " },
  { 4, 37398u, "TIFF_EPStandardId_Type4          " },
  { 4, 37399u, "SensingMethod_Type4              " },
  { 4, 37500u, "MakerNoteOffset_Type4            " },
  { 4, 37510u, "UserComment_Type4                " },
  { 4, 37520u, "SubSecTime_Type4                 " },
  { 4, 37521u, "SubSecTimeOriginal_Type4         " },
  { 4, 37522u, "SubSecTimeDigitized_Type4        " },
*/

#ifdef JON
  { 5, 1,      "FileSystemVersion_Type5          " },
  { 5, 2,      "ISOUsed_Type5                    " },
  { 5, 3,      "ColorMode_Type5                  " },
  { 5, 4,      "Quality_Type5                    " },
  { 5, 5,      "WhiteBalance_Type5               " },
  { 5, 6,      "Sharpening_Type5                 " },
  { 5, 7,      "FocusMode_Type5                  " },
  { 5, 8,      "FlashSetting_Type5               " },
  { 5, 9,      "FlashType_Type5                  " },
  { 5, 10,     "Unknown_Type5                    " },
  { 5, 11,     "WhiteBalanceFine_Type5           " },
  { 5, 12,     "ColorBalance12_Type5             " },
  { 5, 13,     "Unknown_Type5                    " },
  { 5, 14,     "ExposureDifference_Type5         " },
  { 5, 17,     "ThumbnailImageIFDOffset_Type5    " },
  { 5, 18,     "FlashCompensation_Type5          " },
  { 5, 19,     "ISOSet_Type5                     " },
  { 5, 22,     "ImageBoundary_Type5              " },
  { 5, 23,     "Unknown_Type5                    " },
  { 5, 24,     "FlashExposureBracketValue_Type5  " },
  { 5, 25,     "ExposureBracketValue_Type5       " },
  { 5, 26,     "Unknown_Type5                    " },
  { 5, 27,     "Unknown_Type5                    " },
  { 5, 28,     "Unknown_Type5                    " },
  { 5, 29,     "SerialNumber29_Type5             " },
  { 5, 30,     "Unknown_Type5                    " },
  { 5, 129,    "ToneCompensation_Type5           " },
  { 5, 131,    "LensType_Type5                   " },
  { 5, 132,    "Lens_Type5                       " },
  { 5, 135,    "FlashMode_Type5                  " },
  { 5, 136,    "AFPoint_Type5                    " },
  { 5, 137,    "ShootingMode_Type5               " },
  { 5, 138,    "Unknown_Type5                    " },
  { 5, 139,    "LensFStops_Type5                 " },
  { 5, 140,    "NEFCurve1_Type5                  " },
  { 5, 141,    "ColorHue_Type5                   " },
  { 5, 142,    "Unknown_Type5                    " },
  { 5, 144,    "LightSource_Type5                " },
  { 5, 145,    "Unknown_Type5                    " },
  { 5, 146,    "HueAdjustment_Type5              " },
  { 5, 147,    "Unknown_Type5                    " },
  { 5, 149,    "NoiseReduction_Type5             " },
  { 5, 150,    "NEFCurve2_Type5                  " },
  { 5, 151,    "ColorBalance151_Type5            " },
  { 5, 152,    "LensDataBytes_Type5              " },
  { 5, 153,    "NEFThumbnailSize_Type5           " },
  { 5, 154,    "SensorPixelSize_Type5            " },
  { 5, 160,    "SerialNumber_Type5               " },
  { 5, 161,    "Unknown_Type5                    " },
  { 5, 162,    "ImageDataSize_Type5              " },
  { 5, 163,    "Unknown_Type5                    " },
  { 5, 164,    "Unknown_Type5                    " },
  { 5, 165,    "Unknown_Type5                    " },
  { 5, 166,    "Unknown_Type5                    " },
  { 5, 167,    "ShutterCount_Type5               " },
  { 5, 168,    "Unknown_Type5                    " },
  { 5, 169,    "ImageOptimization_Type5          " },
  { 5, 170,    "Saturation_Type5                 " },
  { 5, 171,    "VariProgram_Type5                " },
  { 5, 176,    "Unknown_Type5                    " },
  { 5, 177,    "Unknown_Type5                    " },
  { 5, 254,    "NewSubfileType_Type5             " },
  { 5, 256,    "ImageWidth_Type5                 " },
  { 5, 257,    "ImageHeight_Type5                " },
  { 5, 258,    "BitsPerSample_Type5              " },
  { 5, 259,    "Compression_Type5                " },
  { 5, 262,    "PhotometricInterpretation_Type5  " },
  { 5, 266,    "FillOrder_Type5                  " },
  { 5, 269,    "DocumentName_Type5               " },
  { 5, 270,    "ImageDescription_Type5           " },
  { 5, 271,    "Make_Type5                       " },
  { 5, 272,    "Model_Type5                      " },
  { 5, 273,    "StripOffsets_Type5               " },
  { 5, 274,    "Orientation_Type5                " },
  { 5, 277,    "SamplesPerPixel_Type5            " },
  { 5, 278,    "RowsPerStrip_Type5               " },
  { 5, 279,    "StripByteCounts_Type5            " },
  { 5, 282,    "XResolution_Type5                " },
  { 5, 283,    "YResolution_Type5                " },
  { 5, 284,    "PlanarConfiguration_Type5        " },
  { 5, 296,    "ResolutionUnit_Type5             " },
  { 5, 301,    "TransferFunction_Type5           " },
  { 5, 305,    "Software_Type5                   " },
  { 5, 306,    "ModifyDate_Type5                 " },
  { 5, 315,    "Artist_Type5                     " },
  { 5, 318,    "WhitePoint_Type5                 " },
  { 5, 319,    "PrimaryChromaticities_Type5      " },
  { 5, 330,    "SubIFDOffsets_Type5              " },
  { 5, 342,    "TransferRange_Type5              " },
  { 5, 512,    "JPEGProc_Type5                   " },
  { 5, 513,    "ThumbnailOffset_Type5            " },
  { 5, 514,    "ThumbnailLength_Type5            " },
  { 5, 529,    "YCbCrCoefficients_Type5          " },
  { 5, 530,    "YCbCrSubSampling_Type5           " },
  { 5, 531,    "YCbCrPositioning_Type5           " },
  { 5, 532,    "ReferenceBlackWhite_Type5        " },
  { 5, 3585,   "Unknown_Type5                    " },
  { 5, 3589,   "Unknown_Type5                    " },
  { 5, 3592,   "Unknown_Type5                    " },
  { 5, 3593,   "SoftwareNC_Type5                 " },
  { 5, 3598,   "NikonCaptureOffsets_Type5        " },
  { 5, 3600,   "Unknown_Type5                    " },
  { 5, 3609,   "Unknown_Type5                    " },
  { 5, 4097,   "RelatedImageWidth_Type5          " },
  { 5, 4098,   "RelatedImageLength_Type5         " },
  { 5, 33421u, "CFARepeatPatternDim_Type5        " },
  { 5, 33422u, "CFAPattern_Type5                 " },
  { 5, 33423u, "BatteryLevel_Type5               " },
  { 5, 33432u, "Copyright_Type5                  " },
  { 5, 33434u, "ExposureTime_Type5               " },
  { 5, 33437u, "FNumber_Type5                    " },
  { 5, 33723u, "IPTC/NAA_Type5                   " },
  { 5, 34665u, "ExifOffset_Type5                 " },
  { 5, 34675u, "InterColorProfile_Type5          " },
  { 5, 34850u, "ExposureProgram_Type5            " },
  { 5, 34852u, "SpectralSensitivity_Type5        " },
  { 5, 34853u, "GPSInfoOffset_Type5              " },
  { 5, 34855u, "ISOSpeedRatings_Type5            " },
  { 5, 34856u, "OECF_Type5                       " },
  { 5, 36864u, "ExifVersion_Type5                " },
  { 5, 36867u, "DateTimeOriginal_Type5           " },
  { 5, 36868u, "DateTimeDigitized_Type5          " },
  { 5, 37121u, "ComponentsConfiguration_Type5    " },
  { 5, 37122u, "CompressedBitsPerPixel_Type5     " },
  { 5, 37377u, "ShutterSpeedValue_Type5          " },
  { 5, 37378u, "ApertureValue_Type5              " },
  { 5, 37379u, "BrightnessValue_Type5            " },
  { 5, 37380u, "ExposureBiasValue_Type5          " },
  { 5, 37381u, "MaxApertureValue_Type5           " },
  { 5, 37382u, "SubjectDistance_Type5            " },
  { 5, 37383u, "MeteringMode_Type5               " },
  { 5, 37384u, "LightSourceEXIF_Type5            " },
  { 5, 37385u, "Flash_Type5                      " },
  { 5, 37386u, "FocalLength_Type5                " },
  { 5, 37398u, "TIFF_EPStandardId_Type5          " },
  { 5, 37399u, "SensingMethod_Type5              " },
  { 5, 37500u, "MakerNoteOffset_Type5            " },
  { 5, 37510u, "UserComment_Type5                " },
  { 5, 37520u, "SubSecTime_Type5                 " },
  { 5, 37521u, "SubSecTimeOriginal_Type5         " },
  { 5, 37522u, "SubSecTimeDigitized_Type5        " },
  { 5, 40960u, "FlashPixVersion_Type5            " },
  { 5, 40961u, "ColorSpace_Type5                 " },
  { 5, 40962u, "ExifImageWidth_Type5             " },
  { 5, 40963u, "ExifImageLength_Type5            " },
  { 5, 40964u, "RelatedAudioFile_Type5           " },
  { 5, 40965u, "InteroperabilityOffset_Type5     " },
  { 5, 41483u, "FlashEnergy_Type5                " },
  { 5, 41484u, "SpatialFrequencyResponse_Type5   " },
  { 5, 41486u, "FocalPlaneXResolution_Type5      " },
  { 5, 41487u, "FocalPlaneYResolution_Type5      " },
  { 5, 41488u, "FocalPlaneResolutionUnit_Type5   " },
  { 5, 41492u, "SubjectLocation_Type5            " },
  { 5, 41493u, "ExposureIndex_Type5              " },
  { 5, 41495u, "SensingMethodEXIF_Type5          " },
  { 5, 41728u, "FileSource_Type5                 " },
  { 5, 41729u, "SceneType_Type5                  " },
  { 5, 41730u, "CFAPatternEXIF_Type5             " },
  { 5, 41985u, "CustomRendered_Type5             " },
  { 5, 41986u, "ExposureMode_Type5               " },
  { 5, 41987u, "WhiteBalanceEXIF_Type5           " },
  { 5, 41988u, "DigitalZoomRatio_Type5           " },
  { 5, 41989u, "FocalLengthIn35mmFilm_Type5      " },
  { 5, 41990u, "SceneCaptureType_Type5           " },
  { 5, 41991u, "GainControl_Type5                " },
  { 5, 41992u, "Contrast_Type5                   " },
  { 5, 41993u, "SaturationEXIF_Type5             " },
  { 5, 41994u, "Sharpness_Type5                  " },
  { 5, 41996u, "SubjectDistanceRange_Type5       " },
  { 5, 42240u, "Gamma_Type5                      " },
#endif	// JON

  { 6, 33434u, "ExposureTime_Type6               " },
  { 6, 33437u, "FNumber_Type6                    " },
  { 6, 34850u, "ExposureProgram_Type6            " },
  { 6, 34855u, "ISOSpeedRatings_Type6            " },
  { 6, 36864u, "ExifVersion_Type6                " },
  { 6, 36867u, "DateTimeOriginal_Type6           " },
  { 6, 36868u, "DateTimeDigitized_Type6          " },
  { 6, 37121u, "ComponentsConfiguration_Type6    " },
  { 6, 37122u, "CompressedBitsPerPixel_Type6     " },
  { 6, 37377u, "ShutterSpeedValue_Type6          " },
  { 6, 37378u, "ApertureValue_Type6              " },
  { 6, 37380u, "ExposureBiasValue_Type6          " },
  { 6, 37381u, "MaxApertureValue_Type6           " },
  { 6, 37382u, "SubjectDistance_Type6            " },
  { 6, 37383u, "MeteringMode_Type6               " },
  { 6, 37384u, "LightSourceEXIF_Type6            " },
  { 6, 37385u, "Flash_Type6                      " },
  { 6, 37386u, "FocalLength_Type6                " },
  { 6, 37500u, "MakerNoteOffset_Type6            " },
  { 6, 37510u, "UserComment_Type6                " },
  { 6, 37520u, "SubSecTime_Type6                 " },
  { 6, 37521u, "SubSecTimeOriginal_Type6         " },
  { 6, 37522u, "SubSecTimeDigitized_Type6        " },
  { 6, 40960u, "FlashPixVersion_Type6            " },
  { 6, 40961u, "ColorSpace_Type6                 " },
  { 6, 40962u, "ExifImageWidth_Type6             " },
  { 6, 40963u, "ExifImageLength_Type6            " },
  { 6, 40964u, "RelatedAudioFile_Type6           " },
  { 6, 40965u, "InteroperabilityOffset_Type6     " },
  { 6, 41486u, "FocalPlaneXResolution_Type6      " },
  { 6, 41487u, "FocalPlaneYResolution_Type6      " },
  { 6, 41488u, "FocalPlaneResolutionUnit_Type6   " },
  { 6, 41495u, "SensingMethodEXIF_Type6          " },
  { 6, 41728u, "FileSource_Type6                 " },
  { 6, 41729u, "SceneType_Type6                  " },
  { 6, 41730u, "CFAPatternEXIF_Type6             " },
  { 6, 41985u, "CustomRendered_Type6             " },
  { 6, 41986u, "ExposureMode_Type6               " },
  { 6, 41987u, "WhiteBalanceEXIF_Type6           " },
  { 6, 41988u, "DigitalZoomRatio_Type6           " },
  { 6, 41989u, "FocalLengthIn35mmFilm_Type6      " },
  { 6, 41990u, "SceneCaptureType_Type6           " },
  { 6, 41991u, "GainControl_Type6                " },
  { 6, 41992u, "Contrast_Type6                   " },
  { 6, 41993u, "SaturationEXIF_Type6             " },
  { 6, 41994u, "Sharpness_Type6                  " },
  { 6, 41996u, "SubjectDistanceRange_Type6       " },
  { 6, 42240u, "Gamma_Type6                      " },

/*
  { 6, 33421u, "CFARepeatPatternDim_Type6        " },
  { 6, 33422u, "CFAPattern_Type6                 " },
  { 6, 33423u, "BatteryLevel_Type6               " },
  { 6, 33432u, "Copyright_Type6                  " },
  { 6, 33434u, "ExposureTime_Type6               " },
  { 6, 33437u, "FNumber_Type6                    " },
  { 6, 33723u, "IPTC/NAA_Type6                   " },
  { 6, 34665u, "ExifOffset_Type6                 " },
  { 6, 34675u, "InterColorProfile_Type6          " },
  { 6, 34850u, "ExposureProgram_Type6            " },
  { 6, 34852u, "SpectralSensitivity_Type6        " },
  { 6, 34853u, "GPSInfoOffset_Type6              " },
  { 6, 34855u, "ISOSpeedRatings_Type6            " },
  { 6, 34856u, "OECF_Type6                       " },
  { 6, 36864u, "ExifVersion_Type6                " },
  { 6, 36867u, "DateTimeOriginal_Type6           " },
  { 6, 36868u, "DateTimeDigitized_Type6          " },
  { 6, 37121u, "ComponentsConfiguration_Type6    " },
  { 6, 37122u, "CompressedBitsPerPixel_Type6     " },
  { 6, 37377u, "ShutterSpeedValue_Type6          " },
  { 6, 37378u, "ApertureValue_Type6              " },
  { 6, 37379u, "BrightnessValue_Type6            " },
  { 6, 37380u, "ExposureBiasValue_Type6          " },
  { 6, 37381u, "MaxApertureValue_Type6           " },
  { 6, 37382u, "SubjectDistance_Type6            " },
  { 6, 37383u, "MeteringMode_Type6               " },
  { 6, 37384u, "LightSourceEXIF_Type6            " },
  { 6, 37385u, "Flash_Type6                      " },
  { 6, 37386u, "FocalLength_Type6                " },
  { 6, 37398u, "TIFF_EPStandardId_Type6          " },
  { 6, 37399u, "SensingMethod_Type6              " },
  { 6, 37500u, "MakerNoteOffset_Type6            " },
  { 6, 37510u, "UserComment_Type6                " },
  { 6, 37520u, "SubSecTime_Type6                 " },
  { 6, 37521u, "SubSecTimeOriginal_Type6         " },
  { 6, 37522u, "SubSecTimeDigitized_Type6        " },
  { 6, 40960u, "FlashPixVersion_Type6            " },
  { 6, 40961u, "ColorSpace_Type6                 " },
  { 6, 40962u, "ExifImageWidth_Type6             " },
  { 6, 40963u, "ExifImageLength_Type6            " },
  { 6, 40964u, "RelatedAudioFile_Type6           " },
  { 6, 40965u, "InteroperabilityOffset_Type6     " },
  { 6, 41483u, "FlashEnergy_Type6                " },
  { 6, 41484u, "SpatialFrequencyResponse_Type6   " },
  { 6, 41486u, "FocalPlaneXResolution_Type6      " },
  { 6, 41487u, "FocalPlaneYResolution_Type6      " },
  { 6, 41488u, "FocalPlaneResolutionUnit_Type6   " },
  { 6, 41492u, "SubjectLocation_Type6            " },
  { 6, 41493u, "ExposureIndex_Type6              " },
  { 6, 41495u, "SensingMethodEXIF_Type6          " },
  { 6, 41728u, "FileSource_Type6                 " },
  { 6, 41729u, "SceneType_Type6                  " },
  { 6, 41730u, "CFAPatternEXIF_Type6             " },
  { 6, 41985u, "CustomRendered_Type6             " },
  { 6, 41986u, "ExposureMode_Type6               " },
  { 6, 41987u, "WhiteBalanceEXIF_Type6           " },
  { 6, 41988u, "DigitalZoomRatio_Type6           " },
  { 6, 41989u, "FocalLengthIn35mmFilm_Type6      " },
  { 6, 41990u, "SceneCaptureType_Type6           " },
  { 6, 41991u, "GainControl_Type6                " },
  { 6, 41992u, "Contrast_Type6                   " },
  { 6, 41993u, "SaturationEXIF_Type6             " },
  { 6, 41994u, "Sharpness_Type6                  " },
  { 6, 41996u, "SubjectDistanceRange_Type6       " },
  { 6, 42240u, "Gamma_Type6                      " },
*/

  { 7, 0x0000, "GPSVersionID_Type7               " },
  { 7, 0x0001, "GPSLatitudeRef_Type7             " },
  { 7, 0x0002, "GPSLatitude_Type7                " },
  { 7, 0x0003, "GPSLongitudeRef_Type7            " },
  { 7, 0x0004, "GPSLongitude_Type7               " },
  { 7, 0x0005, "GPSAltitudeRef_Type7             " },
  { 7, 0x0006, "GPSAltitude_Type7                " },
  { 7, 0x0007, "GPSTimeStamp_Type7               " },
  { 7, 0x0008, "GPSSatellites_Type7              " },
  { 7, 0x0009, "GPSStatus_Type7                  " },
  { 7, 0x000a, "GPSMeasureMode_Type7             " },
  { 7, 0x000b, "GPSDOP_Type7                     " },
  { 7, 0x000c, "GPSSpeedRef_Type7                " },
  { 7, 0x000d, "GPSSpeed_Type7                   " },
  { 7, 0x000e, "GPSTrackRef_Type7                " },
  { 7, 0x000f, "GPSTrack_Type7                   " },
  { 7, 0x0010, "GPSImgDirectionRef_Type7         " },
  { 7, 0x0011, "GPSImgDirection_Type7            " },
  { 7, 0x0012, "GPSMapDatum_Type7                " },
  { 7, 0x0013, "GPSDestLatitudeRef_Type7         " },
  { 7, 0x0014, "GPSDestLatitude_Type7            " },
  { 7, 0x0015, "GPSDestLongitudeRef_Type7        " },
  { 7, 0x0016, "GPSDestLongitude_Type7           " },
  { 7, 0x0017, "GPSDestBearingRef_Type7          " },
  { 7, 0x0018, "GPSDestBearing_Type7             " },
  { 7, 0x0019, "GPSDestDistanceRef_Type7         " },
  { 7, 0x001a, "GPSDestDistance_Type7            " },
  { 7, 0x001b, "GPSProcessingMethod_Type7        " },
  { 7, 0x001c, "GPSAreaInformation_Type7         " },
  { 7, 0x001d, "GPSDateStamp_Type7               " },
  { 7, 0x001e, "GPSDifferential_Type7            " },

  { 8, 259,    "Compression_Type8                " },
  { 8, 282,    "XResolution_Type8                " },
  { 8, 283,    "YResolution_Type8                " },
  { 8, 296,    "ResolutionUnit_Type8             " },
  { 8, 513,    "ThumbnailOffset_Type8            " },
  { 8, 514,    "ThumbnailLength_Type8            " },
  { 8, 531,    "YCbCrPositioning_Type8           " },
/*
  { 8, 254,    "NewSubfileType_Type8             " },
  { 8, 256,    "ImageWidth_Type8                 " },
  { 8, 257,    "ImageHeight_Type8                " },
  { 8, 258,    "BitsPerSample_Type8              " },
  { 8, 259,    "Compression_Type8                " },
  { 8, 262,    "PhotometricInterpretation_Type8  " },
  { 8, 266,    "FillOrder_Type8                  " },
  { 8, 269,    "DocumentName_Type8               " },
  { 8, 270,    "ImageDescription_Type8           " },
  { 8, 271,    "Make_Type8                       " },
  { 8, 272,    "Model_Type8                      " },
  { 8, 273,    "StripOffsets_Type8               " },
  { 8, 274,    "Orientation_Type8                " },
  { 8, 277,    "SamplesPerPixel_Type8            " },
  { 8, 278,    "RowsPerStrip_Type8               " },
  { 8, 279,    "StripByteCounts_Type8            " },
  { 8, 282,    "XResolution_Type8                " },
  { 8, 283,    "YResolution_Type8                " },
  { 8, 284,    "PlanarConfiguration_Type8        " },
  { 8, 296,    "ResolutionUnit_Type8             " },
  { 8, 301,    "TransferFunction_Type8           " },
  { 8, 305,    "Software_Type8                   " },
  { 8, 306,    "ModifyDate_Type8                 " },
  { 8, 315,    "Artist_Type8                     " },
  { 8, 318,    "WhitePoint_Type8                 " },
  { 8, 319,    "PrimaryChromaticities_Type8      " },
  { 8, 330,    "SubIFDOffsets_Type8              " },
  { 8, 342,    "TransferRange_Type8              " },
  { 8, 512,    "JPEGProc_Type8                   " },
  { 8, 513,    "ThumbnailOffset_Type8            " },
  { 8, 514,    "ThumbnailLength_Type8            " },
  { 8, 529,    "YCbCrCoefficients_Type8          " },
  { 8, 530,    "YCbCrSubSampling_Type8           " },
  { 8, 531,    "YCbCrPositioning_Type8           " },
  { 8, 532,    "ReferenceBlackWhite_Type8        " },
*/

  { 9, 1,      "FileSystemVersion_Type9          " },
  { 9, 2,      "ISOUsed_Type9                    " },
  { 9, 3,      "ColorMode_Type9                  " },
  { 9, 4,      "Quality_Type9                    " },
  { 9, 5,      "WhiteBalance_Type9               " },
  { 9, 6,      "Sharpening_Type9                 " },
  { 9, 7,      "FocusMode_Type9                  " },
  { 9, 8,      "FlashSetting_Type9               " },
  { 9, 9,      "FlashType_Type9                  " },
  { 9, 10,     "Unknown_Type9                    " },
  { 9, 11,     "WhiteBalanceFine_Type9           " },
  { 9, 12,     "ColorBalance12_Type9             " },
  { 9, 13,     "ProgramShift_Type9               " },
  { 9, 14,     "ExposureDifference_Type9         " },
  { 9, 17,     "ThumbnailImageIFDOffset_Type9    " },
  { 9, 18,     "FlashCompensation_Type9          " },
  { 9, 19,     "ISOSet_Type9                     " },
  { 9, 22,     "ImageBoundary_Type9              " },
  { 9, 23,     "Unknown_Type9                    " },
  { 9, 24,     "FlashExposureBracketValue_Type9  " },
  { 9, 25,     "ExposureBracketValue_Type9       " },
  { 9, 26,     "Unknown_Type9                    " },
  { 9, 27,     "CropHiSpeed_Type9                " },
  { 9, 28,     "Unknown_Type9                    " },
  { 9, 29,     "SerialNumber29_Type9             " },
  { 9, 30,     "ColorSpace_Type9                 " },
  { 9, 31,     "VRInfoSubDirectory_Type9         " },
  { 9, 32,     "ImageAuthentication_Type9        " },

  { 9, 34,     "ActiveD-Lighting_Type9           " },
  { 9, 35,     "PictureControl_Type9             " },
  { 9, 36,     "WorldTime_Type9                  " },
  { 9, 37,     "ISOInfo_Type9                    " },
  { 9, 42,     "VignetteControl_Type9            " },

  { 9, 129,    "ToneCompensation_Type9           " },
  { 9, 131,    "LensType_Type9                   " },
  { 9, 132,    "Lens_Type9                       " },
  { 9, 135,    "FlashMode_Type9                  " },
  { 9, 136,    "AFPoint_Type9                    " },
  { 9, 137,    "ShootingMode_Type9               " },
  { 9, 138,    "AutoBracketRelease_Type9         " },
  { 9, 139,    "LensFStops_Type9                 " },
  { 9, 140,    "NEFCurve1_Type9                  " },
  { 9, 141,    "ColorHue_Type9                   " },
  { 9, 142,    "Unknown_Type9                    " },
  { 9, 144,    "LightSource_Type9                " },
  { 9, 145,    "VRInfo_Type9                     " },
  { 9, 146,    "HueAdjustment_Type9              " },
  { 9, 147,    "NEFCompression_Type9             " },
  { 9, 149,    "NoiseReduction_Type9             " },
  { 9, 150,    "NEFCurve2_Type9                  " },
  { 9, 151,    "ColorBalance151_Type9            " },
  { 9, 152,    "LensDataBytes_Type9              " },
  { 9, 153,    "NEFThumbnailSize_Type9           " },
  { 9, 154,    "SensorPixelSize_Type9            " },
  { 9, 158,    "RetouchHistory_Type9             " },
  { 9, 160,    "SerialNumber_Type9               " },
  { 9, 161,    "Unknown_Type9                    " },
  { 9, 162,    "ImageDataSize_Type9              " },
  { 9, 163,    "Unknown_Type9                    " },
  { 9, 164,    "Unknown_Type9                    " },
  { 9, 165,    "ImageCount_Type9                 " },
  { 9, 166,    "DeletedImageCount_Type9          " },
  { 9, 167,    "ShutterCount_Type9               " },
  { 9, 168,    "FlashInfo0100_Type9              " },
  { 9, 169,    "ImageOptimization_Type9          " },
  { 9, 170,    "Saturation_Type9                 " },
  { 9, 171,    "VariProgram_Type9                " },
  { 9, 175,    "Unknown_Type9                    " },
  { 9, 176,    "MultiExposureSubDirectory_Type9  " },
  { 9, 177,    "HighISONoiseReduction_Type9      " },
  { 9, 179,    "Unknown_Type9                    " },

  { 9, 182,     "Unknown_Type9                    " },
  { 9, 183,     "AFInfo2_Type9                    " },
  { 9, 184,     "FileInfo_Type9                   " },
  { 9, 185,     "Unknown_Type9                    " },
  { 9, 186,     "Unknown_Type9                    " },

  { 9, 3585,   "Unknown_Type9                    " },
  { 9, 3589,   "Unknown_Type9                    " },
  { 9, 3592,   "Unknown_Type9                    " },
  { 9, 3593,   "SoftwareNC_Type9                 " },
  { 9, 3598,   "NikonCaptureOffsets_Type9        " },
  { 9, 3600,   "Unknown_Type9                    " },
  { 9, 3609,   "Unknown_Type9                    " },
  { 9, 3612,   "Unknown_Type9                    " },
  { 9, 4097,   "RelatedImageWidth_Type9          " },
  { 9, 4098,   "RelatedImageLength_Type9         " },
  { 9, 33421u, "CFARepeatPatternDim_Type9        " },

  { 10, 1,     "InteropIndex_Type10              " },
  { 10, 2,     "InteropVersion_Type10            " },
};
#define TAGCOUNT ( sizeof(TagNames) / sizeof(TAGNAMES) )

static struct COMPRESSIONNAMES
{
   unsigned short type;
   char          *name;
}
CompressionNames[] =
{
   { 1,      "Uncompressed" },
   { 2,      "CCITT1D" },
   { 3,      "Group3Fax" },
   { 4,      "Group4Fax" },
   { 5,      "LZW" },
   { 6,      "JPEG" },
   { 32773u, "PackBits" },
};
#define COMPCOUNT ( sizeof(CompressionNames) /\
                    sizeof(COMPRESSIONNAMES) )

static struct IFDTYPENAMES
{
   short		type;
   char         *name;
}
IFDTypeNames[] =
{
   { -1,     "ALL         " },
   { 0,      "SPECIAL     " },
   { 1,      "IFD0        " },
   { 2,      "SUB_IFD0    " },
   { 3,      "SUB_IFD1    " },
   { 4,      "SUB_IFD2    " },
   { 5,      "SUB_IFD3    " },
   { 6,      "EXIF        " },
   { 7,      "GPS         " },
   { 8,      "THUMB_IFD   " },
   { 9,      "MAKER_NOTES " },
   { 10,     "INTEROP     " },
};
#define IFDTYPECOUNT ( sizeof(IFDTypeNames) /\
                    sizeof(IFDTYPENAMES) )

static struct PHOTOMETRICNAMES
{
   unsigned short type;
   char          *name;
}
PhotometricNames[] =
{
   { 0, "WhiteIsZero" },
   { 1, "BlackIsZero" },
   { 2, "RGB" },
   { 3, "RGB Palette" },
   { 4, "Transparency Mask" },
   { 5, "CMYK" },
   { 6, "YCbCr" },
   { 8, "CIELab" },
};
#define PHOTOCOUNT ( sizeof(PhotometricNames) /\
                     sizeof(PHOTOMETRICNAMES) )

static struct DATATYPENAMES
{
   unsigned short type;
   int            size;
   char          *name;
}
DataTypeNames[] =
{
   {  0,  0,  "Unspecified" },
   {  1,  1,  "8-bit unsigned" },
   {  2,  1,  "ASCIIZ string" },
   {  3,  2,  "16-bit unsigned" },
   {  4,  4,  "32-bit unsigned" },
   {  5,  8,  "32-bit unsigned rational" },
   {  6,  1,  "8-bit signed" },
   {  7,  1,  "8-bit undefined" },
   {  8,  2,  "16-bit signed" },
   {  9,  4,  "32-bit signed" },
   { 10,  8,  "32-bit signed rational" },
   { 11,  4,  "4-byte IEEE floating point" },
   { 12,  8,  "8-byte IEEE floating point" },
};
#define DATACOUNT ( sizeof(DataTypeNames) /\
                    sizeof(DATATYPENAMES) )

static struct RESOLUTIONNAMES
{
   unsigned short type;
   char          *name;
}
ResolutionNames[] =
{
   { 1, "Undefined" },
   { 2, "Inches" },
   { 3, "Centimeters" },
};
#define RESCOUNT ( sizeof(ResolutionNames) /\
                   sizeof(RESOLUTIONNAMES) )

static struct PLANARNAMES
{
   unsigned short type;
   char          *name;
}
PlanarNames[] =
{
   { 1, "Chunky" },
   { 2, "Planar" },
};
#define PLANARCOUNT ( sizeof(PlanarNames) /\
                     sizeof(PLANARNAMES) )

static char *Unknown = "Unknown";

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
struct TAGNAMES *TiffTagData( char *local_ptr  )
{
	for( int i=0; i<TAGCOUNT; i++ )
		if( !strncmp(TagNames[i].name, local_ptr, strlen(local_ptr)))
			return &TagNames[i];

	return NULL;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
struct TAGNAMES *TiffTagData( unsigned short usTag, int type  )
{
	for( int i=0; i<TAGCOUNT; i++ )
		if(( TagNames[i].tag == usTag ) && ( TagNames[i].type == type ))
			return &TagNames[i];

	return NULL;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffTagName( int tag, int type )
{
   for( int i=0; i<TAGCOUNT; i++ )
      if(( TagNames[i].tag == tag ) && ( TagNames[i].type == type ))
         return TagNames[i].name;
   return Unknown;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
unsigned short SimpleTiffTagNumber( char *local_ptr )
{
	for( int i=0; i<TAGCOUNT; i++ )
	{
		if( !strncmp(TagNames[i].name, local_ptr, strlen(local_ptr)))
		{
			if(strlen(strip(TagNames[i].name)) == strlen(local_ptr))
				return TagNames[i].tag;
		}
	}

	return (unsigned short) -1;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
unsigned short TiffTagNumber( char *local_ptr, int type )
{
	for( int i=0; i<TAGCOUNT; i++ )
	{
		if(( !strncmp(TagNames[i].name, local_ptr, strlen(local_ptr))) && ( TagNames[i].type == type ))
		{
			if(strlen(strip(TagNames[i].name)) == strlen(local_ptr))
				return TagNames[i].tag;
		}
	}

	return (unsigned short) -1;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
short TiffTypeNumber( int tag )
{
	for( int i=0; i<TAGCOUNT; i++ )
	{
		if( TagNames[i].tag == tag )
			return TagNames[i].type;
	}

	return -1;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int TiffTagCount()
{
	return TAGCOUNT;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffDataTypeName( int type )
{
   for( int i=0; i<DATACOUNT; i++ )
      if( DataTypeNames[i].type == type )
         return DataTypeNames[i].name;
   return Unknown;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int TiffDataTypeSize( int type )
{
   for( int i=0; i<DATACOUNT; i++ )
      if( DataTypeNames[i].type == type )
         return DataTypeNames[i].size;
   return 0;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffCompressionName( int type )
{
   for( int i=0; i<COMPCOUNT; i++ )
      if( CompressionNames[i].type == type )
         return CompressionNames[i].name;
   return Unknown;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffIFDTypeName( int type )
{
   for( int i=0; i<IFDTYPECOUNT; i++ )
      if( IFDTypeNames[i].type == type )
         return IFDTypeNames[i].name;
   return Unknown;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffPhotometricName( int type )
{
   for( int i=0; i<PHOTOCOUNT; i++ )
      if( PhotometricNames[i].type == type )
         return PhotometricNames[i].name;
   return Unknown;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffResolutionName( int type )
{
   for( int i=0; i<RESCOUNT; i++ )
      if( ResolutionNames[i].type == type )
         return ResolutionNames[i].name;
   return Unknown;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *TiffPlanarName( int type )
{
   for( int i=0; i<PLANARCOUNT; i++ )
      if( PlanarNames[i].type == type )
         return PlanarNames[i].name;
   return Unknown;
}

//..................struct TIFFHEADER

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void TIFFHEADER::list( void )
{
	//printf( "TIFFHEADER::order           = %04Xh ", order );
	strGlobalCOut.append("TIFFHEADER::order           =");

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, " %04Xh ", order);
	strGlobalCOut.append(glb_buffer);

	switch( order )
	{
		case IntelORDER :
			//printf( "(Intel)\n" );
			strGlobalCOut.append("(Intel)\n");
			break;
		case MotorORDER :
			//printf( "(Motorola)\n" );
			strGlobalCOut.append("(Motorola)\n");
			break;
		default :
			//printf( "(???)\n" );
			strGlobalCOut.append("(???)\n");
			break;
	}

	//printf( "TIFFHEADER::iden            = %d\n", iden );
	strGlobalCOut.append("TIFFHEADER::iden            =");

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, " %d\n", iden);
	strGlobalCOut.append(glb_buffer);

	//printf( "TIFFHEADER::ifdofs          = %ld\n", ifdofs );
	strGlobalCOut.append("TIFFHEADER::ifdofs          =");

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, " %ld\n", ifdofs);
	strGlobalCOut.append(glb_buffer);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int TIFFHEADER::get( FILE *f )
{
   if( fread(&order,sizeof(TIFFHEADER),1,f) != 1 )
      return -1;
   reverse_byte_order = (order == NativeORDER) ? 0 : 1;
   if( reverse_byte_order )
   {
      iden   = rev_word( iden );
      ifdofs = rev_dword( ifdofs );
   }
   return 0;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int TIFFHEADER::put( FILE *f )
{
   reverse_byte_order = (order == NativeORDER) ? 0 : 1;
   // reverse for writing
   if( reverse_byte_order )
   {
      iden   = rev_word( iden );
      ifdofs = rev_dword( ifdofs );
   }
   int retv = (fwrite(&order,sizeof(TIFFHEADER),1,f) == 1) ?
              0 : -1;
   // restore original order
   if( reverse_byte_order )
   {
      iden   = rev_word( iden );
      ifdofs = rev_dword( ifdofs );
   }
   return retv;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int TIFFHEADER::isvalid( void )
{
   if( iden != TIFFIDEN )
      return 0;
   if( (order != IntelORDER) && (order != MotorORDER) )
      return 0;
   return 1;
}

//..................struct IFDENTRY

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void IFDENTRY::list( void )
{
	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "IFDENTRY::tag               = %u(0x%x) ", tag, tag );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "(%s)\n", TiffTagName( tag, type ) );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "IFDENTRY::type              = %u ", type );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "(%s)\n", TiffDataTypeName( type ) );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "IFDENTRY::count             = %lu\n", count );
	strGlobalCOut.append(glb_buffer);

	int isval = (count * TiffDataTypeSize(type) <= 4) ? '*' : ' ';

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer,  "IFDENTRY::valofs            = %lu(0x%x)  %c  ", valofs, valofs, isval );
	strGlobalCOut.append(glb_buffer);

	// for a few special tags we interpret the value
	int type = int( valofs );

	switch( tag )
	{
		case 259 : // compression
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "(%s)", TiffCompressionName( type ) );
			strGlobalCOut.append(glb_buffer);
			break;
		case 262 : // image type
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "(%s)", TiffPhotometricName( type ) );
			strGlobalCOut.append(glb_buffer);
			break;
		case 284 : // planar configuration
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "(%s)", TiffPlanarName( type ) );
			strGlobalCOut.append(glb_buffer);
			break;
		case 296 : // resolution units
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "(%s)", TiffResolutionName( type ) );
			strGlobalCOut.append(glb_buffer);
			break;
		default :
			//printf( "\n" );
			break;
	}

	//printf( "\n" );
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int IFDENTRY::get( FILE *f )
{
   if( fread(&tag,sizeof(IFDENTRY),1,f) != 1 )
      return -1;
   if( reverse_byte_order )
   {
      tag    = rev_word( tag );
      type   = rev_word( type );
      count  = rev_dword( count );
      valofs = rev_dword( valofs );
   }
   return 0;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int IFDENTRY::put( FILE *f )
{
   // reverse for writing
   if( reverse_byte_order )
   {
      tag    = rev_word( tag );
      type   = rev_word( type );
      count  = rev_dword( count );
      valofs = rev_dword( valofs );
   }
   int retv = (fwrite(&tag,sizeof(IFDENTRY),1,f) == 1) ?
              0 : -1;
   // restore original order
   if( reverse_byte_order )
   {
      tag    = rev_word( tag );
      type   = rev_word( type );
      count  = rev_dword( count );
      valofs = rev_dword( valofs );
   }
   return retv;
}

//..................struct TIFFIMAGEDEF

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
TIFFIMAGEDEF::TIFFIMAGEDEF( )
{
   imgtype   = 0;     // white is zero
   cmptype   = 1;     // uncompressed
   width     = 0;
   height    = 0;
   depth     = 1;     // bilevel
   bitsamp   = 1;
   samppix   = 1;
   planar    = 1;     // chunky
   orient    = 1;     // upper left org
   hmirror   = 0;
   vmirror   = 0;
   rotate    = 0;
   resunit   = 2;     // inches
   xres      = 100;   // typical screen dpi
   yres      = 100;
   nstrips   = 0;
   striphgt  = 0;
   ncolors   = 0;
   colors    = 0;
   stripofs  = 0;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
TIFFIMAGEDEF::~TIFFIMAGEDEF( )
{
   delete colors;
   delete stripofs;
}

// file should be positioned at start of IFD

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int TIFFIMAGEDEF::get( FILE *f )
{
   // read IFD entry count
   unsigned short n;
   if( fread( &n, 2, 1, f ) != 1 ) return xIOERROR;
   if( reverse_byte_order ) n = rev_word( n );

   // read each entry
   long curofs, ltmp, ratnl[2];
   unsigned short stmp;
   int j;
   for( int i=0; i<n; i++ )
   {
      IFDENTRY x;
      if( x.get( f ) ) return xIOERROR;
      curofs = ftell( f );

      switch( x.tag )
      {
         case 256 : // ImageWidth
              width = x.uval();
              break;
         case 257 : // ImageHeight
              height = x.uval();
              break;
         case 258 : // BitsPerSample
              bitsamp = x.uval();
              break;
         case 259 : // Compression
              cmptype = x.uval();
              break;
         case 262 : // PhotometricInterpretation
              imgtype = x.uval();
              break;
         case 273 : // StripOffsets
              nstrips = x.ucnt();
              stripofs = new long [nstrips];
              if( stripofs == 0 ) return xNOMEMORY;
              // where are the offsets?
              if( x.count * TiffDataTypeSize(x.type) <= 4L )
                 fseek( f, -4L, SEEK_CUR );
              else
                 fseek( f, x.valofs, SEEK_SET );
              for( j=0; j<nstrips; j++ )
              {
                 if( (x.type == tULong) || (x.type == tSLong) )
                 {
                    fread( &ltmp, 4, 1, f );
                    stripofs[j] = ltmp;
                 }
                 else
                 {
                    fread( &stmp, 2, 1, f );
                    stripofs[j] = stmp;
                 }
              }
              // restore file position
              fseek( f, curofs, SEEK_SET );
              break;
         case 274 : // Orientation
              orient = x.uval();
              break;
         case 277 : // SamplesPerPixel
              samppix = x.uval();
              break;
         case 278 : // RowsPerStrip
              striphgt = x.uval();
              break;
         case 282 : // X Resolution
              fseek( f, x.valofs, SEEK_SET );
              fread( ratnl, 4, 2, f );
              ltmp = ratnl[0] / ratnl[1];
              xres = (unsigned short) ltmp;
              // restore file position
              fseek( f, curofs, SEEK_SET );
              break;
         case 283 : // Y Resolution
              fseek( f, x.valofs, SEEK_SET );
              fread( ratnl, 4, 2, f );
              ltmp = ratnl[0] / ratnl[1];
              yres = (unsigned short) ltmp;
              // restore file position
              fseek( f, curofs, SEEK_SET );
              break;
         case 284 : // PlanarConfiguration
              planar = x.uval();
              break;
         case 296 : // ResolutionUnit
              resunit = x.uval();
              break;
         case 320 : // ColorMap
              ncolors = x.ucnt() / 3;
              colors = new rgb [ncolors];
              if( colors == 0 ) return xNOMEMORY;
              fseek( f, x.valofs, SEEK_SET );
              for( j=0; j<ncolors; j++ )
              {
                 fread( &stmp, 2, 1, f );
                 stmp >>= 8;
                 colors[j].red = (unsigned char) stmp;
              }
              for( j=0; j<ncolors; j++ )
              {
                 fread( &stmp, 2, 1, f );
                 stmp >>= 8;
                 colors[j].grn = (unsigned char) stmp;
              }
              for( j=0; j<ncolors; j++ )
              {
                 fread( &stmp, 2, 1, f );
                 stmp >>= 8;
                 colors[j].blu = (unsigned char) stmp;
              }
              // restore file position
              fseek( f, curofs, SEEK_SET );
              break;
      }
   }

   // fill in missing values and check...

   depth = samppix * bitsamp;

   if( striphgt == 0 )
      striphgt = height;

   switch( orient )
   {
      case 2 :
           hmirror = 1;
           break;
      case 3 :
           hmirror = 1;
           vmirror = 1;
           break;
      case 4 :
           vmirror = 1;
           break;
   }

   if( imgtype > 3 )
      return xUNSUPPORTED;

   if( planar == 2 )
      return xUNSUPPORTED;

   rgb Black( 0, 0, 0 );
   rgb White( 255, 255, 255 );

   if( ncolors == 0 )
      switch( imgtype )
      {
         case 0 : // 0=white
              ncolors = 2;
              colors = new rgb [ncolors];
              if( colors == 0 ) return xNOMEMORY;
              colors[0] = White;
              colors[1] = Black;
              break;
         case 1 : // 0=black
              ncolors = 2;
              colors = new rgb [ncolors];
              if( colors == 0 ) return xNOMEMORY;
              colors[0] = Black;
              colors[1] = White;
              break;
         case 3 : // Color-mapped
              // missing palette
              return xBADPARAM;
      }

   return xOKAY;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void TIFFIMAGEDEF::list( void )
{
	int i = 0;

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::imgtype  = %u\n", imgtype );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::cmptype  = %u\n", cmptype );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::width    = %u\n", width   );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::height   = %u\n", height  );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::depth    = %u\n", depth   );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::bitsamp  = %u\n", bitsamp );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::samppix  = %u\n", samppix );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::planar   = %u\n", planar  );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::orient   = %u\n", orient  );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::hmirror  = %u\n", hmirror );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::vmirror  = %u\n", vmirror );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::rotate   = %u\n", rotate  );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::resunit  = %u\n", resunit );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::xres     = %u\n", xres    );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::yres     = %u\n", yres    );
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::striphgt = %u\n", striphgt);
	strGlobalCOut.append(glb_buffer);

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::ncolors  = %u\n", ncolors );
	strGlobalCOut.append(glb_buffer);

	for( i=0; i<ncolors; i++ )
	{
		memset(glb_buffer, '\0', GLB_SIZE);
		sprintf(glb_buffer, "              colors[%d] = rgb(%d,%d,%d)\n", i, colors[i].red, colors[i].grn, colors[i].blu );
		strGlobalCOut.append(glb_buffer);
	}

	memset(glb_buffer, '\0', GLB_SIZE);
	sprintf(glb_buffer, "TIFFIMAGEDEF::nstrips  = %d\n", nstrips );
	strGlobalCOut.append(glb_buffer);

	for( i=0; i<nstrips; i++ )
	{
		memset(glb_buffer, '\0', GLB_SIZE);
		sprintf(glb_buffer, "              stripofs[%d] = %ld\n", i, stripofs[i] );
		strGlobalCOut.append(glb_buffer);
	}
}
