/************************************************************************/
/*    $Id: d70reader.cpp,v 1.28 2006/02/18 19:00:13 joman Exp joman $   */
/*                                                                      */
/*      Program name:   d70reader.cpp                                   */
/*                                                                      */
/*      This program will output image file EXIF data to the screen.    */
/*      This program was extended from my nefexif.c program.            */
/*      This may be compiled under Linux GCC like this:  ./make_rotate  */
/*      This can also be compiled within a Visual C++ project.          */
/************************************************************************/
/*    Revisions:                                                        */
/*                                                                      */
/*    Version 0.01 - Sun Dec 12 12:11:21 EST 2004:                      */
/*        First coding to combine nefexif and jpgexif.                  */
/*    Version 0.02 - Thu Dec 23 14:13:18 EST 2004:                      */
/*        Added wildcard support.                                       */
/*    Version 0.03 - Sun Dec 26 11:19:15 EST 2004:                      */
/*        Added edits for Format 7 tags.  Added the -v and -h           */
/*        options.  Fixed a couple of Format 3 edits.                   */
/*    Version 0.04 - Mon Dec 27 16:19:25 EST 2004:                      */
/*        Added LensInfo.  Reformatted raw lens data to be              */
/*        more presentable.  Truncated decimal part of                  */
/*        focal length display.  Added to RCS revision system.          */
/*    Version 0.05 - Wed Dec 29 16:01:24 EST 2004:                      */
/*        Added -w option.  This option will create an HTML             */
/*        table of the EXIF data.  This can be then inserted            */
/*        into an HTML document.                                        */
/*    Version 0.06 - Thu Dec 30 16:20:39 EST 2004:                      */
/*        Added -x option.  This option outputs XML data.               */
/*    Version 0.07 - Sat Jan  1 12:05:28 EST 2005:                      */
/*        Fixed UserComment.  Correct misspelling of 'seconds'.         */
/*        Handled 'unknown' tag for D2H.                                */
/*    Version 0.08 - Mon Jan  3 23:03:38 EST 2005:                      */
/*        Added tag 8827 (ISO).  Modified to use right shift            */
/*        operators for selected formats that are less than             */
/*        4 bytes long.                                                 */
/*    Version 0.09 - Mon Jan 10 16:32:23 EST 2005:                      */
/*        Added -j option to extract JPG thumbnail image from           */
/*        a NEF image.  Additional code cleanup.                        */
/*    Version 0.10 - Wed Jan 12 20:49:51 EST 2005:                      */
/*        Added -c option to generate CSV data.                         */
/*    Version 0.11 - Thu Jan 13 12:03:17 EST 2005:                      */
/*        Corrected a problem with the -c option.  Forgot to            */
/*        include the image file name in the output! Also added         */
/*        double quotes around the data elements.                       */
/*    Version 0.12 - Fri Jan 21 09:34:51 EST 2005:                      */
/*        When using option -j, to extract the JPG image from a         */
/*        NEF image, write EXIF data to the new JPG file.  Also         */
/*        modified the XML output to convert the tags to                */
/*        lowercase.                                                    */
/*    Version 0.13 - Sat Jan 22 15:57:10 EST 2005:                      */
/*        Fixed a problem with wildcard processing, where the           */
/*        program would stop on first error, instead of going           */
/*        on to the next file in the list.  Also fixed, was the         */
/*        JPEG marker scan logic.  Finally, changed the 'ifdef'         */
/*        logic for the FreeBSD system compile.                         */
/*    Version 0.14 - Fri Jan 28 12:17:09 EST 2005:                      */
/*        Added logic to rotate the JPG image when the -j option        */
/*        is used.  The source now uses the IJG JPEG library.           */
/*        A configuration file called 'd70reader.conf' can be           */
/*        used to select the tags that will be displayed.               */
/*    Version 0.15 - Mon Feb 14 22:52:53 EST 2005:                      */
/*        Changed the default location for the config file.             */
/*        The default is to create the file in the user space,          */
/*        not the system space. (Not in /etc, or c:\windows)            */
/*    Version 0.16 - Sat Feb 19 14:01:53 EST 2005:                      */
/*        Added 'assert' logic to prevent bad data processing.          */
/*        Fixed a buffer overflow exception with format #1.             */
/*        Cleaned out dead code.  Better control of iSystemType.        */
/*        Only process MakerNotes for Nikon cameras.  Calculate         */
/*        shutter speed, instead of if/else logic.  Added 'G-VR'        */
/*        and 'D-VR' lens types.                                        */
/*    Version 0.17 - Mon Feb 21 15:47:19 EST 2005:                      */
/*        JPG extraction now works for the D100 and D2H.  Added         */
/*        WhiteBalance and Sharpening tags.  Fixed (hopefully)          */
/*        the last buffer overflow problem.                             */
/*    Version 0.18 - Fri Mar  4 10:53:34 EST 2005:                      */
/*        Added option -js, to generate JavaScript arrays of the        */
/*        EXIF data.  Added place holders for the missing tags          */
/*        for the -c option.  Added the Adobe XMP SDK logic to          */
/*        capture more EXIF data for Adobe modified files.  Fixed       */
/*        a problem with the lens display, where the decimal            */
/*        part was being truncated.  Allow the *.* wildcard,            */
/*        ignoring non-image files.  Added support for the D2X          */
/*        camera.                                                       */
/*    Version 0.19 - Mon Mar 21 17:38:33 EST 2005:                      */
/*        Added ImageWidth and ImageLength tags.  These are             */
/*        embedded in a NEF file.  Added a space before the 'mm'        */
/*        output for focal length and lens.  Truncated FNumber to       */
/*        two decimal places.  Added the -web option.  This             */
/*        allows the generation of image galleries that can be          */
/*        copied as is to a web site for display.                       */
/*    Version 0.20 - Wed Mar 23 22:54:24 EST 2005:                      */
/*        Corrected a problem, where the program was not                */
/*        rotating a JPG image with the -web option.                    */
/*    Version 0.21 - Thu Apr 14 13:17:57 EDT 2005:                      */
/*        Added the -rm option to allow complete removal of the         */
/*        EXIF data from a JPG image.  Fixed a problem with the         */
/*        -web option, where vertical images were not always            */
/*        displayed properly.  Handle a situation where the image       */
/*        file does not contain the orientation tag 0x112.              */
/*        Fixed a problem where the -j option would fail with           */
/*        certain Nikon Capture created files.  When using the          */
/*        -j option, the EXIF data is now correctly formatted           */
/*        into the proper buckets, instead of having it all at          */
/*        the lowest level.  This means that other applications         */
/*        will now correctly read it.  The original -j logic            */
/*        from version 0.20 is kept as -jold.  There are still          */
/*        some image processing software created images that            */
/*        can not be handled correctly with the newer -j option.        */
/*        The -jold option does work with some of these images.         */
/*    Version 0.22 - Tue May 10 14:47:38 EDT 2005:                      */
/*        Converted program to C++, added additional EXIF tags.         */
/*        and made changes to -web option.  Also added -cf              */
/*        option.  Added d70readerweb.conf file, to allow changes to    */
/*        text displayed on the generated web pages.  Made display of   */
/*        EXIF data on web pages optional, defaulting to none.  Added   */
/*        LensID, LensDataVersion, and FocusDistance tags.              */
/*    Version 0.23 - Fri May 27 13:06:32 EDT 2005:                      */
/*        Added the -od option, to allow the user to input a            */
/*        destination directory, for file output.  Coded the            */
/*        decryption method to allow support for the LensID and         */
/*        FocusDistance for the D2X, D50, and D2Hs.  Added the -re      */
/*        option.  This option will copy NEF/Raw EXIF data to a target  */
/*        JPG file.  This means that a JPG file that has been post-     */
/*        processed by an image software package, that normally drops   */
/*        the EXIF data, can have that data restored. Fixed a bug,      */
/*        where the HTML gallery index label was not using the          */
/*        d70readerweb.conf file for its display values.                */
/*    Version 0.24 - Sat Jun  4 19:47:41 EDT 2005:                      */
/*        Added the -fd option.  This will allow the internal image     */
/*        dates to be modified.  This makes use of the                  */
/*        d70readerdate.conf file, to set the three possible dates.     */
/*    Version 0.25 - Sat Jun 18 19:24:36 EDT 2005:                      */
/*        Added main option -njf.  This will create a JPEG FINE image   */
/*        file from a NEF image file.  Also added -cwb (use camera	*/
/*        whitebalance), -awb (use auto whitebalance), -crgb (use       */
/*        camera RGB), -noclip (don't clip colors), -br:n.n (set image  */
/*        brightness), -red:n.n (set red multiplier), and -blue:n.n     */
/*        (set blue multiplier).                                        */
/*    Version 0.26 - Sat Jul  9 12:56:00 EDT 2005:                      */
/*        Added support for the Nikon D50.  Added support for the       */
/*        Mac OS X 10.4 operating system. Added NikonCaptureOffsets     */
/*        tag.  Added ExposureDifference tag.  Updated the version of   */
/*        DCRAW to use, to fully support the D2X.  Fixed a problem with */
/*        the -re option, where not passing all parameters would cause  */
/*        the program to crash.  Fixed a problem where Nikon Capture    */
/*        4.3 modified files would cause the D70Reader program to       */
/*        crash.  Added additional Start Of Frame markers to the logic  */
/*        used to determine a JPG images size.  The previous version    */
/*        only checked on the standard one.  Some image programs write  */
/*        the image using higher numbered SOF markers.  Found a problem */
/*        with some images that did not properly handle the offset to   */
/*        the next marker.  Changed the code to allow for this problem. */
/*        Renamed the ColorBalance1 tag to ColorBalance12, to reflect   */
/*        its tag number.  Renamed the ColorBalanceD70 to               */
/*        ColorBalance151, using the tag number, and making the name    */
/*        generic, so that it could be used with non-D70 cameras.       */
/*        Added the SenserPixelSize tag.  Fixed the Adobe XMP code,     */
/*        where some improperly created images, would cause the         */
/*        D70Reader to crash, because the XMP data was trash.           */
/*    Version 0.27 - Sat Aug 13 12:40:13 EDT 2005:                      */
/*        Increased display length of hex dumped tags to a max of 40    */
/*        bytes, instead of 20 bytes.  Upgraded to the latest version   */
/*        of DCRAW, that fully supports the D50.  Corrected LensID      */
/*        display for the D100.  Renamed the second occurance of the    */
/*        CFAPattern tag to CFAPatternEXIF.  Added the AFAperture,      */
/*        FocusPosition, MinFocalLength, MaxFocalLength,                */
/*        MaxApertureAtMinFocal, MaxApertureAtMaxFocal, MCUVersion,     */
/*        and EffectiveMaxAperture tags.  These are derived from the    */
/*        LensDataBytes hex values.                                     */
/*    Version 0.28 - Mon Sep 12 16:58:44 EDT 2005:                      */
/*        Changed the DCRAW code to version 7.60.  This version has     */
/*        a new demosaicing algorithm called Adaptive homogeneity-      */
/*        directed.  This replaces the old Variable Number of Gradients */
/*        Method.  The older algorithm produces clearly visible color   */
/*        artifacts.  Also added the -black option to control the black */
/*        point of the image.                                           */
/*    Version 0.29 - Sun Dec 25 19:48:57 EST 2005:                      */
/*        Added support for 94 more EXIF tags.  To support additional   */
/*        EXIF tags, added suffix to tag names to allow for uniqueness. */
/*        Modified -web option to support new tag name structure.  Also */
/*        modified -web option to dynamically look up tag names.  All   */
/*        files created be the program go into sub-directories under    */
/*        the 'd70reader' directory (fix_date, jpeg_fine, no_exif,      */
/*        restore_exif, thumbnail, and web).  Added option -s to sort   */
/*        the standard output.  Fixed flag that impacted -js and -web   */
/*        options.  The program now handles some non-Nikkor lenses.     */
/*        Default configuration file renamed to 'd70reader29.conf'.     */
/*        Fixed a bug with the -re option that resulted in failure of   */
/*        this option (don't know how long this has been there!).       */
/*        Added check to handle invalid EXIF directory structure.       */
/*        Added logic to handle GPS tags (I was not able to test this). */
/*        Reduced amount of verbose data displayed with the -v option.  */
/*        Added EXIF data type indicator to the standard output.  Fixed */
/*        a bug with the -c, -js, and -web options when using a subset  */
/*        of all possible tags.  Finally, added support for the D200.   */
/*        Fixed bug with the -j, -re, and -njf options where the JPG    */
/*        file created, was invalid, if the source NEF file was a Nikon */
/*        Capture modified file.  For -web option, JPG image files      */
/*        were not being properly rotated, even though NEF files were.  */
/*    Version 0.30 - Thu Jan 12 18:02:39 EST 2006:                      */
/*        Added -coc (Circle of Confusion) option.  Added DOF (Depth    */
/*        of Field) tags.                                               */
/*    Version 0.31 - Thu Jan 26 10:23:05 EST 2006:                      */
/*        Added GPS map link to -web option.  Added -fdwo option.  This */
/*        option allows an image timestamp to be modified by an offset  */
/*        of minutes.  A negative value can be entered to subtract      */
/*        time.                                                         */
/*    Version 0.32 - Fri Feb  3 09:52:51 EST 2006:                      */
/*        Fixed a problem created in version 0.31, with the -web option.*/
/*        The previous changes made it necesary to include the GPS tags */
/*        in the data, even if you had no need for them.  The effect    */
/*        was a JavaScript error in the generated HTML page.            */
/*    Version 0.33 - Sat Feb 18 13:56:59 EST 2006:                      */
/*        Added the -ren option to rename image file based on shutter   */
/*        count tag, and user supplied prefix.                          */
/*    Version 0.34 - Mon Jan 15 12:45:09 EST 2007:                      */
/*        Added support for the Nikon D40, and D80.  Updated the logic  */
/*        to use the new XMP library.  Fixed -ren bug.  Added           */
/*        ProgramShift, CropHiSpeed, ColorSpace, AutoBracketRelease,    */
/*        and HighISONoiseReduction tags.  Added additional lenses to   */
/*        the lens database.                                            */
/*    Version 0.35 - Thu Jul 24 17:02:46 EDT 2008:                      */
/*        Changed MeteringMode descriptive text.  Improved AFPoint      */
/*        reporting.  Added additional FlashMode items.  Added          */
/*        processing for MultiExposure and AFInfo2 tags.  Added support */
/*        for D60, D300, D700, and D3 cameras.  Updated web site        */
/*        address.  Added logic for new tags.  Added support for new    */
/*        lenses.                                                       */
/*                                                                      */
/************************************************************************/

#ifdef WIN_GUI
#include <direct.h>
#include <stdlib.h>
#include <stdio.h>

#include "stdafx.h"
#include "ProgressBar.h"
#include "d70wantedlist.h"
#include "FileTreeCtrl.h"
#endif

#include <sys/stat.h>

#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <list>
#include <vector>
#include <time.h>
#include <math.h>

#ifdef WIN32
#include <sys/utime.h>
#else
#include <utime.h>
#endif

#include "D70Exception.h"
#include "d70config.h"
#include "d70revbyte.hpp"
#include "d70tif.hpp"
#include "d70reader.h"
#include "d70claifd.hpp"
#include "d70inc1.h"
#include "d70inc2.h"
#include "fmountlens.h"

using namespace std;

//..................program globals

const char * const d70Version = "0.35";
const char * const d70VersionDate = "Thu Jul 24 18:26:02 UTC 2008";
static char rcsid[] = "$Id: d70reader.cpp,v 1.28 2006/02/18 19:00:13 joman Exp joman $";

typedef list<IFDElement> LISTIFDElEMENTS;
LISTIFDElEMENTS listIFDElements;
LISTIFDElEMENTS::iterator itrIFDElements;

typedef list<IFDElement> LISTIFDElEMENTS2;
LISTIFDElEMENTS2 listIFDElements2;
LISTIFDElEMENTS2::iterator itrIFDElements2;

typedef list<WantedTags> LISTWANTEDTAGS;
LISTWANTEDTAGS listWantedTags;
LISTWANTEDTAGS::iterator itrWantedTags;

typedef list<WebConfig> LISTWEBCONFIG;
LISTWEBCONFIG listWebConfig;
LISTWEBCONFIG::iterator itrWebConfig;

string& d70_main( int argc, char *argv[], char* env[] );
void handleEnvironment( int argc, char *argv[], char* env[] );
int handleArguments( int argc, char *argv[], char* env[] );
void outputJSCout(bool bFirstTime, int iIndex);
string& xmp_display(char *filename);
void eraseStringBuffer();
string& instruct2();
bool isRestoreEXIFTargetFileValid(char *filename);

struct WantedFlags
{
	unsigned short tag;
	short type;
} strWantedFlags;

#ifndef WIN_GUI
//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
int main( int argc, char *argv[], char* env[] )
{
	if(argc < 2)
	{
		cout << instruct();
		cout.flush();
		exit(1);
	}

	try
	{
		cout << d70_main( argc, argv, env);
		cout.flush();
	}
	catch(CD70Exception *pExc)
	{
		cout << pExc->GetD70ExceptionTxt();
		cout.flush();
		exit(1);
	}
	catch(CD70ArgException *pExc)
	{
		cout << pExc->GetD70ArgExceptionTxt();
		cout << instruct2();
		cout.flush();
		exit(1);
	}
	catch(CD70HelpException *pExc)
	{
		cout << pExc->GetD70HelpExceptionTxt();
		cout << instruct();
		cout.flush();
		exit(1);
	}
}
#endif

#ifdef WIN_GUI
CTreeFileCtrl *pClassPtr;

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
void setClassPtr(CTreeFileCtrl *pPtr)
{
	pClassPtr = pPtr;
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
BOOL PeekAndPump()
{
	static MSG msg;

	while (::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE)) {
		if (!AfxGetApp()->PumpMessage()) {
			::PostQuitMessage(0);
			return FALSE;
		}	
	}

	return TRUE;
}
#endif

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& d70_main( int argc, char *argv[], char* env[] )
{
	int ctr1 = 0;
	int ctr2 = 0;
	struct stat buf;
	char *filename = NULL;
	bool bDemoVersion = false;

	//strGlobalCOut.append("\nd70reader.cpp\n");

#ifndef WIN_GUI
	handleEnvironment( argc, argv, env );
#else
	iSystemType = 1;
#endif

	ctr1 = handleArguments( argc, argv, env );

#ifdef WIN_GUI
	bSortPlain = true;
#endif

	if(strConfigFile.length() == 0)
		strConfigFile.append("d70reader29.conf");

	if(bHtml || bXml || bThumbnail || bRestoreEXIF || bCsv || bJS || bWeb || bRemoveEXIF || bFixDate || bRenameImage || bFixDateWithOffset || bJpegFine)
		bOutputPlain = false;

#ifndef WIN_GUI
	checkForConfig();
	checkForWebConfig();
	checkForDateConfig();
#endif

	for(ctr2 = 0; ctr2 < ctr1; ctr2++)
		argv++;

	if(bRestoreEXIF)
	{
		if(bOutputDirectory)
		{
			if(argc != 5)
			{
				ThrowD70ArgException(string("\nIncorrect format for Output Directory Option"));
			}
		}
		else
		{
			if(argc != 4)
			{
				ThrowD70ArgException(string("\nIncorrect format for Restore EXIF Option"));
			}
		}

		filename = *++argv;

		if(stat(filename, &buf) == (-1))
		{
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\n%s", filename);
			ThrowD70ArgException(string(glb_buffer));
			//perror(filename);
		}

		lFileSize = buf.st_size;

		if(lFileSize == 0l)
		{
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\nZero size file: %s", filename);
			ThrowD70ArgException(string(glb_buffer));
		}

		string strSrcFileName;

		strSrcFileName = string(filename);

		filename = *++argv;

		if(stat(filename, &buf) == (-1))
		{
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\n%s", filename);
			ThrowD70ArgException(string(glb_buffer));
			//perror(filename);
		}

		lFileSize = buf.st_size;

		if(lFileSize == 0l)
		{
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\nZero size file: %s", filename);
			ThrowD70ArgException(string(glb_buffer));
		}

		strTargetEXIFFileName = string(filename);

		if(!isRestoreEXIFTargetFileValid((char *) filename))
			ThrowD70Exception(string("\nTarget File is not a JPG, or has an invalid format: ") +
							  string(filename) +
							  string("\n"));

		cout << ProcessFile((char *) strSrcFileName.c_str());
		cout.flush();

		//if(bVerbos)
			strGlobalCOut.erase();
	}
	else
	{
		while(filename = *++argv)
		{
			if(stat(filename, &buf) == (-1))
			{
				memset(glb_buffer, '\0', GLB_SIZE);
				sprintf(glb_buffer, "\n%s", filename);
				ThrowD70ArgException(string(glb_buffer));
				//perror(filename);
			}
			else
			{
				bool bFound = isDirectoryFound(string(filename));

				if(bFound)
				{
#ifdef WIN_GUI
					if(bJS)
						js_total_count--;
#endif

					continue;
				}

				lFileSize = buf.st_size;

				if(lFileSize == 0l)
				{
					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "\nZero size file: %s", filename);

#ifdef WIN_GUI
					if(bJS)
						js_total_count--;
#endif
					continue;

					//ThrowD70ArgException(string(glb_buffer));
				}

				cout << ProcessFile(filename);
				cout.flush();

				//if(bVerbos)
					strGlobalCOut.erase();
			}
		}
	}

	if(bJS)
	{
#ifdef WIN_GUI
		char buffer[256];

		if(js_total_count == js_ctr1)
		{
			sprintf(buffer, "var intExifDataArrayCount=%d\n", (js_ctr1 - 1));
			cout << buffer;
			bJSFlagFirstTime = true;
		}
#else
		//printf("var intExifDataArrayCount=%d\n", (js_ctr1 - 1));
		strGlobalCOut.append("var intExifDataArrayCount=");

		memset(glb_buffer, '\0', GLB_SIZE);
		sprintf(glb_buffer, "%d", (js_ctr1 - 1));
		strGlobalCOut.append(glb_buffer);

		strGlobalCOut.append("\n");
#endif
	}

	if(bWeb)
	{
		if(gallery_ofd != NULL)
		{
			fprintf(gallery_ofd, "var intExifDataArrayCount=%d\n", (iWeb_Image_count - 1));
			fclose(gallery_ofd);

			fprintf(root_ofd, "var intExifDataArrayCount=%d\n", (iRoot_Web_Image_count - 1));
			fclose(root_ofd);

#ifndef WIN_GUI
			if(str_root_temp_dir != NULL)
			{
				my_rmdir(str_root_temp_dir);
				free(str_root_temp_dir);
			}
#endif
		}

#ifdef WIN_GUI
		iRoot_Web_Image_count = 0;
		iWeb_Image_count = 0;
		iWeb_First_Time = 1;
		iWeb_Gallery_count = 0;
		bJSFirstTime = true;

		if(bNewlyCreatedD70)
			pClassPtr->InsertNewDirectory(CString(strWebPathName.c_str()), CString("d70readerw"));

		if(bNewlyCreatedWeb)
			pClassPtr->InsertNewDirectory(CString(strWebPathName.c_str()) + CString("d70readerw\\"), CString("web"));
#endif
	}

#ifndef WIN_GUI
	if(file_sys_dir_ptr != NULL)
		free(file_sys_dir_ptr);

	listWantedTags.clear();
	listWebConfig.clear();
#endif

	return strGlobalCOut;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void handleEnvironment( int argc, char *argv[], char* env[] )
{
	int ctr3 = 0;
	char *temp_ptr = NULL;

	do
	{
		temp_ptr = (char *) malloc(strlen(env[ctr3]) + 1);
		memset(temp_ptr, (int) '\0', strlen(env[ctr3]) + 1);
		memcpy(temp_ptr, env[ctr3], strlen(env[ctr3]));
		temp_ptr = upperCase(temp_ptr);

		if(bTrace)
		{
			//cout << "\n" << temp_ptr  << '\n';
			strGlobalCOut.append("\n");
			strGlobalCOut.append(temp_ptr);
			strGlobalCOut.append("\n");
		}

		if(strlen(temp_ptr) >= 5)
		{
			if(!memcmp(temp_ptr, "PATH=", 5))
			{
				if(memchr(temp_ptr,'\\', 20))
					iSystemType = 1;
				else
					iSystemType = 0;
			}
		}

		if(file_sys_dir_ptr == NULL)
		{
			if(strlen(temp_ptr) >= 12)
			{
				// if found, this should be windows:
				if(!memcmp(temp_ptr, "USERPROFILE=", 12))
				{
					file_sys_dir_ptr = (char *) malloc(strlen(temp_ptr) + 1);
					memset(file_sys_dir_ptr, (int) '\0', strlen(temp_ptr) + 1);
					memcpy(file_sys_dir_ptr, &env[ctr3][12], strlen(temp_ptr) - 12);
	
					if(bTrace)
					{
						//cout << "\nfile_sys_dir_ptr: " << file_sys_dir_ptr << '\n';
						strGlobalCOut.append("\nfile_sys_dir_ptr: ");
						strGlobalCOut.append(file_sys_dir_ptr);
						strGlobalCOut.append("\n");
					}
				}
			}

			if(strlen(temp_ptr) >= 5)
			{
				// if found, this should be linux/unix:
				if(!memcmp(temp_ptr, "HOME=", 5))
				{
					file_sys_dir_ptr = (char *) malloc(strlen(temp_ptr) + 1);
					memset(file_sys_dir_ptr, (int) '\0', strlen(temp_ptr) + 1);
					memcpy(file_sys_dir_ptr, &env[ctr3][5], strlen(temp_ptr) - 5);
	
					if(bTrace)
					{
						//cout << "\nfile_sys_dir_ptr: " << file_sys_dir_ptr << '\n';
						strGlobalCOut.append("\nfile_sys_dir_ptr: ");
						strGlobalCOut.append(file_sys_dir_ptr);
						strGlobalCOut.append("\n");
					}
				}
			}

			if(strlen(temp_ptr) >= 10)
			{
				// if found, this should be OS/2:
				if(!memcmp(temp_ptr, "OS2_SHELL=", 10))
				{
					file_sys_dir_ptr = (char *) malloc(strlen(temp_ptr) + 10);
					memset(file_sys_dir_ptr, (int) '\0', strlen(temp_ptr) + 10);
					memcpy(file_sys_dir_ptr, &env[ctr3][10], 2);
					memcpy(&file_sys_dir_ptr[2], "\\OS2\\APPS", 9);
	
					if(bTrace)
					{
						//cout << "\nfile_sys_dir_ptr: " << file_sys_dir_ptr << '\n';
						strGlobalCOut.append("\nfile_sys_dir_ptr: ");
						strGlobalCOut.append(file_sys_dir_ptr);
						strGlobalCOut.append("\n");
					}
				}
			}
		}

		ctr3++;
	}
	while(env[ctr3] != NULL);

	// Suggestion by BRN, in case nothing is set
	if(file_sys_dir_ptr == NULL)
	{
		file_sys_dir_ptr = (char *) malloc(5);
		memset(file_sys_dir_ptr, (int) '\0', 5);
		//put the config file in the current directory
		strcpy (file_sys_dir_ptr, ".");
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int handleArguments( int argc, char *argv[], char* env[] )
{
	int ctr1 = 0;
	int ctr2 = 0;
	int argn = 0;
	char *arg = NULL;

	for(argn = 1; argn < argc; argn++)
	{
		arg = argv[argn];

		if(arg[0] != '-') 
			break; // Filename(s) have started.

		if(!strcmp(arg,"-v"))
		{
			bVerbos = true;
		}
		else if(!strncmp(arg,"-coc", 3))
		{
			bNewCoC = true;

			if(arg[4] != ':') 
			{
				ThrowD70ArgException(string("\nIncorrect format for Circle of Confusion Option"));
			}

			strCicleOfConfusion.append(&arg[5]);

			double circleOfConfusionDiameter = atof(strCicleOfConfusion.c_str());

			if(!circleOfConfusionDiameter)
			{
    			ThrowD70ArgException(string("\nIncorrect format for CoC Option"));
			}

		}
		else if(!strncmp(arg,"-cf", 3))
		{
			bNewConfig = true;

			if(arg[3] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Configuration File Option"));
			}

			strConfigFile.append(&arg[4]);
		}
		else if(!strncmp(arg,"-od", 3))
		{
			bOutputDirectory = true;

			if(arg[3] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Output Directory Option"));
			}

			strOutputDirectory.append(&arg[4]);

			int iSeperatorOffset = 0;
			int iOutputDirLength = 0;

			if(iSystemType)
			{
				iSeperatorOffset = strOutputDirectory.find_last_of("\\", strOutputDirectory.length());
				iOutputDirLength = strOutputDirectory.length() - 1;

				if(iSeperatorOffset < iOutputDirLength)
				{
        			ThrowD70ArgException(string("\nIncorrect format for Output Directory Option"));
				}
			}
			else
			{
				iSeperatorOffset = strOutputDirectory.find_last_of("/", strOutputDirectory.length());
				iOutputDirLength = strOutputDirectory.length() - 1;

				if(iSeperatorOffset < iOutputDirLength)
				{
        			ThrowD70ArgException(string("\nIncorrect format for Output Directory Option"));
				}
			}
		}
		else if(!strcmp(arg,"-njf"))
		{
			bJpegFine = true;
		}
		else if(!strcmp(arg,"-s"))
		{
			bSortPlain = true;
		}
		else if(!strncmp(arg,"-ren", 4))
		{
			bRenameImage = true;

			if(arg[4] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Rename Image Option"));
			}

			strRenamePrefix.append(&arg[5]);
		}
		else if(!strcmp(arg,"-fd"))
		{
			bFixDate = true;
		}
		else if(!strncmp(arg,"-fdwo", 5))
		{
			bFixDateWithOffset = true;

			if(arg[5] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for File Date With Offset Option"));
			}

			string strTest;
			strTest.append(&arg[6]);

			if(strTest.length() == 0)
			{
    			ThrowD70ArgException(string("\nIncorrect format for File Date With Offset Option"));
			}

			iFixDateOffset = (int) atoi(&arg[6]);
		}

		else if(!strcmp(arg,"-noclip"))
		{
			bMainDoNotClipColor = true;
		}
		else if(!strcmp(arg,"-cwb"))
		{
			bMainUseCameraWB = true;
		}
		else if(!strcmp(arg,"-awb"))
		{
			bMainUseAutoWB = true;
		}
		else if(!strcmp(arg,"-crgb"))
		{
			bMainUseCameraRGB = true;
		}
		else if(!strncmp(arg,"-br", 3))
		{
			bMainBright = true;

			if(arg[3] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Image Brightness Option"));
			}

			string strTest;
			strTest.append(&arg[4]);

			if(strTest.length() == 0)
			{
    			ThrowD70ArgException(string("\nIncorrect format for Image Brightness Option"));
			}

			fMainBright = (float) atof(&arg[4]);
		}
		else if(!strncmp(arg,"-red", 4))
		{
			bMainRed = true;

			if(arg[4] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Red Multiplier Option"));
			}

			string strTest;
			strTest.append(&arg[5]);

			if(strTest.length() == 0)
			{
    			ThrowD70ArgException(string("\nIncorrect format for Red Multiplier Option"));
			}

			fMainRed = (float) atof(&arg[5]);
		}
		else if(!strncmp(arg,"-blue", 5))
		{
			bMainBlue = true;

			if(arg[5] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Blue Multiplier Option"));
			}

			string strTest;
			strTest.append(&arg[6]);

			if(strTest.length() == 0)
			{
    			ThrowD70ArgException(string("\nIncorrect format for Blue Multiplier Option"));
			}

			fMainBlue = (float) atof(&arg[6]);
		}

		else if(!strncmp(arg,"-black", 6))
		{
			bMainBlack = true;

			if(arg[6] != ':') 
			{
    			ThrowD70ArgException(string("\nIncorrect format for Black Point Option"));
			}

			string strTest;
			strTest.append(&arg[7]);

			if(strTest.length() == 0)
			{
    			ThrowD70ArgException(string("\nIncorrect format for Black Point Option"));
			}

			iMainBlack = (int) atoi(&arg[7]);
		}

		else if(!strcmp(arg,"-w"))
		{
			bHtml = true;
		}
		else if(!strcmp(arg,"-c"))
		{
			bCsv = true;
		}
		else if(!strcmp(arg,"-re"))
		{
			bRestoreEXIF = true;
		}
		else if(!strcmp(arg,"-web"))
		{
			bWeb = true;
		}
		else if(!strcmp(arg,"-js"))
		{
			bJS = true;
		}
		else if(!strcmp(arg,"-x"))
		{
			bXml = true;
		}
		else if(!strcmp(arg,"-j"))
		{
			bThumbnail = true;
		}
		else if(!strcmp(arg,"-rm"))
		{
			bRemoveEXIF = true;
		}
		else if(!strcmp(arg,"-t"))
		{
			bTrace = true;
		}
		else if(!strcmp(arg,"-h"))
		{
   			ThrowD70HelpException(string(""));
		}
		else
		{
   			ThrowD70HelpException(string(""));
		}

		ctr1++;
	}

	if(argc == argn)
	{
		ThrowD70Exception(string("\nInternal Error in: handleArguments()"));
	}

	return ctr1;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& instruct()
{
    //cout << string("\nD70Reader - Version ") << d70Version << string(" - ") << d70VersionDate; 
	strGlobalCOut.append("\nD70Reader - Version ");
	strGlobalCOut.append(d70Version);
	strGlobalCOut.append(" - ");
	strGlobalCOut.append(d70VersionDate);

    //cout << string("\nCopyright (C) 2007, by Jonathan D. Oman. All Rights Reserved.\n");
	strGlobalCOut.append("\nCopyright (C) 2008, by Jonathan D. Oman. All Rights Reserved.\n");

	//cout << string("\nThis software makes use of \"The Independent JPEG Group's JPEG Software\"");
	strGlobalCOut.append("\nThis software makes use of \"The Independent JPEG Group's JPEG Software\"");

	//cout << string("\nSpecifically, it uses their image rotation and resizing logic.\n");
	strGlobalCOut.append("\nSpecifically, it uses their image rotation and resizing logic.\n");


#ifndef APPLE
	//cout << string("\nThis software makes use of \"The Adobe XMP SDK Software\"");
	strGlobalCOut.append("\nThis software makes use of \"The Adobe XMP SDK Software\"");

	//cout << string("\nPlease read their copyright information, found in the ");
	strGlobalCOut.append("\nPlease read their copyright information, found in the ");

	//cout << string("\n(ZIP/gz file)./docs/LICENSE.TXT file.\n");
	strGlobalCOut.append("\n(ZIP/gz file)./docs/LICENSE.TXT file.\n");
#endif	// APPLE

	//cout << string("\nThis software makes use of \"Dave Coffin's raw photo decoder (dcraw)\"\n");
	strGlobalCOut.append("\nThis software makes use of \"Dave Coffin's raw photo decoder (dcraw)\"\n");

    //cout << string("\nThis program only works with Nikon D50/D70/D80/D100/D200/D2H/D2X image files.");
	strGlobalCOut.append("\nThis program only works with the following Nikon cameras:");
	strGlobalCOut.append("\nD40[x]/D50/D60/D70/D80/D100/D200/D300/D2H[s]/D2X[s]/D700/D3");
    //cout << string("\n(It will work with NEF/RAW and JPG files.)\n");
	strGlobalCOut.append("\n(It will work with NEF/RAW and JPG files.)\n");

    //cout << string("\nCheck here for latest version:");
	strGlobalCOut.append("\nCheck here for latest version:");
    //cout << string("\nhttp://www.gogebic-pc.com/d70/index.html\n");
	strGlobalCOut.append("\nhttp://gogebic-pc.home.comcast.net/~gogebic-pc/d70/\n");

    //cout << string("\nFormat:");
	strGlobalCOut.append("\nFormat:");
    //cout << string("\n\td70reader [options] imageFileName");
	strGlobalCOut.append("\n\td70reader [options] imageFileName");
    //cout << string("\n\td70reader [options] *.NEF *.JPG\n");
	strGlobalCOut.append("\n\td70reader [options] *.NEF *.JPG\n");
    //cout << string("\n\tOptions:\n");
	strGlobalCOut.append("\n\tOptions:\n");
    //cout << string("\n\t-h                  = this screen");
	strGlobalCOut.append("\n\t-h                  = this screen");
    //cout << string("\n\t-s                  = sort the standard output");
	strGlobalCOut.append("\n\t-s                  = sort the standard output");
    //cout << string("\n\t-v                  = verbose (raw data)");
	strGlobalCOut.append("\n\t-v                  = verbose (raw data)");
    //cout << string("\n\t-w                  = generate HTML table");
	strGlobalCOut.append("\n\t-w                  = generate HTML table");
    //cout << string("\n\t-x                  = generate XML data tree");
	strGlobalCOut.append("\n\t-x                  = generate XML data tree");
    //cout << string("\n\t-c                  = generate Comma Separated Values (CSV) data");
	strGlobalCOut.append("\n\t-c                  = generate Comma Separated Values (CSV) data");
    //cout << string("\n\t-js                 = generate JavaScript Array data");
	strGlobalCOut.append("\n\t-js                 = generate JavaScript Array data");
    //cout << string("\n\t-j                  = extract JPG thumbnail image from NEF");
	strGlobalCOut.append("\n\t-j                  = extract JPG thumbnail image from NEF");
    //cout << string("\n\t-coc:n.n            = set Circle Of Confusion (default = 0.0203)");
	strGlobalCOut.append("\n\t-coc:n.n            = set Circle Of Confusion (default = 0.0203)");
    //cout << string("\n\t-njf                = create JPG FINE image from NEF");
	strGlobalCOut.append("\n\t-njf                = create JPG FINE image from NEF");
    //cout << string("\n\t-cwb                = use Camera WhiteBalance (with -njf)");
	strGlobalCOut.append("\n\t-cwb                = use Camera WhiteBalance (with -njf)");
    //cout << string("\n\t-awb                = use Auto WhiteBalance (with -njf)");
	strGlobalCOut.append("\n\t-awb                = use Auto WhiteBalance (with -njf)");
    //cout << string("\n\t-crgb               = use Camera RGB (with -njf)");
	strGlobalCOut.append("\n\t-crgb               = use Camera RGB (with -njf)");
    //cout << string("\n\t-noclip             = don't Clip Colors (with -njf)");
	strGlobalCOut.append("\n\t-noclip             = don't Clip Colors (with -njf)");
    //cout << string("\n\t-br:n.n             = set Image Brightness (default = 1.0) (with -njf)");
	strGlobalCOut.append("\n\t-br:n.n             = set Image Brightness (default = 1.0) (with -njf)");
    //cout << string("\n\t-red:n.n            = set Red Multiplier (default = 1.0) (with -njf)");
	strGlobalCOut.append("\n\t-red:n.n            = set Red Multiplier (default = 1.0) (with -njf)");
    //cout << string("\n\t-blue:n.n           = set Blue Multiplier (default = 1.0) (with -njf)");
	strGlobalCOut.append("\n\t-blue:n.n           = set Blue Multiplier (default = 1.0) (with -njf)");
    //cout << string("\n\t-black:n            = set Black Point (greater than 0) (with -njf)");
	strGlobalCOut.append("\n\t-black:n            = set Black Point (greater than 0) (with -njf)");
    //cout << string("\n\t-fd                 = fix the internal dates of an image file");
	strGlobalCOut.append("\n\t-fd                 = fix the internal dates of an image file");
    //cout << string("\n\t-fdwo:n             = fix the internal dates with minute offset");
	strGlobalCOut.append("\n\t-fdwo:n             = fix the internal dates with minute offset");
    //cout << string("\n\t-rm                 = remove all EXIF data from JPG file");
	strGlobalCOut.append("\n\t-rm                 = remove all EXIF data from JPG file");
    //cout << string("\n\t-re nefFile jpgFile = restore EXIF data to target JPG file");
	strGlobalCOut.append("\n\t-re nefFile jpgFile = restore EXIF data to target JPG file");
    //cout << string("\n\t-ren:prefix         = rename image file based on Shutter Count");
	strGlobalCOut.append("\n\t-ren:prefix         = rename image file based on Shutter Count");
    //cout << string("\n\t-web                = generate Photo Website");
	strGlobalCOut.append("\n\t-web                = generate Photo Website");
    //cout << string("\n\t-cf:configFile      = use alternate configuration file");
	strGlobalCOut.append("\n\t-cf:configFile      = use alternate configuration file");
#ifdef WIN32
    //cout << string("\n\t-od:\\outputDir\\     = use specified output directory\n\n");
	strGlobalCOut.append("\n\t-od:\\outputDir\\     = use specified output directory\n\n");
#else
    //cout << string("\n\t-od:/outputDir/     = use specified output directory\n\n");
	strGlobalCOut.append("\n\t-od:/outputDir/     = use specified output directory\n\n");
#endif

	return strGlobalCOut;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& instruct2()
{
	strGlobalCOut.append("\n\nFormat:");
    strGlobalCOut.append("\n\td70reader [options] imageFileName");
    strGlobalCOut.append("\n\td70reader [options] *.NEF *.JPG\n");
    strGlobalCOut.append("\n\tOptions:\n");
    strGlobalCOut.append("\n\t-h                  = this screen");
    strGlobalCOut.append("\n\t-s                  = sort the standard output");
    strGlobalCOut.append("\n\t-v                  = verbose (raw data)");
    strGlobalCOut.append("\n\t-w                  = generate HTML table");
    strGlobalCOut.append("\n\t-x                  = generate XML data tree");
    strGlobalCOut.append("\n\t-c                  = generate Comma Separated Values (CSV) data");
    strGlobalCOut.append("\n\t-js                 = generate JavaScript Array data");
    strGlobalCOut.append("\n\t-j                  = extract JPG thumbnail image from NEF");
    strGlobalCOut.append("\n\t-coc:n.n            = set Circle Of Confusion (default = 0.0203)");
    strGlobalCOut.append("\n\t-njf                = create JPG FINE image from NEF");
    strGlobalCOut.append("\n\t-cwb                = use Camera WhiteBalance (with -njf)");
    strGlobalCOut.append("\n\t-awb                = use Auto WhiteBalance (with -njf)");
    strGlobalCOut.append("\n\t-crgb               = use Camera RGB (with -njf)");
    strGlobalCOut.append("\n\t-noclip             = don't Clip Colors (with -njf)");
    strGlobalCOut.append("\n\t-br:n.n             = set Image Brightness (default = 1.0) (with -njf)");
    strGlobalCOut.append("\n\t-red:n.n            = set Red Multiplier (default = 1.0) (with -njf)");
    strGlobalCOut.append("\n\t-blue:n.n           = set Blue Multiplier (default = 1.0) (with -njf)");
    strGlobalCOut.append("\n\t-black:n            = set Black Point (greater than 0) (with -njf)");
    strGlobalCOut.append("\n\t-fd                 = fix the internal dates of an image file");
    strGlobalCOut.append("\n\t-fdwo:n             = fix the internal dates with minute offset");
    strGlobalCOut.append("\n\t-rm                 = remove all EXIF data from JPG file");
    strGlobalCOut.append("\n\t-re nefFile jpgFile = restore EXIF data to target JPG file");
    strGlobalCOut.append("\n\t-ren:prefix         = rename image file based on Shutter Count");
    strGlobalCOut.append("\n\t-web                = generate Photo Website");
    strGlobalCOut.append("\n\t-cf:configFile      = use alternate configuration file");
#ifdef WIN32
    strGlobalCOut.append("\n\t-od:\\outputDir\\     = use specified output directory\n\n");
#else
    strGlobalCOut.append("\n\t-od:/outputDir/     = use specified output directory\n\n");
#endif

	return strGlobalCOut;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int imageType()
{
	return iImageType;
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
int fileTypeOffset()
{
	int iType = 0;

	switch(iImageType)
	{
		case 0:
			iType = 30;
			break;
		case 1:
			iType = 12;
			break;
		case 99:
			iType = 0;
			break;
		default:
			iType = -1;
			break;
	}

	return(iType);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
int dumpImageFileDirectory(FILE *rfd, long offset, long additional_offset, const string& strType, int numType)
{
	char buffer[32];
	long local_offset = ftell(rfd);

	while( offset )
	{
		if(bTrace)
		{
			if(!listIFDElements.empty())
			{
				//cout << '\n' << string("Size of List                = ") << listIFDElements.size() << '\n';
				strGlobalCOut.append("\n");
				strGlobalCOut.append("Size of List                = ");
				memset(glb_buffer, (int) '\0', GLB_SIZE);
				sprintf(glb_buffer, "%d", listIFDElements.size());
				strGlobalCOut.append(glb_buffer);
				strGlobalCOut.append("\n");

				//cout << "\nEntering dumpImageFileDirectory: " << strType;
				strGlobalCOut.append("\nEntering dumpImageFileDirectory: ");
				strGlobalCOut.append(strType);

				for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
				{
					itrIFDElements->list();
				}
			}
		}

		// seek to start
		fseek( rfd, offset, SEEK_SET );
		long cur_offset = offset;

		// get IFD entry count
		unsigned short nentries;
		fread( &nentries, 2, 1, rfd );

		if( ReverseByteOrder() )
			nentries = rev_word( nentries );

		if(nentries > 100)
		{
			if(bVerbos)
			{
				//cout << "\nThe following file has a problem with this data:\n";
				strGlobalCOut.append("\nThe following file has a problem with this data:\n");

				//cout << "\noffset: " << offset;
				strGlobalCOut.append("\noffset: ");
				memset(glb_buffer, (int) '\0', GLB_SIZE);
				sprintf(glb_buffer, "%ld", offset);
				strGlobalCOut.append(glb_buffer);

				//cout << "\nadditional_offset: " << additional_offset;
				strGlobalCOut.append("\nadditional_offset: ");
				memset(glb_buffer, (int) '\0', GLB_SIZE);
				sprintf(glb_buffer, "%ld", additional_offset);
				strGlobalCOut.append(glb_buffer);

				//cout << "\nstrType: " << strType;
				strGlobalCOut.append("\nstrType: ");
				strGlobalCOut.append(strType);

				//cout << "\nnumType: " << numType;
				strGlobalCOut.append("\nnumType: ");
				memset(glb_buffer, (int) '\0', GLB_SIZE);
				sprintf(glb_buffer, "%d", numType);
				strGlobalCOut.append(glb_buffer);

				//cout << '\n';
				strGlobalCOut.append("\n");
				cout.flush();
			}

			return(-1);
		}

		if(bTrace)
		{
			// print subfile header
			//cout << '\n' << "IFD at Offset " << offset << " has " << nentries << " Entries.....\n";
			strGlobalCOut.append("\n");
			strGlobalCOut.append("IFD at Offset ");

			memset(glb_buffer, (int) '\0', GLB_SIZE);
			sprintf(glb_buffer, "%ld", offset);
			strGlobalCOut.append(glb_buffer);
			
			strGlobalCOut.append(" has ");
			
			memset(glb_buffer, (int) '\0', GLB_SIZE);
			sprintf(glb_buffer, "%d", nentries);
			strGlobalCOut.append(glb_buffer);
			
			strGlobalCOut.append(" Entries.....\n");
		}

		// list each IFD entrie
		for( int i = 0; i < nentries; i++ )
		{
			IFDENTRY ifde;
			ifde.get( rfd );

			if(bTrace)
			{
				//cout << "\n";
				strGlobalCOut.append("\n");
				ifde.list();
			}

			if(ifde.tag == 0x927C)
			{
				ulMakerNoteHeaderOffset = ifde.offsetval() + ulMakerNotePreHeaderLength + fileTypeOffset();
				ulCalcMakerNoteHeaderOffset = ulMakerNoteHeaderOffset;
			}

			IFDElement ifdelement = IFDElement(ifde, numType);
			ifdelement.setFileTypeOffset(fileTypeOffset());
			ifdelement.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
			ifdelement.setFileType(iFileType);
			ifdelement.setASCILabelSkipLength(iASCILabelSkipLength);
			ifdelement.setValueWriteOffset(ifde.offsetval());
			ifdelement.setData(rfd);

			if(bTrace)
			{
				if(ifdelement.getTag() != 0x19)
				{
					//cout << "\nExposureBracketValue (0x19) Raw      : " << getDataValueRaw(0x19, 9, true);
					strGlobalCOut.append("\nExposureBracketValue (0x19) Raw      : ");
					strGlobalCOut.append(getDataValueRaw(0x19, 9, true));

					//cout << "\nExposureBracketValue (0x19) Converted: " << getDataValue(0x19, 9, true);
					strGlobalCOut.append("\nExposureBracketValue (0x19) Converted: ");
					strGlobalCOut.append(getDataValue(0x19, 9, true));
				}

				//cout << "\nJust set data, just before list insert: " << strType;
				strGlobalCOut.append("\nJust set data, just before list insert: ");
				strGlobalCOut.append(strType);

				ifdelement.list();
			}

			listIFDElements.insert(listIFDElements.end(), ifdelement);

			if(ifde.tag == 0x0011)	//ThumbnailImageIFDOffset
			{
				string strModel = string(upperCase((char *) getDataValue(0x110, 1, true).c_str()));

				int old_reverse_byte_order = reverse_byte_order;
				long tmp_offset = ftell(rfd);

				if(bTrace)
					//cout << "\n================Starting ThumbnailImageIFDOffset================" << '\n';
					strGlobalCOut.append("\n================Starting ThumbnailImageIFDOffset================\n");

				const string& strType = string("THUMB_IFD");
				dumpImageFileDirectory(rfd, ifde.valofs + ulMakerNoteHeaderOffset, ulMakerNoteHeaderOffset, strType, itThumbIFD);

				if(bTrace)
				{
					//cout << "\n=================Ending ThumbnailImageIFDOffset=================" << '\n';
					strGlobalCOut.append("\n=================Ending ThumbnailImageIFDOffset=================\n");
					//cout << '\n' << string("Resuming: ") + strType << '\n';
					strGlobalCOut.append("\nResuming: ");
					strGlobalCOut.append(strType);
					strGlobalCOut.append("\n");
				}

				fseek(rfd, tmp_offset, SEEK_SET);
				reverse_byte_order = old_reverse_byte_order;

				if((!strModel.compare(0, 10, "NIKON D100")) || (!strModel.compare(0, 9, "NIKON D2H")))
				{
					lThumbnailOffset = atol(getDataValue(0x201, 8, true).c_str());
					lThumbnailLength = atol(getDataValue(0x202, 8, true).c_str());
				}
			}


			if(ifde.tag == 0x8825)	// GPSInfo
			{
				string strMake = getDataValue(0x10f, 1, true);

				if(!strMake.compare(0, 17, "NIKON CORPORATION"))
				{
					int old_reverse_byte_order = reverse_byte_order;
					long tmp_offset = ftell(rfd);

					fseek(rfd, ulMakerNoteHeaderOffset, SEEK_SET);

					if(bTrace)
						//cout << "\n================Starting GPS================" << '\n';
						strGlobalCOut.append("\n================Starting GPS================\n");

					unsigned long ulOffset = inspectImageFileHeader(rfd, 1);

					if(ulOffset != -1l)
					{
						const string& strType = string("GPS");
						int iType = fileTypeOffset();
						dumpImageFileDirectory(rfd, ifde.valofs + iType, 0, strType, itGPS);
					}

					if(bTrace)
					{
						//cout << "\n=================Ending GPS=================" << '\n';
						strGlobalCOut.append("\n=================Ending GPS=================\n");
						//cout << '\n' << string("Resuming: ") + strType << '\n';
						strGlobalCOut.append("\nResuming: ");
						strGlobalCOut.append(strType);
						strGlobalCOut.append("\n");
					}

					fseek(rfd, tmp_offset, SEEK_SET);
					reverse_byte_order = old_reverse_byte_order;
				}
			}

			if(ifde.tag == 0xA005)	// INTEROP
			{
				string strMake = getDataValue(0x10f, 1, true);

				if(!strMake.compare(0, 17, "NIKON CORPORATION"))
				{
					int old_reverse_byte_order = reverse_byte_order;
					long tmp_offset = ftell(rfd);

					fseek(rfd, ulMakerNoteHeaderOffset, SEEK_SET);

					if(bTrace)
						//cout << "\n================Starting INTEROP================" << '\n';
						strGlobalCOut.append("\n================Starting INTEROP================\n");

					unsigned long ulOffset = inspectImageFileHeader(rfd, 1);

					if(ulOffset != -1l)
					{
						const string& strType = string("INTEROP");
						int iType = fileTypeOffset();
						dumpImageFileDirectory(rfd, ifde.valofs + iType, 0, strType, itInterop);
					}

					if(bTrace)
					{
						//cout << "\n=================Ending INTEROP=================" << '\n';
						strGlobalCOut.append("\n=================Ending INTEROP=================\n");
						//cout << '\n' << string("Resuming: ") + strType << '\n';
						strGlobalCOut.append("\nResuming: ");
						strGlobalCOut.append(strType);
						strGlobalCOut.append("\n");
					}

					fseek(rfd, tmp_offset, SEEK_SET);
					reverse_byte_order = old_reverse_byte_order;
				}
			}

			if(ifde.tag == 0x014A)
			{
				long tmp_offset = ftell(rfd);
				long lByte = 0l;
				char buffer[32];

				if((TiffDataTypeSize(ifde.type) * ifde.count) > 4)
				{

					fseek(rfd, ifde.valofs, SEEK_SET);

					for( unsigned long j = 0; j < ifde.count; j++)
					{
						memset(buffer, (int) '\0', 32);
						sprintf(buffer, "%d", j);

						fread(&lByte, sizeof(long), 1, rfd);

						if( ReverseByteOrder() )
							lByte = rev_dword( lByte );

						if(bTrace)
						{
							//cout << '\n' << string("================Starting SUB_IFD") + string(buffer) << "================" << '\n';
							strGlobalCOut.append("\n================Starting SUB_IFD");
							strGlobalCOut.append(buffer);
							strGlobalCOut.append("================\n");
						}

						const string& strType = string("SUB_IFD")  + string(buffer);
						dumpImageFileDirectory(rfd, lByte, 0, strType, j + 2);

						if(bTrace)
						{
							//cout << '\n' << string("================Ending SUB_IFD") + string(buffer) << "================" << '\n';
							strGlobalCOut.append("\n================Ending SUB_IFD");
							strGlobalCOut.append(buffer);
							strGlobalCOut.append("================\n");
						}
					}

				}
				else
				{
					if(bTrace)
						//cout << '\n' << string("================Starting SUB_IFD0") << "================" << '\n';
						strGlobalCOut.append("\n================Starting SUB_IFD0================\n");

					const string& strType = string("SUB_IFD0");
					dumpImageFileDirectory(rfd, ifde.valofs, 0, strType, 2);

					if(bTrace)
						//cout << '\n' << string("================Ending SUB_IFD0") << "=================" << '\n';
						strGlobalCOut.append("\n================Ending SUB_IFD0================\n");
				}

				if(bTrace)
				{
					//cout << '\n' << string("Resuming: ") + strType << '\n';
					strGlobalCOut.append("\nResuming: ");
					strGlobalCOut.append(strType);
					strGlobalCOut.append("\n");
				}

				fseek(rfd, tmp_offset, SEEK_SET);
			}

			if(ifde.tag == 0x927C)
			{
				string strMake = getDataValue(0x10f, 1, true);

				if(!strMake.compare(0, 17, "NIKON CORPORATION"))
				{
					string strModel = string(upperCase((char *) getDataValue(0x110, 1, true).c_str()));

					int old_reverse_byte_order = reverse_byte_order;
					long tmp_offset = ftell(rfd);

					fseek(rfd, ulMakerNoteHeaderOffset, SEEK_SET);

					if(bTrace)
						//cout << "\n================Starting MAKER_NOTES================" << '\n';
						strGlobalCOut.append("\n================Starting MAKER_NOTES================\n");

					unsigned long ulOffset = inspectImageFileHeader(rfd, 1);

					if(ulOffset != -1l)
					{
						const string& strType = string("MAKER_NOTES");
						dumpImageFileDirectory(rfd, (ulMakerNoteHeaderOffset + ulOffset), 0, strType, itMakerNotes);
					}

					if(bTrace)
					{
						//cout << "\n=================Ending MAKER_NOTES=================" << '\n';
						strGlobalCOut.append("\n=================Ending MAKER_NOTES=================\n");
						//cout << '\n' << string("Resuming: ") + strType << '\n';
						strGlobalCOut.append("\nResuming: ");
						strGlobalCOut.append(strType);
						strGlobalCOut.append("\n");
					}

					fseek(rfd, tmp_offset, SEEK_SET);
					reverse_byte_order = old_reverse_byte_order;

					if((strModel.compare(0, 10, "NIKON D100")) && (strModel.compare(0, 9, "NIKON D2H")))
					{
						lThumbnailOffset = atol(getDataValue(0x201, 2, true).c_str());
						lThumbnailLength = atol(getDataValue(0x202, 2, true).c_str());
					}
				}
			}

			if(ifde.tag == 0x8769)
			{
				long tmp_offset = ftell(rfd);

				if(bTrace)
					//cout << "\n================Starting EXIF================" << '\n';
					strGlobalCOut.append("\n================Starting EXIF================\n");

				const string& strType = string("EXIF");
				dumpImageFileDirectory(rfd, ifde.valofs + fileTypeOffset(), 0, strType, itEXIF);

				if(bTrace)
				{
					//cout << "\n=================Ending EXIF=================" << '\n';
					strGlobalCOut.append("\n=================Ending EXIF=================\n");
					//cout << '\n' << string("Resuming: ") + strType << '\n';
					strGlobalCOut.append("\nResuming: ");
					strGlobalCOut.append(strType);
					strGlobalCOut.append("\n");
				}

				fseek(rfd, tmp_offset, SEEK_SET);
			}
		}	// end of for loop

/*	// this code breaks NC 4.3, and does not appear to be needed?
		// get offset to next subfile
		fread( &offset, 4, 1, rfd );	// JON JON JON

		if( ReverseByteOrder() )
			offset = rev_dword( offset );

		if(!iFileType)	//JPG file does not have two null bytes to indicate end of IFD's
*/
			offset = 0;

	}	// end of while

	if(bTrace)
	{
		//cout << '\n' << string("Size of List                = ") << listIFDElements.size() << '\n';
		strGlobalCOut.append("\nSize of List                = ");
		memset(glb_buffer, (int) '\0', GLB_SIZE);
		sprintf(glb_buffer, "%d", listIFDElements.size());
		strGlobalCOut.append(glb_buffer);
		strGlobalCOut.append("\n");

		//cout << "\nExiting dumpImageFileDirectory: " << strType;
		strGlobalCOut.append("\nExiting dumpImageFileDirectory: ");
		strGlobalCOut.append(strType);

		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			itrIFDElements->list();
		}
	}

	fseek(rfd, local_offset, SEEK_SET);
	return 0;
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
bool isTagWanted(unsigned short usTag, int iType)
{
	bool isFound = false;

	for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
	{
/*
		if(usTag == 0 && iType == 7)
		{
			//cout << "\nTag: " << itrWantedTags->getTag();
			strGlobalCOut.append("\nTag: ");
			strGlobalCOut.append(itrWantedTags->getTag());

			//cout << "\nType: " << itrWantedTags->getIFDType();
			strGlobalCOut.append("\nType: ");
			strGlobalCOut.append(itrWantedTags->getIFDType());
		}
*/

		if((itrWantedTags->getTag() == usTag) && (itrWantedTags->getIFDType() == iType))
			return itrWantedTags->isWanted();
	}

	return isFound;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
bool checkForExifData(FILE *rfd)
{
	int ctr1 = 0;
	bool bFoundEXIF = false;

	long position = 0l;
	long old_position = 0l;

	struct marker
	{
		unsigned char pad;
		unsigned char typeMarker;
		unsigned short length;
		char test_data[6];
	};

	struct marker *temp_ptr;

	old_position = ftell(rfd);

	temp_ptr = (struct marker *) malloc(sizeof(struct marker));
	memset(temp_ptr, (int) '\0', sizeof(struct marker));

	fseek(rfd, 2, 0);

	do
	{
		ctr1++;
		position = ftell(rfd);

		memset(temp_ptr, (int) '\0', sizeof(struct marker));
		fread(temp_ptr, sizeof(struct marker), 1, rfd);

		if(!strncmp(temp_ptr->test_data, "Exif\0\0", 6))
		{
			bFoundEXIF = true;
			break;
		}
		else
#ifdef APPLE
			fseek(rfd, temp_ptr->length - sizeof(struct marker) + 2, 1);
#else
			fseek(rfd, Get16u(&temp_ptr->length) - sizeof(struct marker) + 2, 1);
#endif	// APPLE

		if(temp_ptr->typeMarker == 0xda)
			break;
	}
	while(strncmp(temp_ptr->test_data, "Exif\0\0", 6));	

	fseek(rfd, old_position, 0);

	free(temp_ptr);

	return(bFoundEXIF);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
unsigned long getJpgTypeAndOffset(FILE *rfd, unsigned long offset)
{
	char buffer[256];
	static char ExifHeader[] = "Exif\0\0";
	unsigned long offset_ifd = 0l;
	struct jpg_image_file_header *jpg_ifh_ptr = NULL;

	jpg_ifh_ptr = (struct jpg_image_file_header *) malloc(sizeof(struct jpg_image_file_header));
	memset(jpg_ifh_ptr, (int) '\0', sizeof(struct jpg_image_file_header));

	if((fread(jpg_ifh_ptr, sizeof(struct jpg_image_file_header), 1, rfd)) != 0)
	{
        if (strncmp(jpg_ifh_ptr->exif_header_text, ExifHeader, 6))
		{
			ThrowD70Exception(string("\nEXIF header text incorrect\nFile is not a JPG/JPEG file\n"));
		}
			
		memset(buffer, (int) '\0', 256);
		memcpy(buffer, jpg_ifh_ptr->byte_order, 2);

		if(!strncmp(buffer, strNativeORDER, 2))
			reverse_byte_order = 0;
		else
			reverse_byte_order = 1;
		
		offset_ifd = Get32u(&jpg_ifh_ptr->offset_to_first_ifd) + offset;
				
		if((!strncmp(buffer, "MM", 2)) && (Get16u(&jpg_ifh_ptr->image_file_type) == 42))
		{
		}
		else if((!strncmp(buffer, "II", 2)) && (Get16u(&jpg_ifh_ptr->image_file_type) == 42))
		{
		}
		else
		{
			ThrowD70Exception(string("\nFile type is not Nikon JPG\n\n"));
		}
				
	}
	else
	{
		my_error(rfd);
	}

	free(jpg_ifh_ptr);

	return(offset_ifd);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
unsigned long inspectImageFileHeader(FILE *rfd, int iType)
{
	char buffer[256];
	static unsigned char ExifHeader[] = "Exif\0\0";
	struct image_file_header *ifh_ptr = NULL;
	unsigned long offset_ifd = 0;
	unsigned long ulRetCode = (unsigned long) -1l;

	struct jpg_indicator_header
	{
		unsigned char start;
		unsigned char soi;
		unsigned char pad;
		unsigned char exif_ind;
	};

	struct jpg_indicator_header *jpg_ind_hdr_ptr = NULL;

	if(iType)
	{
		ifh_ptr = (struct image_file_header *) malloc(sizeof(struct image_file_header));
		memset(ifh_ptr, (int) '\0', sizeof(struct image_file_header));

		if((fread(ifh_ptr, sizeof(struct image_file_header), 1, rfd)) != 0)
		{
			memset(buffer, (int) '\0', 256);
			memcpy(buffer, ifh_ptr->byte_order, 2);

			if(!strncmp(buffer, strNativeORDER, 2))
				reverse_byte_order = 0;
			else
				reverse_byte_order = 1;
			
			offset_ifd = Get32u(&ifh_ptr->offset_to_first_ifd);
	
			if((!strncmp(buffer, "MM", 2)) && (Get16u(&ifh_ptr->image_file_type) == 42))
			{
			}
			else if((!strncmp(buffer, "II", 2)) && (Get16u(&ifh_ptr->image_file_type) == 42))
			{
			}
			else
			{
				if(bVerbos)
					//cout << "\nFile type is not NEF/Nikon RAW\n\n";
					strGlobalCOut.append("\nFile type is not NEF/Nikon RAW\n\n");

				free(ifh_ptr);
				return ulRetCode;
			}
	
		}
		else
		{
			free(ifh_ptr);
			return ulRetCode;
		}

		free(ifh_ptr);
	}
	else
	{
		jpg_ind_hdr_ptr = (struct jpg_indicator_header *) malloc(sizeof(struct jpg_indicator_header));
		memset(jpg_ind_hdr_ptr, (int) '\0', sizeof(struct jpg_indicator_header));

		if((fread(jpg_ind_hdr_ptr, sizeof(struct jpg_indicator_header), 1, rfd)) != 0)
		{
			if(jpg_ind_hdr_ptr->start != 0xff)
			{
				free(jpg_ind_hdr_ptr);
				return ulRetCode;
			}

			if(jpg_ind_hdr_ptr->soi != 0xd8)
			{
				free(jpg_ind_hdr_ptr);
				return ulRetCode;
			}

			if(jpg_ind_hdr_ptr->pad != 0xff)
			{
				free(jpg_ind_hdr_ptr);
				return ulRetCode;
			}

			fseek(rfd, 0, 0);

			switch(jpg_ind_hdr_ptr->exif_ind)
			{
				case 0xe0:
					iImageType = 0;
					fseek(rfd, 18, 0);
					offset_ifd = getJpgTypeAndOffset(rfd, fileTypeOffset());
					break;
				case 0xe1:
					iImageType = 1;
					offset_ifd = getJpgTypeAndOffset(rfd, fileTypeOffset());
					break;
				default:
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "Fourth byte is unknown: %x", jpg_ind_hdr_ptr->exif_ind);
					return ulRetCode;
					break;
			}
		}

		free(jpg_ind_hdr_ptr);
	}

	return(offset_ifd);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
bool checkForJPGHeader(FILE *rfd)
{
	bool bFoundJPGHeader = true;

	long position = 0l;
	long old_position = 0l;

	struct marker
	{
		unsigned char byte1;
		unsigned char byte2;
	};

	struct marker *temp_ptr;

	old_position = ftell(rfd);

	temp_ptr = (struct marker *) malloc(sizeof(struct marker));
	memset(temp_ptr, (int) '\0', sizeof(struct marker));

	fseek(rfd, 0, SEEK_SET);

	fread(temp_ptr, sizeof(struct marker), 1, rfd);

	if((temp_ptr->byte1 != 0xff) || (temp_ptr->byte2 != 0xd8))
		bFoundJPGHeader = false;

	fseek(rfd, old_position, 0);

	free(temp_ptr);

	return(bFoundJPGHeader);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
double nikonApertureConversions(double dValue)
{
	double dNumber = 0.0;

	dNumber = pow((double) 2.0, ((double) dValue / (double) 24.0));

	return(dNumber);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
double nikonFocalConversions(double dValue)
{
	double dNumber = 0.0;

	dNumber = 5 * pow((double) 2.0, ((double) dValue / (double) 24.0));

	return(dNumber);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
string& ProcessFile(char *filename)
{
	struct xmp_packet *xmp_packet_ptr = NULL;

	char *file_name_ptr = NULL;

#ifdef WIN_GUI
	CProgressBar bar(_T(filename), 40, 500, TRUE);
	//CProgressBar bar(_T(filename));

	for (int i = 0; i < 500; i++) {
		bar.StepIt();
		PeekAndPump();
	}
#endif

	if((!bCsv) && (!bJS))
	{
		//cout << "\nProcessing File: " << filename; // << '\n';
		strGlobalCOut.append("\nProcessing File: ");
		strGlobalCOut.append(filename);
	}

	// open the file
	FILE *rfd = fopen( filename, "rb" );

	if( rfd == 0 )
	{
		if((!bHtml) && (!bXml) && (!bCsv) && (!bJS))
		{
			//cout << "\nFile not readable: " << filename << "\n\n";
			strGlobalCOut.append("\nFile not readable: ");
			strGlobalCOut.append(filename);
			strGlobalCOut.append("\n\n");
		}

#ifdef WIN_GUI
		if(bJS)
			js_total_count--;
#endif

		return strGlobalCOut;
	}

	dFocusDistance = 0.0;

	// read header and validate
	TIFFHEADER tifh;
	tifh.get( rfd );

	if( !tifh.isvalid() )
	{
		if(bRestoreEXIF)
		{
			//cout << "\nThe source file must be a NEF file when using option -re\n";
			strGlobalCOut.append("\nThe source file must be a NEF file when using option -re\n");
			fclose(rfd);
			return strGlobalCOut;
		}

		if(bThumbnail)
		{
			//cout << "\nOnly files of type NEF can be processed using option -j\n";
			strGlobalCOut.append("\nOnly files of type NEF can be processed using option -j\n");
			fclose(rfd);
			return strGlobalCOut;
		}

		if(bJpegFine)
		{
			//cout << "\nOnly files of type NEF can be processed using option -njf\n";
			strGlobalCOut.append("\nOnly files of type NEF can be processed using option -njf\n");
			fclose(rfd);
			return strGlobalCOut;
		}

		if(!checkForJPGHeader(rfd))
		{
			if((!bHtml) && (!bXml) && (!bCsv) && (!bJS))
			{
				//cout << "\nFile not readable: " << filename << "\n\n";
				strGlobalCOut.append("\nFile not readable: ");
				strGlobalCOut.append(filename);
				strGlobalCOut.append("\n\n");
			}

#ifdef WIN_GUI
			if(bJS)
				js_total_count--;
#endif

			fclose(rfd);
			return strGlobalCOut;
		}

		iFileType = 0;
		iImageType = 0;

		if(!checkForExifData(rfd))
		{
			if((!bCsv) && (!bJS))
			{
				//cout << '\n' << string("No EXIF data found: ") << filename << '\n';
				strGlobalCOut.append("\nNo EXIF data found: ");
				strGlobalCOut.append(filename);
				strGlobalCOut.append("\n");
			}

#ifdef WIN_GUI
			if(bJS)
				js_total_count--;
#endif

			fclose(rfd);
			return strGlobalCOut;
		}
	}
	else
	{
		iFileType = 1;
		iImageType = 99;
	}

	if((bHtml) || (bXml) || (bCsv) || (bJS) || (bWeb))
	{
		if(bWeb)
		{
#ifdef WIN_GUI
			string strDrive = getDrive(filename);
			string strFilePath;
			string strFileName;

			strFilePath.append(getFilePath(filename));
			strFileName.append(getFileName(filename));

			strWebPathName.resize(0);
			strWebPathName.append(getFilePath(filename));

			_chdir((char *) strFilePath.c_str());

			char szDrive[2];
			memset(szDrive, (int) '\0', 2);
			sprintf(szDrive, "%s", strDrive.c_str());
			_chdrive( szDrive[0] - 'A' + 1 );

			if(iFileType)
			{
				if(bOutputDirectory)
				{
					web_file_name_src_ptr = strdup(strFileName.c_str());
					strFileName.append(".JPG");
					web_file_name_tmp_ptr = strdup(strFileName.c_str());

					if(strFilePath.length() != 0)
						strWebSrcPathname = strFilePath;
					else
						strWebSrcPathname = string(".");
				}
				else
				{
					web_file_name_src_ptr = strdup((char *) strFileName.c_str());

					file_name_ptr = (char *) malloc((strlen(filename) + strlen(".JPG") + 1));
					memset(file_name_ptr, (int) '\0', (strlen(filename) + strlen(".JPG") + 1));
					sprintf(file_name_ptr, "%s.JPG", web_file_name_src_ptr);

					web_file_name_tmp_ptr = strdup(file_name_ptr);

					if(iSystemType)
						strWebSrcPathname = string(".\\");
					else
						strWebSrcPathname = string("./");
				}
			}
			else
			{
				if(bOutputDirectory)
				{
					web_file_name_tmp_ptr = strdup(strFileName.c_str());

					if(strFilePath.length() != 0)
						strWebSrcPathname = strFilePath;
					else
						strWebSrcPathname = string(".");
				}
				else
				{
					web_file_name_tmp_ptr = strdup(strFileName.c_str());

					if(iSystemType)
						strWebSrcPathname = string(".\\");
					else
						strWebSrcPathname = string("./");
				}
			}
#else
			string strFilePath;
			string strFileName;

			if(bOutputDirectory)
			{
				strFilePath.append(getFilePath(filename));
				strFileName.append(getFileName(filename));
			}

			if(iFileType)
			{
				if(bOutputDirectory)
				{
					web_file_name_src_ptr = strdup(strFileName.c_str());
					strFileName.append(".JPG");
					web_file_name_tmp_ptr = strdup(strFileName.c_str());

					if(strFilePath.length() != 0)
						strWebSrcPathname = strFilePath;
					else
						strWebSrcPathname = string(".");
				}
				else
				{
					web_file_name_src_ptr = strdup(filename);

					file_name_ptr = (char *) malloc((strlen(filename) + strlen(".JPG") + 1));
					memset(file_name_ptr, (int) '\0', (strlen(filename) + strlen(".JPG") + 1));
					sprintf(file_name_ptr, "%s.JPG", filename);

					web_file_name_tmp_ptr = strdup(file_name_ptr);

					if(iSystemType)
						strWebSrcPathname = string(".\\");
					else
						strWebSrcPathname = string("./");
				}
			}
			else
			{
				if(bOutputDirectory)
				{
					web_file_name_tmp_ptr = strdup(strFileName.c_str());

					if(strFilePath.length() != 0)
						strWebSrcPathname = strFilePath;
					else
						strWebSrcPathname = string(".");
				}
				else
				{
					web_file_name_tmp_ptr = strdup(filename);

					if(iSystemType)
						strWebSrcPathname = string(".\\");
					else
						strWebSrcPathname = string("./");
				}
			}
#endif
		}

		if(file_name_ptr)
			free(file_name_ptr);
	}

	if(bTrace)
	{
		//cout << "Listing of image file: " << filename << '\n';
		strGlobalCOut.append("Listing of image file: ");
		strGlobalCOut.append(filename);
		strGlobalCOut.append("\n");

		if(iFileType)
			tifh.list();
	}

	IFDENTRY ifde(0xFFFE, 2, 0, 0);

	IFDElement ifdelement(ifde, itNone);
	ifdelement.setProgramName(string("d70reader"));
	ifdelement.setProgramVersion(string(d70Version));
	ifdelement.setProgramDate(string(d70VersionDate));
	ifdelement.setFileTypeOffset(fileTypeOffset());
	ifdelement.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement.setFileType(iFileType);
	ifdelement.setASCILabelSkipLength(iASCILabelSkipLength);

	listIFDElements.insert(listIFDElements.end(), ifdelement);

	IFDENTRY ifde2(0xFFFD, 2, 0, 0);

	IFDElement ifdelement2(ifde2, itNone);
	ifdelement2.setFileName(string(filename));
	ifdelement2.setFileTypeOffset(fileTypeOffset());
	ifdelement2.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement2.setFileType(iFileType);
	ifdelement2.setASCILabelSkipLength(iASCILabelSkipLength);

	listIFDElements.insert(listIFDElements.end(), ifdelement2);

	IFDENTRY ifde3(0xFFFC, 2, 0, 0);

	IFDElement ifdelement3(ifde3, itNone);
	ifdelement3.setFileName(string(filename));
	ifdelement3.setFileSize(lFileSize);
	ifdelement3.setFileTypeOffset(fileTypeOffset());
	ifdelement3.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement3.setFileType(iFileType);
	ifdelement3.setASCILabelSkipLength(iASCILabelSkipLength);

	listIFDElements.insert(listIFDElements.end(), ifdelement3);

	long ifd_offset = 0l;

	if(iFileType)
		ifd_offset = tifh.ifdofs;
	else
	{
		fseek(rfd, 0, SEEK_SET);
		ifd_offset = inspectImageFileHeader(rfd, 0);

		if(ifd_offset == -1l)
		{
			fclose(rfd);
			return strGlobalCOut;
		}
	}

	const string& strType = string("IFD0");
	dumpImageFileDirectory(rfd, ifd_offset, 0, strType, itIFD0);

	string strModel = string(upperCase((char *) getDataValue(0x110, 1, true).c_str()));
	string strShutterCount = getDataValue(0x00a7, 9, true);
	string strSerialNumber29 = getDataValue(0x001d, 9, true);
	string strLens = getDataValue(0x0084, 9, true);
	string strMake = string(upperCase((char *) getDataValue(0x10f, 1, true).c_str()));

/* JON JON JON - attempt to work on VR on/off states

	unsigned char ucLensType = (unsigned char) atoi(getTagValue(0x0083, 9).c_str());

	if(ucLensType == 0x0a || ucLensType == 0x0e)
	{
		if(!strMake.compare(0, 17, "NIKON CORPORATION"))
		{
			for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
			{
				if(itrIFDElements->getTag() == 0x0091)	//Structure containing VR on/off/active flag - while tag 0x0091 points to a structure where the position index 102 contains 0, 1, or 2 - vr off, vr on, vr on active.
				{
					struct VRData
					{
						char VRDataVersion[4];
						unsigned char byte4[98];
						unsigned char ucByteVRState;

					} VRDataStruct;

					long tmp_offset = ftell(rfd);
					unsigned long ulOffset = itrIFDElements->getOffset();
					unsigned long ulAdjustedOffset = itrIFDElements->getAdjustedOffset();
					unsigned char ucByte;
					unsigned long ulVROffset = 0l;
					unsigned long ulFinalOffset = ulVROffset + ulOffset + ulAdjustedOffset;
					string strVRMode;
					string strDecryptedData;

					fseek( rfd, ulFinalOffset, SEEK_SET );
					fread(&VRDataStruct, sizeof(struct VRData), 1, rfd);

					ucByte = VRDataStruct.ucByteVRState;

					if(VRDataStruct.VRDataVersion[1] == '2')
					{
						strDecryptedData = DecryptNikonData(strModel, itrIFDElements->getRawData(), strSerialNumber29, strShutterCount, itrIFDElements->getRawData().length(), 4);

						ucByte = strDecryptedData[99];
					}

					//0, 1, or 2 - vr off, vr on, vr on active
					if(ucByte == 0x00)
						strVRMode = string("VR Off");
					else if(ucByte == 0x01)
						strVRMode = string("VR On");
					else if(ucByte == 0x02)
						strVRMode = string("VR On Active");
					else
						strVRMode = string("VR State Unknown");

					IFDENTRY ifde22(0xFFEB, 2, 0, 0);

					IFDElement ifdelement22(ifde22, itNone);
					ifdelement22.setProgramName(string("d70reader"));
					ifdelement22.setProgramVersion(string(d70Version));
					ifdelement22.setProgramDate(string(d70VersionDate));
					ifdelement22.setFileTypeOffset(fileTypeOffset());
					ifdelement22.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement22.setFileType(iFileType);
					ifdelement22.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement22.setDisplayData(strVRMode);

					listIFDElements.insert(listIFDElements.end(), ifdelement22);

					fseek(rfd, tmp_offset, SEEK_SET);

					break;
				}
			}
		}
	}
JON JON JON */

	if(!strMake.compare(0, 17, "NIKON CORPORATION"))
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x0098)	//LensDataBytes
			{
				//extract and create objects for the following data:
				//LensID(0xfffb)
				//LensDataVersion(0xfff9)
				//FocusDistance(0xfffa)
/*
  { 0, 65516u, "NearFocusLimitDOF_Type0          " },	//0xFFEC
  { 0, 65517u, "FarFocusLimitDOF_Type0           " },	//0xFFED
  { 0, 65518u, "HyperfocalDistance_Type0         " },	//0xFFEE
  { 0, 65519u, "TotalDOF_Type0                   " },	//0xFFEF
  { 0, 65520u, "CircleOfConfusion_Type0          " },	//0xFFF0

  { 65521u, "AFAperture                 " },	//0xFFF1
  { 65522u, "FocusPosition              " },	//0xFFF2
  { 65523u, "MinFocalLength             " },	//0xFFF3
  { 65524u, "MaxFocalLength             " },	//0xFFF4
  { 65525u, "MaxApertureAtMinFocal      " },	//0xFFF5
  { 65526u, "MaxApertureAtMaxFocal      " },	//0xFFF6
  { 65527u, "MCUVersion                 " },	//0xFFF7
  { 65528u, "EffectiveMaxAperture       " },	//0xFFF8

*/
				char buffer[256];
				string strLensString;

				struct lensData
				{
					char lensDataVersion[4];
					unsigned char byte4;
					unsigned char byte5;	//AFAperture
					unsigned char byte6;
					unsigned char byte7;
					unsigned char byte8;	//FocusPosition
					unsigned char byte9;	//focusDistance
					unsigned char byteA;	//FocalLength
					unsigned char byteB;	//lensID
					unsigned char byteC;	//LensFStops
					unsigned char byteD;	//MinFocalLength
					unsigned char byteE;	//MaxFocalLength
					unsigned char byteF;	//MaxApertureAtMinFocal
					unsigned char byte10;	//MaxApertureAtMaxFocal
					unsigned char byte11;	//MCUVersion
					unsigned char byte12;	//EffectiveMaxAperture

				} lensDataStruct;

				int iSpaceOffset = strLens.find_first_of(" ", 0);
				strLens.resize(iSpaceOffset);
				string strLensCompare;
				strLensCompare.append(" ");
				strLensCompare.append(strLens);

				long tmp_offset = ftell(rfd);
				unsigned long ulOffset = itrIFDElements->getOffset();
				unsigned long ulAdjustedOffset = itrIFDElements->getAdjustedOffset();

				fseek( rfd, ulOffset + ulAdjustedOffset, SEEK_SET );
				fread(&lensDataStruct, sizeof(struct lensData), 1, rfd);

				//handle lensDataVersion
				string strLensDataVersion = string(lensDataStruct.lensDataVersion);
				strLensDataVersion.resize(4);

				IFDENTRY ifde(0xFFF9, 2, 0, 0);

				IFDElement ifdelement(ifde, itNone);
				ifdelement.setProgramName(string("d70reader"));
				ifdelement.setProgramVersion(string(d70Version));
				ifdelement.setProgramDate(string(d70VersionDate));
				ifdelement.setFileTypeOffset(fileTypeOffset());
				ifdelement.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
				ifdelement.setFileType(iFileType);
				ifdelement.setASCILabelSkipLength(iASCILabelSkipLength);
				ifdelement.setDisplayData(strLensDataVersion);

				listIFDElements.insert(listIFDElements.end(), ifdelement);

				if(!strLensDataVersion.compare(0, 4, "0100"))
				{
					strLensString = IdentifyLensString(lensDataStruct.byte6,
													   lensDataStruct.byte7,
													   lensDataStruct.byte8,
													   lensDataStruct.byte9,
													   lensDataStruct.byteA,
													   lensDataStruct.byteB,
													   lensDataStruct.byteC,
													   (unsigned char) atoi(getTagValue(0x0083, 9).c_str()));

					//handle MinFocalLength
					double dMinFocalLength = (double) lensDataStruct.byte8;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonFocalConversions(dMinFocalLength));

					IFDENTRY ifde22(0xFFF3, 2, 0, 0);

					IFDElement ifdelement22(ifde22, itNone);
					ifdelement22.setProgramName(string("d70reader"));
					ifdelement22.setProgramVersion(string(d70Version));
					ifdelement22.setProgramDate(string(d70VersionDate));
					ifdelement22.setFileTypeOffset(fileTypeOffset());
					ifdelement22.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement22.setFileType(iFileType);
					ifdelement22.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement22.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement22);

					//handle MaxFocalLength
					double dMaxFocalLength = (double) lensDataStruct.byte9;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonFocalConversions(dMaxFocalLength));

					IFDENTRY ifde23(0xFFF4, 2, 0, 0);

					IFDElement ifdelement23(ifde23, itNone);
					ifdelement23.setProgramName(string("d70reader"));
					ifdelement23.setProgramVersion(string(d70Version));
					ifdelement23.setProgramDate(string(d70VersionDate));
					ifdelement23.setFileTypeOffset(fileTypeOffset());
					ifdelement23.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement23.setFileType(iFileType);
					ifdelement23.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement23.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement23);

					//handle MaxApertureAtMinFocal
					double dMaxApertureAtMinFocal = (double) lensDataStruct.byteA;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dMaxApertureAtMinFocal));

					IFDENTRY ifde24(0xFFF5, 2, 0, 0);

					IFDElement ifdelement24(ifde24, itNone);
					ifdelement24.setProgramName(string("d70reader"));
					ifdelement24.setProgramVersion(string(d70Version));
					ifdelement24.setProgramDate(string(d70VersionDate));
					ifdelement24.setFileTypeOffset(fileTypeOffset());
					ifdelement24.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement24.setFileType(iFileType);
					ifdelement24.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement24.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement24);

					//handle MaxApertureAtMaxFocal
					double dMaxApertureAtMaxFocal = (double) lensDataStruct.byteB;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dMaxApertureAtMaxFocal));

					IFDENTRY ifde25(0xFFF6, 2, 0, 0);

					IFDElement ifdelement25(ifde25, itNone);
					ifdelement25.setProgramName(string("d70reader"));
					ifdelement25.setProgramVersion(string(d70Version));
					ifdelement25.setProgramDate(string(d70VersionDate));
					ifdelement25.setFileTypeOffset(fileTypeOffset());
					ifdelement25.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement25.setFileType(iFileType);
					ifdelement25.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement25.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement25);

					//handle MCUVersion
					unsigned short usMCUVersion = (unsigned short) lensDataStruct.byteC;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usMCUVersion);

					IFDENTRY ifde26(0xFFF7, 2, 0, 0);

					IFDElement ifdelement26(ifde26, itNone);
					ifdelement26.setProgramName(string("d70reader"));
					ifdelement26.setProgramVersion(string(d70Version));
					ifdelement26.setProgramDate(string(d70VersionDate));
					ifdelement26.setFileTypeOffset(fileTypeOffset());
					ifdelement26.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement26.setFileType(iFileType);
					ifdelement26.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement26.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement26);

					//handle lensID
					unsigned short usLensID = (unsigned short) lensDataStruct.byte6;

					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usLensID);

					IFDENTRY ifde3(0xFFFB, 2, 0, 0);

					IFDElement ifdelement3(ifde3, itNone);
					ifdelement3.setProgramName(string("d70reader"));
					ifdelement3.setProgramVersion(string(d70Version));
					ifdelement3.setProgramDate(string(d70VersionDate));
					ifdelement3.setFileTypeOffset(fileTypeOffset());
					ifdelement3.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement3.setFileType(iFileType);
					ifdelement3.setASCILabelSkipLength(iASCILabelSkipLength);
					//ifdelement3.setDisplayData(string(buffer));
					ifdelement3.setDisplayStringData(strLensString);

					if(ifdelement3.getDisplayStringData().find(strLensCompare, 0) == -1)
					{
						if(bVerbos)
						{
							//cout << "\nReported Lens: " << ifdelement3.getDisplayStringData();
							strGlobalCOut.append("\nReported Lens: ");
							strGlobalCOut.append(ifdelement3.getDisplayStringData());

							//cout << "\nReported LensID: " << buffer;
							strGlobalCOut.append("\nReported LensID: ");
							strGlobalCOut.append(buffer);
						}

						ifdelement3.setDisplayData(string("Unknown"));
					}

					listIFDElements.insert(listIFDElements.end(), ifdelement3);
				}

				if(!strLensDataVersion.compare(0, 4, "0101"))
				{
					strLensString = IdentifyLensString(lensDataStruct.byteB,
													   lensDataStruct.byteC,
													   lensDataStruct.byteD,
													   lensDataStruct.byteE,
													   lensDataStruct.byteF,
													   lensDataStruct.byte10,
													   lensDataStruct.byte11,
													   (unsigned char) atoi(getTagValue(0x0083, 9).c_str()));

					//handle AFAperture
					double dAFAperture = (double) lensDataStruct.byte5;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dAFAperture));

					IFDENTRY ifde20(0xFFF1, 2, 0, 0);

					IFDElement ifdelement20(ifde20, itNone);
					ifdelement20.setProgramName(string("d70reader"));
					ifdelement20.setProgramVersion(string(d70Version));
					ifdelement20.setProgramDate(string(d70VersionDate));
					ifdelement20.setFileTypeOffset(fileTypeOffset());
					ifdelement20.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement20.setFileType(iFileType);
					ifdelement20.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement20.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement20);

					//handle FocusPosition
					unsigned short usFocusPosition = (unsigned short) lensDataStruct.byte8;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usFocusPosition);

					IFDENTRY ifde21(0xFFF2, 2, 0, 0);

					IFDElement ifdelement21(ifde21, itNone);
					ifdelement21.setProgramName(string("d70reader"));
					ifdelement21.setProgramVersion(string(d70Version));
					ifdelement21.setProgramDate(string(d70VersionDate));
					ifdelement21.setFileTypeOffset(fileTypeOffset());
					ifdelement21.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement21.setFileType(iFileType);
					ifdelement21.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement21.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement21);

					//handle MinFocalLength
					double dMinFocalLength = (double) lensDataStruct.byteD;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonFocalConversions(dMinFocalLength));

					IFDENTRY ifde22(0xFFF3, 2, 0, 0);

					IFDElement ifdelement22(ifde22, itNone);
					ifdelement22.setProgramName(string("d70reader"));
					ifdelement22.setProgramVersion(string(d70Version));
					ifdelement22.setProgramDate(string(d70VersionDate));
					ifdelement22.setFileTypeOffset(fileTypeOffset());
					ifdelement22.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement22.setFileType(iFileType);
					ifdelement22.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement22.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement22);

					//handle MaxFocalLength
					double dMaxFocalLength = (double) lensDataStruct.byteE;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonFocalConversions(dMaxFocalLength));

					IFDENTRY ifde23(0xFFF4, 2, 0, 0);

					IFDElement ifdelement23(ifde23, itNone);
					ifdelement23.setProgramName(string("d70reader"));
					ifdelement23.setProgramVersion(string(d70Version));
					ifdelement23.setProgramDate(string(d70VersionDate));
					ifdelement23.setFileTypeOffset(fileTypeOffset());
					ifdelement23.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement23.setFileType(iFileType);
					ifdelement23.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement23.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement23);

					//handle MaxApertureAtMinFocal
					double dMaxApertureAtMinFocal = (double) lensDataStruct.byteF;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dMaxApertureAtMinFocal));

					IFDENTRY ifde24(0xFFF5, 2, 0, 0);

					IFDElement ifdelement24(ifde24, itNone);
					ifdelement24.setProgramName(string("d70reader"));
					ifdelement24.setProgramVersion(string(d70Version));
					ifdelement24.setProgramDate(string(d70VersionDate));
					ifdelement24.setFileTypeOffset(fileTypeOffset());
					ifdelement24.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement24.setFileType(iFileType);
					ifdelement24.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement24.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement24);

					//handle MaxApertureAtMaxFocal
					double dMaxApertureAtMaxFocal = (double) lensDataStruct.byte10;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dMaxApertureAtMaxFocal));

					IFDENTRY ifde25(0xFFF6, 2, 0, 0);

					IFDElement ifdelement25(ifde25, itNone);
					ifdelement25.setProgramName(string("d70reader"));
					ifdelement25.setProgramVersion(string(d70Version));
					ifdelement25.setProgramDate(string(d70VersionDate));
					ifdelement25.setFileTypeOffset(fileTypeOffset());
					ifdelement25.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement25.setFileType(iFileType);
					ifdelement25.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement25.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement25);

					//handle MCUVersion
					unsigned short usMCUVersion = (unsigned short) lensDataStruct.byte11;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usMCUVersion);

					IFDENTRY ifde26(0xFFF7, 2, 0, 0);

					IFDElement ifdelement26(ifde26, itNone);
					ifdelement26.setProgramName(string("d70reader"));
					ifdelement26.setProgramVersion(string(d70Version));
					ifdelement26.setProgramDate(string(d70VersionDate));
					ifdelement26.setFileTypeOffset(fileTypeOffset());
					ifdelement26.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement26.setFileType(iFileType);
					ifdelement26.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement26.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement26);

					//handle EffectiveMaxAperture
					double dEffectiveMaxAperture = (double) lensDataStruct.byte12;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dEffectiveMaxAperture));

					IFDENTRY ifde27(0xFFF8, 2, 0, 0);

					IFDElement ifdelement27(ifde27, itNone);
					ifdelement27.setProgramName(string("d70reader"));
					ifdelement27.setProgramVersion(string(d70Version));
					ifdelement27.setProgramDate(string(d70VersionDate));
					ifdelement27.setFileTypeOffset(fileTypeOffset());
					ifdelement27.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement27.setFileType(iFileType);
					ifdelement27.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement27.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement27);

					//handle focusDistance
					unsigned short usFocusDistance = (unsigned short) lensDataStruct.byte9;

					if(usFocusDistance)
						dFocusDistance = pow((double) 10.0, ((double) usFocusDistance / (double) 40.0));

					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f cm | %.4f in | %.4f ft", dFocusDistance, dFocusDistance/2.54, dFocusDistance/2.54/12);

					IFDENTRY ifde2(0xFFFA, 2, 0, 0);

					IFDElement ifdelement2(ifde2, itNone);
					ifdelement2.setProgramName(string("d70reader"));
					ifdelement2.setProgramVersion(string(d70Version));
					ifdelement2.setProgramDate(string(d70VersionDate));
					ifdelement2.setFileTypeOffset(fileTypeOffset());
					ifdelement2.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement2.setFileType(iFileType);
					ifdelement2.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement2.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement2);

					//handle lensID
					unsigned short usLensID = (unsigned short) lensDataStruct.byteB;

					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usLensID);

					IFDENTRY ifde3(0xFFFB, 2, 0, 0);

					IFDElement ifdelement3(ifde3, itNone);
					ifdelement3.setProgramName(string("d70reader"));
					ifdelement3.setProgramVersion(string(d70Version));
					ifdelement3.setProgramDate(string(d70VersionDate));
					ifdelement3.setFileTypeOffset(fileTypeOffset());
					ifdelement3.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement3.setFileType(iFileType);
					ifdelement3.setASCILabelSkipLength(iASCILabelSkipLength);
					//ifdelement3.setDisplayData(string(buffer));
					ifdelement3.setDisplayStringData(strLensString);

					if(ifdelement3.getDisplayStringData().find(strLensCompare, 0) == -1)
					{
						if(bVerbos)
						{
							//cout << "\nReported Lens: " << ifdelement3.getDisplayStringData();
							strGlobalCOut.append("\nReported Lens: ");
							strGlobalCOut.append(ifdelement3.getDisplayStringData());

							//cout << "\nReported LensID: " << buffer;
							strGlobalCOut.append("\nReported LensID: ");
							strGlobalCOut.append(buffer);
						}

						ifdelement3.setDisplayData(string("Unknown"));
					}

					listIFDElements.insert(listIFDElements.end(), ifdelement3);
				}

				if(!strLensDataVersion.compare(0, 4, "0201") || !strLensDataVersion.compare(0, 4, "0202") || !strLensDataVersion.compare(0, 4, "0203"))
				{
					string strDecryptedData;

					strDecryptedData = DecryptNikonData(strModel, itrIFDElements->getRawData(), strSerialNumber29, strShutterCount, itrIFDElements->getRawData().length(), 4);

					unsigned char *pChar = new unsigned char[strDecryptedData.length()];
					memcpy(pChar, strDecryptedData.c_str(), strDecryptedData.length());

					strLensString = IdentifyLensString((unsigned short) pChar[7],
													   (unsigned short) pChar[8],
													   (unsigned short) pChar[9],
													   (unsigned short) pChar[10],
													   (unsigned short) pChar[11],
													   (unsigned short) pChar[12],
													   (unsigned short) pChar[13],
													   (unsigned char) atoi(getTagValue(0x0083, 9).c_str()));

					//handle AFAperture
					double dAFAperture = (double) pChar[1];	//lensDataStruct.byte5;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dAFAperture));

					IFDENTRY ifde20(0xFFF1, 2, 0, 0);

					IFDElement ifdelement20(ifde20, itNone);
					ifdelement20.setProgramName(string("d70reader"));
					ifdelement20.setProgramVersion(string(d70Version));
					ifdelement20.setProgramDate(string(d70VersionDate));
					ifdelement20.setFileTypeOffset(fileTypeOffset());
					ifdelement20.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement20.setFileType(iFileType);
					ifdelement20.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement20.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement20);

					//handle FocusPosition
					unsigned short usFocusPosition = (unsigned short) pChar[4];	//lensDataStruct.byte8;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usFocusPosition);

					IFDENTRY ifde21(0xFFF2, 2, 0, 0);

					IFDElement ifdelement21(ifde21, itNone);
					ifdelement21.setProgramName(string("d70reader"));
					ifdelement21.setProgramVersion(string(d70Version));
					ifdelement21.setProgramDate(string(d70VersionDate));
					ifdelement21.setFileTypeOffset(fileTypeOffset());
					ifdelement21.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement21.setFileType(iFileType);
					ifdelement21.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement21.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement21);

					//handle MinFocalLength
					double dMinFocalLength = (double) pChar[9];	//lensDataStruct.byteD;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonFocalConversions(dMinFocalLength));

					IFDENTRY ifde22(0xFFF3, 2, 0, 0);

					IFDElement ifdelement22(ifde22, itNone);
					ifdelement22.setProgramName(string("d70reader"));
					ifdelement22.setProgramVersion(string(d70Version));
					ifdelement22.setProgramDate(string(d70VersionDate));
					ifdelement22.setFileTypeOffset(fileTypeOffset());
					ifdelement22.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement22.setFileType(iFileType);
					ifdelement22.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement22.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement22);

					//handle MaxFocalLength
					double dMaxFocalLength = (double) pChar[10];	//lensDataStruct.byteE;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonFocalConversions(dMaxFocalLength));

					IFDENTRY ifde23(0xFFF4, 2, 0, 0);

					IFDElement ifdelement23(ifde23, itNone);
					ifdelement23.setProgramName(string("d70reader"));
					ifdelement23.setProgramVersion(string(d70Version));
					ifdelement23.setProgramDate(string(d70VersionDate));
					ifdelement23.setFileTypeOffset(fileTypeOffset());
					ifdelement23.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement23.setFileType(iFileType);
					ifdelement23.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement23.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement23);

					//handle MaxApertureAtMinFocal
					double dMaxApertureAtMinFocal = (double) pChar[11];	//lensDataStruct.byteF;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dMaxApertureAtMinFocal));

					IFDENTRY ifde24(0xFFF5, 2, 0, 0);

					IFDElement ifdelement24(ifde24, itNone);
					ifdelement24.setProgramName(string("d70reader"));
					ifdelement24.setProgramVersion(string(d70Version));
					ifdelement24.setProgramDate(string(d70VersionDate));
					ifdelement24.setFileTypeOffset(fileTypeOffset());
					ifdelement24.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement24.setFileType(iFileType);
					ifdelement24.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement24.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement24);

					//handle MaxApertureAtMaxFocal
					double dMaxApertureAtMaxFocal = (double) pChar[12];	//lensDataStruct.byte10;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dMaxApertureAtMaxFocal));

					IFDENTRY ifde25(0xFFF6, 2, 0, 0);

					IFDElement ifdelement25(ifde25, itNone);
					ifdelement25.setProgramName(string("d70reader"));
					ifdelement25.setProgramVersion(string(d70Version));
					ifdelement25.setProgramDate(string(d70VersionDate));
					ifdelement25.setFileTypeOffset(fileTypeOffset());
					ifdelement25.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement25.setFileType(iFileType);
					ifdelement25.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement25.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement25);

					//handle MCUVersion
					unsigned short usMCUVersion = (unsigned short) pChar[13];	//lensDataStruct.byte11;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usMCUVersion);

					IFDENTRY ifde26(0xFFF7, 2, 0, 0);

					IFDElement ifdelement26(ifde26, itNone);
					ifdelement26.setProgramName(string("d70reader"));
					ifdelement26.setProgramVersion(string(d70Version));
					ifdelement26.setProgramDate(string(d70VersionDate));
					ifdelement26.setFileTypeOffset(fileTypeOffset());
					ifdelement26.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement26.setFileType(iFileType);
					ifdelement26.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement26.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement26);

					//handle EffectiveMaxAperture
					double dEffectiveMaxAperture = (double) pChar[14];	//lensDataStruct.byte12;
					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f", nikonApertureConversions(dEffectiveMaxAperture));

					IFDENTRY ifde27(0xFFF8, 2, 0, 0);

					IFDElement ifdelement27(ifde27, itNone);
					ifdelement27.setProgramName(string("d70reader"));
					ifdelement27.setProgramVersion(string(d70Version));
					ifdelement27.setProgramDate(string(d70VersionDate));
					ifdelement27.setFileTypeOffset(fileTypeOffset());
					ifdelement27.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement27.setFileType(iFileType);
					ifdelement27.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement27.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement27);

					//handle focusDistance
					unsigned short usFocusDistance = (unsigned short) pChar[5];
					//double dFocusDistance = 0.0;

					if(usFocusDistance)
						dFocusDistance = pow((double) 10.0, ((double) usFocusDistance / (double) 40.0));

					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%.4f cm | %.4f in | %.4f ft", dFocusDistance, dFocusDistance/2.54, dFocusDistance/2.54/12);

					IFDENTRY ifde2(0xFFFA, 2, 0, 0);

					IFDElement ifdelement2(ifde2, itNone);
					ifdelement2.setProgramName(string("d70reader"));
					ifdelement2.setProgramVersion(string(d70Version));
					ifdelement2.setProgramDate(string(d70VersionDate));
					ifdelement2.setFileTypeOffset(fileTypeOffset());
					ifdelement2.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement2.setFileType(iFileType);
					ifdelement2.setASCILabelSkipLength(iASCILabelSkipLength);
					ifdelement2.setDisplayData(string(buffer));

					listIFDElements.insert(listIFDElements.end(), ifdelement2);

					//handle lensID
					unsigned short usLensID = (unsigned short) pChar[7];

					memset(buffer, (int) '\0', 256);
					sprintf(buffer, "%d", usLensID);

					IFDENTRY ifde3(0xFFFB, 2, 0, 0);

					IFDElement ifdelement3(ifde3, itNone);
					ifdelement3.setProgramName(string("d70reader"));
					ifdelement3.setProgramVersion(string(d70Version));
					ifdelement3.setProgramDate(string(d70VersionDate));
					ifdelement3.setFileTypeOffset(fileTypeOffset());
					ifdelement3.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
					ifdelement3.setFileType(iFileType);
					ifdelement3.setASCILabelSkipLength(iASCILabelSkipLength);
					//ifdelement3.setDisplayData(string(buffer));
					ifdelement3.setDisplayStringData(strLensString);

					if(ifdelement3.getDisplayStringData().find(strLensCompare, 0) == -1)
					{
						if(bVerbos)
						{
							//cout << "\nReported Lens: " << ifdelement3.getDisplayStringData();
							strGlobalCOut.append("\nReported Lens: ");
							strGlobalCOut.append(ifdelement3.getDisplayStringData());

							//cout << "\nReported LensID: " << buffer;
							strGlobalCOut.append("\nReported LensID: ");
							strGlobalCOut.append(buffer);
						}

						ifdelement3.setDisplayData(string("Unknown"));
					}

					listIFDElements.insert(listIFDElements.end(), ifdelement3);
				}

				fseek(rfd, tmp_offset, SEEK_SET);

				break;
			}
		}
	}

	getDOFData();

	if(bTrace)
	{
		//cout << '\n' << string("Size of List                = ") << listIFDElements.size() << '\n';
		strGlobalCOut.append("\nSize of List                = ");

		char buffer[32];
		memset(glb_buffer, (int) '\0', GLB_SIZE);
		sprintf(glb_buffer, "%d", listIFDElements.size());
		strGlobalCOut.append(glb_buffer);

		strGlobalCOut.append("\n");
	}

	if(bTrace)
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			itrIFDElements->list();
		}
	}

	for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
	{
		if((!strModel.compare(0, 10, "NIKON D100")) || (!strModel.compare(0, 9, "NIKON D2H")))
		{
			if((bThumbnail || bRestoreEXIF || bJpegFine) && (itrIFDElements->getTag() == 0x0011))
				itrIFDElements->setIsWanted(false);
			else
				itrIFDElements->setIsWanted(isTagWanted(itrIFDElements->getTag(), itrIFDElements->getIFDType()));
		}
		else
			itrIFDElements->setIsWanted(isTagWanted(itrIFDElements->getTag(), itrIFDElements->getIFDType()));

		if(bTrace)
		{
			//cout << '\n' << string("IFDType                     = ") << itrIFDElements->getIFDType(itrIFDElements->getIFDType()) << '\n';
			strGlobalCOut.append("\nIFDType                     = ");
			strGlobalCOut.append(itrIFDElements->getIFDType(itrIFDElements->getIFDType()));
			strGlobalCOut.append("\n");

			itrIFDElements->getIFD().list();
		}

		if(itrIFDElements->getTag() == 0xfffe)
		{
			if(bTrace)
			{
				//cout << '\n' << string("ProgramName                 = ") << itrIFDElements->getProgramName();
				strGlobalCOut.append("ProgramName                 = ");
				strGlobalCOut.append(itrIFDElements->getProgramName());

				//cout << '\n' << string(TiffTagName(0xfffe, -1)) << string(" = ") << itrIFDElements->getProgramVersion();
				strGlobalCOut.append("\n");
				strGlobalCOut.append(string(TiffTagName(0xfffe, -1)));
				strGlobalCOut.append(" = ");
				strGlobalCOut.append(itrIFDElements->getProgramVersion());
			}

			itrIFDElements->setDisplayStringData(itrIFDElements->getProgramVersion());
		}

		if(itrIFDElements->getTag() == 0xfffd)
		{
			if(bTrace)
			{
				//cout << '\n' << string(TiffTagName(0xfffd, -1)) << string(" = ") << itrIFDElements->getFileName();
				strGlobalCOut.append("\n");
				strGlobalCOut.append(string(TiffTagName(0xfffd, -1)));
				strGlobalCOut.append(" = ");
				strGlobalCOut.append(itrIFDElements->getFileName());
			}

			itrIFDElements->setDisplayStringData(itrIFDElements->getFileName());
		}

		if(itrIFDElements->getTag() == 0xfffc)
		{
			if(bTrace)
			{
				//cout << '\n' << string(TiffTagName(0xfffd, -1)) << string(" = ") << itrIFDElements->getFileName();
				strGlobalCOut.append("\n");
				strGlobalCOut.append(string(TiffTagName(0xfffd, -1)));
				strGlobalCOut.append(" = ");
				strGlobalCOut.append(itrIFDElements->getFileName());
			}

			string strBuffer;
			string strBuffer2;

			strBuffer.resize(32);

			//ltoa( itrIFDElements->getFileSize(), (char *) strBuffer.c_str(), 10 );
			sprintf((char *) strBuffer.c_str(), "%d", itrIFDElements->getFileSize());

			strBuffer2 = string(strBuffer.c_str());
			strBuffer2.append(" bytes");

			itrIFDElements->setDisplayStringData(strBuffer2);
		}

		if(bTrace)
		{
			//cout << '\n' << string("RawData                     = ") << itrIFDElements->getData();
			strGlobalCOut.append("\nRawData                     = ");
			strGlobalCOut.append(itrIFDElements->getData());

			//cout << '\n' << string("ConvertedData               = ") << itrIFDElements->getDisplayStringData();
			//cout << '\n';
			strGlobalCOut.append("\nConvertedData               = ");
			strGlobalCOut.append(itrIFDElements->getDisplayStringData());
			strGlobalCOut.append("\n");
		}
	}

#ifdef XMP_SDK

	if(imageType() == 0)
	{
		if((xmp_packet_ptr = chk_get_xmp_data(rfd)) != NULL)
		{
			if(bVerbos)
			{
				//cout << '\n';
				//strGlobalCOut.append("\nXMP_DISPLAY\n");
				strGlobalCOut.append(xmp_display(filename));
				eraseStringBuffer();
			}
		}
	}

#endif	//XMP_SDK

	int tagCounter = 0;

	for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
	{
		if(bVerbos)
		{
			char buffer1[16];
			char buffer2[16];
			char buffer3[16];

			memset(buffer1, (int) '\0', 16);
			sprintf(buffer1, "0x%04x", itrIFDElements->getTag());

			memset(buffer2, (int) '\0', 16);
			sprintf(buffer2, "(%05ld)", itrIFDElements->getTag());

			tagCounter++;
			memset(buffer3, (int) '\0', 16);
			sprintf(buffer3, "%03d", tagCounter);

/*
			//cout << '\n' << string(buffer3) 
				 << string(": ") 
				 << string(buffer1) 
				 << string(buffer2) 
				 << string(": ") 
				 << TiffIFDTypeName(itrIFDElements->getIFDType()) 
				 << string("(") 
				 << itrIFDElements->getIFDType() 
				 << string("): ") 
				 << string(TiffTagName( itrIFDElements->getTag(), itrIFDElements->getIFDType() )) 
				 << string(": ") 
				 << itrIFDElements->getDisplayStringData();
*/

			strGlobalCOut.append("\n");
			strGlobalCOut.append(string(buffer3));
			strGlobalCOut.append(": ");
			strGlobalCOut.append(string(buffer1));
			strGlobalCOut.append(string(buffer2));
			strGlobalCOut.append(": ");
			strGlobalCOut.append(TiffIFDTypeName(itrIFDElements->getIFDType()));
			strGlobalCOut.append("(");

			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "%d", itrIFDElements->getIFDType());
			strGlobalCOut.append(glb_buffer);

			strGlobalCOut.append("): ");
			strGlobalCOut.append(string(TiffTagName( itrIFDElements->getTag(), itrIFDElements->getIFDType() )));
			strGlobalCOut.append(": ");
			strGlobalCOut.append(itrIFDElements->getDisplayStringData());
		}
		else
		{
			if(itrIFDElements->isWanted())
			{
				if(bTrace)
				{
					//cout << '\n' 
					//<< TiffIFDTypeName(itrIFDElements->getIFDType()) 
					//	<< string(": ") 
					//	<< string(TiffTagName( itrIFDElements->getTag(), itrIFDElements->getIFDType() )) 
					//	<< string(": ") 
					//	<< itrIFDElements->getDisplayStringData();

					strGlobalCOut.append("\n");
					strGlobalCOut.append(TiffIFDTypeName(itrIFDElements->getIFDType()));
					strGlobalCOut.append(": ");
					strGlobalCOut.append(string(TiffTagName( itrIFDElements->getTag(), itrIFDElements->getIFDType() )));
					strGlobalCOut.append(": ");
					strGlobalCOut.append(itrIFDElements->getDisplayStringData());
				}

				bool bFound = false;

				for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
				{
					if(*itrIFDElements == *itrIFDElements2)
					{
						//bFound = true;	JON JON JON try to include ALL types and tags!
						break;
					}
				}

				if(bThumbnail || bRestoreEXIF || bJpegFine)
				{
					if(itrIFDElements->getTag() != 0x014a)	//SubIFDOffsets are not part of JPG files
					{
						if(bFound)
							*itrIFDElements2 = *itrIFDElements;
						else
							listIFDElements2.insert(listIFDElements2.end(), *itrIFDElements);
					}
				}
				else
				{
					if(bFound)
						*itrIFDElements2 = *itrIFDElements;
					else
						listIFDElements2.insert(listIFDElements2.end(), *itrIFDElements);
				}
			}
			else
			{
				if(bThumbnail)
				{
					switch(itrIFDElements->getTag())
					{
						case 0x8769:	//ExifOffset
						case 0x927c:	//MakerNoteOffset
							listIFDElements2.insert(listIFDElements2.end(), *itrIFDElements);
							break;
						default:
							break;
					}

				}
			}
		}
	}

	if(bOutputPlain)
		outputPlain();

	if(bHtml)
		outputHtml();

	if(bXml)
		outputXml();

	if(bCsv)
		outputCsv();

	if(bJS)
	{
		if(bJSFlagFirstTime)
		{
			bJSFlagFirstTime = false;
#ifdef WIN_GUI
			outputJSCout(true, js_ctr1++);
#else
			outputJs(stdout, true, js_ctr1++);
#endif
		}
		else
#ifdef WIN_GUI
			outputJSCout(false, js_ctr1++);
#else
			outputJs(stdout, false, js_ctr1++);
#endif
	}

	if(bRemoveEXIF)
	{
		if(!iFileType)
			removeExif(filename, rfd);
		else
			//cout << "\nCan't process NEF files\n\n";
			strGlobalCOut.append("\nCan't process NEF files\n\n");

		listIFDElements.clear();
		listIFDElements2.clear();

		int iList1 = listIFDElements.size();
		int iList2 = listIFDElements2.size();

		fclose(rfd);
		return strGlobalCOut;
	}

#ifdef ALLOW_ROTATE
	if(GlobalOrientationPtr != NULL)
		free(GlobalOrientationPtr);

	GlobalOrientationPtr = (char *) malloc(5);
	memset(GlobalOrientationPtr, (int) '\0', 5);
	sprintf(GlobalOrientationPtr, "%s", getOrientation());

	if(bWeb)
	{
		//cout << "\nD70ReaderProgramVersion    : " << d70Version;
		strGlobalCOut.append("\nD70ReaderProgramVersion    : ");
		strGlobalCOut.append(d70Version);

		//cout << "\nFilename                   : " << filename;
		strGlobalCOut.append("\nFilename                   : ");
		strGlobalCOut.append(filename);

		getSOF0Data(rfd);
		outputWeb();
	}
#endif //ALLOW_ROTATE

	if(iFileType)
	{
		if(bThumbnail)
			outputThumbnail(filename, rfd);
	}

	if(bRestoreEXIF)
	{
		restoreEXIF(filename, NULL);
	}

	if(bFixDate)
	{
		fixDate(filename, rfd);
	}

	if(bFixDateWithOffset)
	{
		fixDateWithOffset(filename, rfd);
	}

	if(bRenameImage)
	{
		renameImage(filename, rfd);
	}

	fclose(rfd);

	if(bJpegFine)
		outputJpegFine(filename);

	listIFDElements.clear();
	listIFDElements2.clear();

	int iList1 = listIFDElements.size();
	int iList2 = listIFDElements2.size();

	return strGlobalCOut;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& getWebConfigElement(const string& strElement)
{
	static string strLocalString;
	strLocalString.resize(0);
	strLocalString.append("Unknown");

	for( itrWebConfig = listWebConfig.begin(); itrWebConfig != listWebConfig.end(); itrWebConfig++)
	{
		if(!itrWebConfig->getTextElement().compare(0, strElement.length(), strElement))
			return itrWebConfig->getTextElementValue();
	}

	return strLocalString;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
int getSOF0Data(FILE *rfd)
{
	int ctr1 = 0;
	int iFoundEXIF = 0;

	long position = 0l;
	long old_position = 0l;
	int oldMotorolaOrder = 0;
	int height = LARGE_IMAGE_HEIGHT;
	int width = LARGE_IMAGE_WIDTH;
	char str_width[10];
	char str_height[10];

	unsigned char m_percision;
	unsigned short m_height;
	unsigned short m_width;

	struct marker
	{
		unsigned char pad;
		unsigned char typeMarker;
		unsigned short length;
	};

	struct marker *temp_ptr;

	oldMotorolaOrder = reverse_byte_order;
	reverse_byte_order = 1;

	old_position = ftell(rfd);

	temp_ptr = (struct marker *) malloc(sizeof(struct marker));
	memset(temp_ptr, (int) '\0', sizeof(struct marker));

	if(!iFileType)
		fseek(rfd, 2, 0);
	else
	{
		string strModel = string(upperCase((char *) getDataValue(0x110, 1, true).c_str()));

		if((!strModel.compare(0, 10, "NIKON D100")) || (!strModel.compare(0, 9, "NIKON D2H")))
		{
			fseek(rfd, lThumbnailOffset + ulCalcMakerNoteHeaderOffset + 2, 0);
		}
		else
		{
			fseek(rfd, lThumbnailOffset + 2, 0);
		}
	}

	do
	{
		ctr1++;
		position = ftell(rfd);

		memset(temp_ptr, (int) '\0', sizeof(struct marker));
		fread(temp_ptr, sizeof(struct marker), 1, rfd);

		if((temp_ptr->typeMarker == 0xc0) || (temp_ptr->typeMarker == 0xc1) || (temp_ptr->typeMarker == 0xc2))
		{
			fread(&m_percision, sizeof(unsigned char), 1, rfd);
			fread(&m_height, sizeof(unsigned short), 1, rfd);
			fread(&m_width, sizeof(unsigned short), 1, rfd);
			iFoundEXIF = 1;
			break;
		}
		else
#ifdef APPLE
			fseek(rfd, temp_ptr->length - sizeof(struct marker) + 2, 1);
#else
			fseek(rfd, Get16u(&temp_ptr->length) - sizeof(struct marker) + 2, 1);
#endif	// APPLE

		if(temp_ptr->typeMarker == 0xda)
			break;
	}
	while(temp_ptr->typeMarker != 0xc0);	

	fseek(rfd, old_position, 0);

	if(iFoundEXIF)
	{
#ifdef APPLE
		iGlobalImageWidth = m_width;
		iGlobalImageHeight = m_height;
#else
		iGlobalImageWidth = Get16u(&m_width);
		iGlobalImageHeight = Get16u(&m_height);
#endif	// APPLE

		if(bWeb)
		{
			if((iGlobalImageWidth != 0) && (iGlobalImageHeight != 0))
			{
				if(bTrace)
				{
					//printf("\nGlobalOrientationPtr: %s", GlobalOrientationPtr);
					strGlobalCOut.append("\nGlobalOrientationPtr: %s");
					strGlobalCOut.append(GlobalOrientationPtr);
				}

				if((!strcmp(GlobalOrientationPtr, "6") || !strcmp(GlobalOrientationPtr, "8")) || 
					(!strcmp(GlobalOrientationPtr, "1") && (iGlobalImageHeight > iGlobalImageWidth)) ||
					(!strcmp(GlobalOrientationPtr, "X") && (iGlobalImageHeight > iGlobalImageWidth)))
				{
					if(bTrace)
						//printf("\ndisplay_data - vertical");
						strGlobalCOut.append("\ndisplay_data - vertical");

					height = LARGE_IMAGE_LENGTH;

					if(!strcmp(GlobalOrientationPtr, "1") || !strcmp(GlobalOrientationPtr, "X"))
						width = (height * iGlobalImageWidth) / iGlobalImageHeight;
					else
					{
						if(iGlobalImageHeight > iGlobalImageWidth)
							width = (height * iGlobalImageWidth) / iGlobalImageHeight;
						else
							width = (height * iGlobalImageHeight) / iGlobalImageWidth;
					}
				}
				else
				{
					if(bTrace)
						//printf("\ndisplay_data - horizontal");
						strGlobalCOut.append("\ndisplay_data - horizontal");

					width = LARGE_IMAGE_LENGTH;
					height = (width * iGlobalImageHeight) / iGlobalImageWidth;
				}

				memset(str_width, (int) '\0', 10);
				memset(str_height, (int) '\0', 10);

				sprintf(str_width, "%d", width);
				sprintf(str_height, "%d", height);

				if(bTrace)
				{
					//printf("\nwidth: %s", str_width);
					strGlobalCOut.append("\nwidth: ");
					strGlobalCOut.append(str_width);

					//printf("\nheight: %s", str_height);
					strGlobalCOut.append("\nheight: ");
					strGlobalCOut.append(str_height);
				}

				replaceDescriptionValue(0xA002, 6, str_width);
				replaceDescriptionValue(0xA003, 6, str_height);
			}
		}
	}

	reverse_byte_order = oldMotorolaOrder;

	free(temp_ptr);

	return(iFoundEXIF);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void replaceDescriptionValue(unsigned short usTag, int iType, char *localPtr)
{
	bool bNotFound = true;

	if(!listIFDElements2.empty())
	{
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if((itrIFDElements2->getTag() == usTag) && (itrIFDElements2->getIFDType() == iType))
			{
				bNotFound = false;
				itrIFDElements2->setData(string(localPtr));
				itrIFDElements2->setDisplayData(string(localPtr));

				if(bTrace)
				{
					//cout << '\n' << "localPtr: " << localPtr;
					strGlobalCOut.append("\nlocalPtr: ");
					strGlobalCOut.append(localPtr);

					//cout << '\n' << "usTag: " << usTag;
					strGlobalCOut.append("\nusTag: ");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", usTag);
					strGlobalCOut.append(glb_buffer);

					//cout << '\n' << "data: " << itrIFDElements2->getData();
					strGlobalCOut.append("\ndata: ");
					strGlobalCOut.append(itrIFDElements2->getData());

					//cout << '\n' << "displayData: " << itrIFDElements2->getDisplayStringData();
					strGlobalCOut.append("\ndisplayData: ");
					strGlobalCOut.append(itrIFDElements2->getDisplayStringData());
				}

				break;
			}
		}

		if(bNotFound)
		{
			IFDENTRY ifde(usTag, iType, 0, 0);

			IFDElement ifdelement(ifde, iType);
			ifdelement.setProgramName(string("d70reader"));
			ifdelement.setProgramVersion(string(d70Version));
			ifdelement.setProgramDate(string(d70VersionDate));
			ifdelement.setFileTypeOffset(fileTypeOffset());
			ifdelement.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
			ifdelement.setFileType(iFileType);
			ifdelement.setASCILabelSkipLength(iASCILabelSkipLength);
			ifdelement.setData(string(localPtr));
			ifdelement.setDisplayData(string(localPtr));

			listIFDElements2.insert(listIFDElements2.end(), ifdelement);

		}
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *getOrientation()
{
	static string strLocalString;

	strLocalString.resize(0);
	strLocalString.append("X");

	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if((itrIFDElements->getTag() == 0x0112) && (itrIFDElements->getIFDType() == itIFD0))
			{
				strLocalString.resize(0);
				strLocalString.append(itrIFDElements->getData());
				break;
			}
		}
	}

	return((char*) strLocalString.c_str());
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
struct xmp_packet *chk_get_xmp_data(FILE *rfd)
{
	int ctr1 = 0;
	bool bFoundHTTP = false;
	char * data_ptr = NULL;
	struct xmp_packet *xmp_ptr = NULL;
	
	unsigned short length = 0;
	long position = 0l;
	long old_position = 0l;
	int oldMotorolaOrder = 0;

	struct marker
	{
		unsigned char pad;
		unsigned char typeMarker;
		unsigned short length;
		char test_data[6];
	};

	struct marker *temp_ptr = NULL;

	old_position = ftell(rfd);

	oldMotorolaOrder = reverse_byte_order;
	reverse_byte_order = 1;

	temp_ptr = (struct marker *) malloc(sizeof(struct marker));
	memset(temp_ptr, (int) '\0', sizeof(struct marker));

	fseek(rfd, 2, 0);

	do
	{
		ctr1++;
		position = ftell(rfd);

		memset(temp_ptr, (int) '\0', sizeof(struct marker));
		fread(temp_ptr, sizeof(struct marker), 1, rfd);

		length = Get16u(&temp_ptr->length);

		if(!strncmp(temp_ptr->test_data, "http:", strlen("http:")))
		{
			bFoundHTTP = 1;
			break;
		}
		else
			fseek(rfd, Get16u(&temp_ptr->length) - sizeof(struct marker) + 2, 1);

		if(temp_ptr->typeMarker == 0xda)
			break;
	}
	while(strncmp(temp_ptr->test_data, "http:", strlen("http:")));	

	if(bFoundHTTP)
	{
		data_ptr = (char *) malloc((Get16u(&temp_ptr->length) + 1));
		memset(data_ptr, (int) '\0', (Get16u(&temp_ptr->length) + 1));

		fseek(rfd, position + 4, 0);
		fread(data_ptr, Get16u(&temp_ptr->length), 1, rfd);

		xmp_ptr = (struct xmp_packet *) malloc(sizeof(struct xmp_packet));
		memset(xmp_ptr, (int) '\0', sizeof(struct xmp_packet));
		xmp_ptr->length = Get16u(&temp_ptr->length);
		xmp_ptr->value = data_ptr;
	}

	reverse_byte_order = oldMotorolaOrder;
	fseek(rfd, old_position, 0);

	return(xmp_ptr);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
void outputPlain()
{
	if(bSortPlain)
	{
		vector<string> vEXIFData;
		string strEXIFData;
		
		if(!listIFDElements2.empty())
		{
			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				//strEXIFData = string(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() )) + string(": ") + string(itrIFDElements2->getDisplayStringData().c_str());

				
				 strEXIFData = string(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() )) 
								 + string(": ") 
								 + TiffIFDTypeName(itrIFDElements2->getIFDType()) 
								 + string(": ") 
								 + itrIFDElements2->getDisplayStringData();
				
				
				
				vEXIFData.push_back(strEXIFData);
			}
		}

		string strTemp;
		QuickSortCString(vEXIFData, 0, vEXIFData.size() - 1, strTemp);

		vector<string>::iterator iter;

		for(iter = vEXIFData.begin(); iter != vEXIFData.end(); iter++)
		{
			//cout << "\n" << iter->c_str();
			strGlobalCOut.append("\n");
			strGlobalCOut.append(iter->c_str());
		}

		//cout << '\n';
		strGlobalCOut.append("\n");
	}
	else
	{
		if(!listIFDElements2.empty())
		{
			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				//cout << '\n' 
				//	 << string(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() )) 
				//	 << string(": ") 
				//	 << TiffIFDTypeName(itrIFDElements2->getIFDType()) 
				//	 << string(": ") 
				//	 << itrIFDElements2->getDisplayStringData();

				strGlobalCOut.append("\n");
				strGlobalCOut.append(string(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() )) );
				strGlobalCOut.append(": ");
				strGlobalCOut.append(TiffIFDTypeName(itrIFDElements2->getIFDType()));
				strGlobalCOut.append(": ");
				strGlobalCOut.append(itrIFDElements2->getDisplayStringData());
			}
		}

		//cout << '\n';
		strGlobalCOut.append("\n");
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputHtml()
{
	//cout << "\n\n<table border=\"1\" width=\"450\" cellpadding=\"1\" align=\"center\">\n";
	strGlobalCOut.append("\n\n<table border=\"1\" width=\"450\" cellpadding=\"1\" align=\"center\">\n");

	if(!listIFDElements2.empty())
	{
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			//cout 
			//  << "<tr><td>" 
			//    << string(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() )) 
			//	<< "</td><td>" 
			//	<< itrIFDElements2->getDisplayStringData() 
			//	<< "</td></tr>\n";

			strGlobalCOut.append("<tr><td>");
			strGlobalCOut.append(string(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() )) );
			strGlobalCOut.append("</td><td>");
			strGlobalCOut.append(itrIFDElements2->getDisplayStringData());
			strGlobalCOut.append("</td></tr>\n");
		}
	}

	//cout << "</table>\n";
	strGlobalCOut.append("</table>\n");
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputXml()
{
	//cout << "\n\n<?xml version=\"1.0\" standalone=\"yes\"?>\n";
	strGlobalCOut.append("\n\n<?xml version=\"1.0\" standalone=\"yes\"?>\n");

	//cout << "<data>\n";
	strGlobalCOut.append("<data>\n");

	if(!listIFDElements2.empty())
	{
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			//cout 
			//	<< "<" 
			//	<< string(strip(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType()) )) 
			//	<< ">" 
			//	<< itrIFDElements2->getDisplayStringData() 
			//	<< "</" 
			//	<< string(strip(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() ))) 
			//	<< ">\n";

			strGlobalCOut.append("<");
			strGlobalCOut.append(string(strip(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType()) )) );
			strGlobalCOut.append(">");
			strGlobalCOut.append(itrIFDElements2->getDisplayStringData());
			strGlobalCOut.append("</");
			strGlobalCOut.append(string(strip(TiffTagName( itrIFDElements2->getTag(), itrIFDElements2->getIFDType() ))));
			strGlobalCOut.append(">\n");
		}
	}

	//cout << "</data>\n";
	strGlobalCOut.append("</data>\n");
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputCsv()
{
	int ctr1 = 0;
	struct all_wanted_tags *local_header_all_ptr = NULL;
	struct html_header *local_header_start_ptr = NULL;
	bool bFirstTag = false;

	if(bCsvFirstTime)
	{
		if(!listWantedTags.empty())
		{
			for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
			{
				if(itrWantedTags->isWanted())
				{
					if(bFirstTag)
						//cout << ",";
						strGlobalCOut.append(",");
					else
						bFirstTag = true;

					//cout << "\"" 
					//	<< string(strip(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType()) )) 
					//	<< "\"";
					strGlobalCOut.append("\"");
					strGlobalCOut.append(string(strip(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType()) )));
					strGlobalCOut.append("\"");

				}
			}
		}

		//cout << "\n";
		strGlobalCOut.append("\n");

		bCsvFirstTime = 0;
	}

	bFirstTag = false;

	if(!listWantedTags.empty())
	{
		for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
		{
			if(itrWantedTags->isWanted())
			{
				if(bFirstTag)
					//cout << ",";
					strGlobalCOut.append(",");
				else
					bFirstTag = true;

				//cout <<  "\"" << getDataValue(itrWantedTags->getTag(), itrWantedTags->getIFDType(), false) << "\"";
				strGlobalCOut.append("\"");
				strGlobalCOut.append(getDataValue(itrWantedTags->getTag(), itrWantedTags->getIFDType(), false));
				strGlobalCOut.append("\"");
			}
		}
	}

	//cout << "\n";
	strGlobalCOut.append("\n");
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputJs(FILE *ofd, bool bFirstTime, int iIndex)
{
	int ctr1 = 0;
	bool bFirstTag = false;

	if(bTrace)
	{
		if(!listWantedTags.empty())
		{
			for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
			{
				if(itrWantedTags->isWanted())
				{
					//cout << '\n' << "Tag: " << itrWantedTags->getTag();
					strGlobalCOut.append("\n");
					strGlobalCOut.append("Tag: ");

					//cout << '\n' << strip(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType()));
					strGlobalCOut.append("\n");
					strGlobalCOut.append(strip(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType())));
				}
			}

			//cout << '\n';
			strGlobalCOut.append("\n");
		}
	}

	if(bFirstTime)
	{
		fprintf(ofd, "aExifData = new Array()");
		fprintf(ofd, "\naExifDataHeader = new Array()");
		fprintf(ofd, "\naExifDataHeader[0] = new Array(");

		if(!listWantedTags.empty())
		{
			for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
			{
				if(itrWantedTags->isWanted())
				{
					if(bFirstTag)
						fprintf(ofd, ",");
					else
						bFirstTag = true;

					//if(itrWantedTags != listWantedTags.begin())
					//	fprintf(ofd, ",");

					fprintf(ofd, "\"%s\"", strip(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType())));
				}
			}
		}

		fprintf(ofd, ")\n");

		bFirstTime = false;
	}

	bFirstTag = false;

	fprintf(ofd, "aExifData[%d] = new Array(", iIndex);

	if(!listWantedTags.empty())
	{
		for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
		{
			if(itrWantedTags->isWanted())
			{
				if(bFirstTag)
					fprintf(ofd, ",");
				else
					bFirstTag = true;

				//if(itrWantedTags != listWantedTags.begin())
				//	fprintf(ofd, ",");

				string strTemp;
				strTemp = getDataValue(itrWantedTags->getTag(), itrWantedTags->getIFDType(), false);

				fprintf(ofd, "\"%s\"", strTemp.c_str() );
			}
		}
	}

	fprintf(ofd, ")\n");
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
#ifdef WIN32
string& getDrive(char *filename)
{
	static string strPathname;

	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];

	_splitpath( filename, drive, dir, fname, ext );

	strPathname.resize(0);
	drive[1] = (char) '\0';
	strPathname.append(drive);

	return strPathname;
}
#endif	//WIN32

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& getFilePath(char *filename)
{
	static string strPathname;

#ifdef WIN32
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];

	_splitpath( filename, drive, dir, fname, ext );

	strPathname.resize(0);
	strPathname.append(drive);
	strPathname.append(dir);
#else
    string strTemp = string(filename);
    int iPos = 0;
    iPos = strTemp.find_last_of("/", strTemp.length());
    strTemp.resize(iPos + 1);
    strPathname.resize(0);
    strPathname.append(strTemp);
#endif	//WIN32

	return strPathname;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
#ifdef WIN32
string& getFileExt(char *filename)
{
	static string strFilename;

	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];

	_splitpath( filename, drive, dir, fname, ext );

	strFilename.resize(0);
	strFilename.append(ext);

	return strFilename;
}
#endif	//WIN32

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& getFileName(char *filename)
{
	static string strFilename;

#ifdef WIN32
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];

	_splitpath( filename, drive, dir, fname, ext );

	strFilename.resize(0);
	strFilename.append(fname);
	strFilename.append(ext);
#else
    string strTemp = string(filename);
    int iPos = 0;
    iPos = strTemp.find_last_of("/", strTemp.length());
    string strTemp2 = strTemp.substr((iPos + 1), (strTemp.length() - iPos));
    strFilename.resize(0);
    strFilename.append(strTemp2);
#endif	//WIN32

	return strFilename;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& getBaseName(char *filename)
{
	static string strFilename;

#ifdef WIN32
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];

	_splitpath( filename, drive, dir, fname, ext );

	strFilename.resize(0);
	strFilename.append(fname);
#else
    string strTemp = string(filename);
    int iPos = 0;
    iPos = strTemp.find_last_of("/", strTemp.length());
    string strTemp2 = strTemp.substr((iPos + 1), (strTemp.length() - iPos));
    iPos = strTemp2.find_last_of(".", strTemp2.length());
    string strTemp3 = strTemp2.substr(0, iPos);
    strFilename.resize(0);
    strFilename.append(strTemp3);
#endif	//WIN32

	return strFilename;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void removeExif(char *filename, FILE *rfd)
{
	unsigned long size = 0l;
	FILE *ofd = NULL;
	char *buf_ptr = NULL;
	long position = 0l;
	long db_position = 0l;
	long c0_position = 0l;

	string strFilePath;
	string strFileName;

	strFilePath.append(getFilePath(filename));
	strFileName.append(getFileName(filename));

	string strMake = getDataValue(0x10f, 1, true);

	if(!strMake.compare(0, 17, "NIKON CORPORATION"))
	{
		db_position = getMarkerOffset(rfd, 0x00db);
		c0_position = getMarkerOffset(rfd, 0x00c0);

		if((c0_position != 0) && (c0_position < db_position))
			position = c0_position;
		else
			position = db_position;

		if(position)
		{
			fseek(rfd, position, 0);

			buf_ptr = (char *) malloc((lFileSize - position + 1));
			memset(buf_ptr, (int) '\0', (lFileSize - position + 1));
			fread(buf_ptr, (lFileSize - position), 1, rfd);

			string strTempFile;

			//d70reader -rm -od:p:\work\junk\ f:\share\d70\*.jpg
			if(bOutputDirectory)
				strTempFile = strOutputDirectory;
			else
				strTempFile = strFilePath;

#ifdef WIN_GUI
			strTempFile += string("d70readerw");
#else
			strTempFile += string("d70reader");
#endif

			bool bFound = isDirectoryFound(strTempFile);

			if(!bFound)
#ifdef WIN_GUI
			{
				_mkdir(strTempFile.c_str());
				pClassPtr->InsertNewDirectory(CString(strFilePath.c_str()), CString("d70readerw"));
			}
#else
				my_mkdir((char *) strTempFile.c_str());
#endif

			if(iSystemType)
				strTempFile += string("\\no_exif");
			else
				strTempFile += string("/no_exif");

			bFound = isDirectoryFound(strTempFile);

			if(!bFound)
#ifdef WIN_GUI
			{
				_mkdir(strTempFile.c_str());
				pClassPtr->InsertNewDirectory(CString(strFilePath.c_str()) + CString("d70readerw") + CString("\\"), CString("no_exif"));
			}
#else
				my_mkdir((char *) strTempFile.c_str());
#endif

			if(iSystemType)
				strTempFile += string("\\");
			else
				strTempFile += string("/");

			strTempFile.append(strFileName);

			if((ofd = fopen(strTempFile.c_str(), "wb")) == NULL)
			{
				//printf("\nFile not opened: %s\n\n", strTempFile.c_str());
				strGlobalCOut.append("\nFile not opened: ");
				strGlobalCOut.append(strTempFile);
				strGlobalCOut.append("\n\n");
				return;
			}

			fputc(0xff, ofd);
			fputc(0xd8, ofd);

			if((size = fwrite(buf_ptr, (lFileSize - position), 1, ofd)) == 0)
			{
				//printf("\nFailed to write to file: %s", strTempFile.c_str());
				strGlobalCOut.append("\nFailed to write to file: ");
				strGlobalCOut.append(strTempFile);
				return;
			}

			fclose(ofd);

			//printf("\nCreated file: %s", strTempFile.c_str());
			strGlobalCOut.append("\nCreated file: ");
			strGlobalCOut.append(strTempFile);

			free(buf_ptr);
		}
	}
	else
	{
		string strFileName = getDataValue(0xfffd, 0, true);

		//cout << "\n\nCan't handle non-Nikon image file: " << strFileName << "\n";
		strGlobalCOut.append("\n\nCan't handle non-Nikon image file: ");
		strGlobalCOut.append(strFileName);
		strGlobalCOut.append("\n");
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
long getMarkerOffset(FILE *rfd, unsigned short us_marker)
{
	int ctr1 = 0;
	bool bFoundDB = false;

	long position = 0l;
	long old_position = 0l;
	int oldMotorolaOrder = 0;

	struct marker
	{
		unsigned char pad;
		unsigned char typeMarker;
		unsigned short length;
	};

	struct marker *temp_ptr;

	oldMotorolaOrder = reverse_byte_order;
	reverse_byte_order = 1;

	old_position = ftell(rfd);

	temp_ptr = (struct marker *) malloc(sizeof(struct marker));
	memset(temp_ptr, (int) '\0', sizeof(struct marker));

	fseek(rfd, 2, 0);

	do
	{
		ctr1++;
		position = ftell(rfd);

		memset(temp_ptr, (int) '\0', sizeof(struct marker));
		fread(temp_ptr, sizeof(struct marker), 1, rfd);

		if(temp_ptr->pad != 0xff)
		{
			do
			{
				fseek(rfd, (position - sizeof(struct marker) + 1), 0);
				fread(temp_ptr, sizeof(struct marker), 1, rfd);
				position = ftell(rfd);
			}
			while(temp_ptr->pad != 0xff);

			position = position - sizeof(struct marker);
		}

		if(temp_ptr->typeMarker == us_marker)
		{
			bFoundDB = true;
			break;
		}
		else
#ifdef APPLE
			fseek(rfd, temp_ptr->length - sizeof(struct marker) + 2, 1);
#else
			fseek(rfd, Get16u(&temp_ptr->length) - sizeof(struct marker) + 2, 1);
#endif	// APPLE

		if(temp_ptr->typeMarker == 0xda)
			break;
	}
	while(temp_ptr->typeMarker != us_marker);

	fseek(rfd, old_position, 0);

	reverse_byte_order = oldMotorolaOrder;

	free(temp_ptr);

	if(bFoundDB)
		return(position);
	else
		return(0);
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
string& getDataValue(unsigned short usTag, int iTag, bool bMainList)
{
	static string strLocalString;

	strLocalString.resize(0);

	if(bMainList)
	{
		if(!listIFDElements.empty())
		{
			for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
			{
				if((itrIFDElements->getTag() == usTag) && (itrIFDElements->getIFDType() == iTag))
				{
					strLocalString.append(itrIFDElements->getDisplayStringData());
					break;
				}
			}
		}
	}
	else
	{
		if(!listIFDElements2.empty())
		{
			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				if((itrIFDElements2->getTag() == usTag) && (itrIFDElements2->getIFDType() == iTag))
				//if(itrIFDElements2->getTag() == usTag)
				{
					strLocalString.append(itrIFDElements2->getDisplayStringData());
					break;
				}
			}
		}
	}

	return strLocalString;
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
string& getDataValueRaw(unsigned short usTag, int iTag, bool bMainList)
{
	static string strLocalString;

	strLocalString.resize(0);

	if(bMainList)
	{
		if(!listIFDElements.empty())
		{
			for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
			{
				if((itrIFDElements->getTag() == usTag) && (itrIFDElements->getIFDType() == iTag))
				{
					strLocalString.append(itrIFDElements->getData());
					break;
				}
			}
		}
	}
	else
	{
		if(!listIFDElements2.empty())
		{
			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				if((itrIFDElements2->getTag() == usTag) && (itrIFDElements2->getIFDType() == iTag))
				{
					strLocalString.append(itrIFDElements2->getData());
					break;
				}
			}
		}
	}

	return strLocalString;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *strip(char *str_input)
{
	char *temp_ptr = NULL;
	char *token = NULL;

	temp_ptr = strdup(str_input);
	token = strtok(temp_ptr, " ");

	return(token);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
bool isTagYes(char *local_ptr)
{
	bool iWanted = false;
	char *temp_ptr = NULL;
	char *final_ptr = NULL;
	char *token = NULL;

	temp_ptr = strdup(local_ptr);

	token = strtok(temp_ptr, "=");

	do
	{
		if(token != NULL)
			final_ptr = token;

		token = strtok(NULL, "=");
	}
	while(token != NULL);

	if(!strncmp(upperCase(final_ptr), "YES", strlen("YES")))
		iWanted = true;
	else
		iWanted = false;

	return(iWanted);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
unsigned short getTagFromDescription(char *local_ptr)
{
	unsigned short usTag = 0;
	char *temp_ptr = NULL;
	char *temp2_ptr = NULL;
	char *token = NULL;
	char *token2 = NULL;
	char *token3 = NULL;
	int iType = 0;

	temp_ptr = strdup(local_ptr);

	token = strtok(temp_ptr, "=");

	temp2_ptr = strdup(token);

	token2 = strtok(temp_ptr, "_\n");
	token3 = strtok(NULL, "_\n");

	iType = atoi(&token3[4]);

	usTag = TiffTagNumber(temp2_ptr, iType);

	free(temp_ptr);
	free(temp2_ptr);

	strWantedFlags.type = iType;
	strWantedFlags.tag = usTag;

	return usTag;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void checkForConfig()
{
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char buffer[256];
	char *file_ptr = NULL;
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;

	if(iSystemType)
	{
		file_ptr = (char *) malloc(strlen(file_sys_dir_ptr) + strlen(strConfigFile.c_str()) + 2);
		memset(file_ptr, (int) '\0', (strlen(file_sys_dir_ptr) + strlen(strConfigFile.c_str()) + 2));
		strcat(file_ptr, file_sys_dir_ptr);
		strcat(file_ptr, "\\");
		strcat(file_ptr, strConfigFile.c_str());
	}
	else
	{
		file_ptr = (char *) malloc(strlen(file_sys_dir_ptr) + strlen(strConfigFile.c_str()) + 2);
		memset(file_ptr, (int) '\0', (strlen(file_sys_dir_ptr) + strlen(strConfigFile.c_str()) + 2));
		strcat(file_ptr, file_sys_dir_ptr);
		strcat(file_ptr, "/");
		strcat(file_ptr, strConfigFile.c_str());
	}

	if(stat(file_ptr, &buf) == (-1))
	{
		//cout << "\nInitial creation of \"" << file_ptr << "\"\n";
		strGlobalCOut.append("\nInitial creation of \"");
		strGlobalCOut.append(file_ptr);
		strGlobalCOut.append("\"\n");

		if((ofd = fopen(file_ptr, "w")) == NULL)
		{
			string strTemp = string("\nFile not created: ") + 
							 string(file_ptr) + 
							 string("\nThis must be done by the 'root' user.\n\n");

			ThrowD70Exception(strTemp);
		}

		for(int ctr = 0; ctr < 11; ctr++)
		{
			for(ctr1 = 0; ctr1 < 0xffff; ctr1++)
			{
				char *pTiffTagName = strip(TiffTagName(ctr1, ctr));

				string strTagName(pTiffTagName);

				if(strTagName.compare(0, 7, "Unknown"))
				{
					data_buffer_ptr = (char *) malloc(strlen(strip(TiffTagName(ctr1, ctr))) + 6);
					memset(data_buffer_ptr, (int) '\0', (strlen(strip(TiffTagName(ctr1, ctr))) + 6));
					sprintf(data_buffer_ptr, "%s=yes\n", strip(TiffTagName(ctr1, ctr)));

					if((size = fwrite(data_buffer_ptr, (strlen(strip(TiffTagName(ctr1, ctr))) + 5), 1, ofd)) == 0)
					{
						//cout << "\nFile not written: " << file_ptr << "\n\n";
						strGlobalCOut.append("\nFile not written: ");
						strGlobalCOut.append(file_ptr);
						strGlobalCOut.append("\n\n");
						return;
					}

					WantedTags wtagelement = WantedTags(true, ctr1, ctr);
					listWantedTags.insert(listWantedTags.begin(), wtagelement);

					free(data_buffer_ptr);
				}
			}
		}

		fclose(ofd);

#ifdef __UNIX_ONLY__
		chmod(file_ptr, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
#endif
	}
	else
	{
		if((rfd = fopen(file_ptr, "r")) == NULL)
		{
			//cout << "\nFile not opened: " << file_ptr << "\n\n";
			strGlobalCOut.append("\nFile not opened: ");
			strGlobalCOut.append(file_ptr);
			strGlobalCOut.append("\n\n");
			return;
		}

		do
		{
			memset(buffer, (int) '\0', 256);
			data_buffer_ptr = fgets(buffer, 256, rfd);

			if(data_buffer_ptr == NULL)
				break;

			WantedTags wtagelement;
			unsigned short usTag = getTagFromDescription(buffer);
			int iType = strWantedFlags.type;

			if(isTagYes(buffer))
				wtagelement = WantedTags(true, usTag, iType);
			else
				wtagelement = WantedTags(false, usTag, iType);

			listWantedTags.insert(listWantedTags.begin(), wtagelement);
		}
		while(data_buffer_ptr != NULL);

		fclose(rfd);
	}

	free(file_ptr);

/*
	for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
	{
		//cout << "\nbIsWanted: " << itrWantedTags->isWanted();
		//cout << "\nTag: " << itrWantedTags->getTag();
		//cout << "\nType: " << itrWantedTags->getIFDType();
	}
*/
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void checkForWebConfig()
{
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char buffer[1024];
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;

	string strWebConfigFile;

	if(iSystemType)
	{
		strWebConfigFile.append(file_sys_dir_ptr);
		strWebConfigFile.append("\\");
		strWebConfigFile.append("d70readerweb.conf");
	}
	else
	{
		strWebConfigFile.append(file_sys_dir_ptr);
		strWebConfigFile.append("/");
		strWebConfigFile.append("d70readerweb.conf");
	}

	if(stat(strWebConfigFile.c_str(), &buf) == (-1))
	{
		if((ofd = fopen(strWebConfigFile.c_str(), "w")) == NULL)
		{
			string strTemp = string("\nFile not created: ") + strWebConfigFile + string("\n");
			ThrowD70Exception(strTemp);
		}

		fputs("D70ReaderSelectionIETitle=D70Reader Gallery Selection", ofd);
		fputs("\nD70ReaderSelectionTitle=D70Reader Gallery Selection", ofd);
		fputs("\nD70ReaderSelectionThumbnailNote=click on thumbnails for gallery index", ofd);
		fputs("\nD70ReaderSelectionGalleryPrefix=Gallery", ofd);
		fputs("\nD70ReaderGalleryIETitle=D70Reader Gallery", ofd);
		fputs("\nD70ReaderGalleryPrefix=Gallery", ofd);
		fputs("\nD70ReaderGalleryBackIndex=Back to Gallery Index", ofd);
		fputs("\nD70ReaderGalleryThumbnailNote=click on thumbnails for full image", ofd);
		fputs("\nD70ReaderSlideshowIETitle=D70Reader Slideshow", ofd);
		fputs("\nD70ReaderSlideshowTitle=D70Reader Image &amp; EXIF Data", ofd);
		fputs("\nD70ReaderSlideshowExifOn=exif on", ofd);
		fputs("\nD70ReaderSlideshowExifOff=exif off", ofd);
		fputs("\nD70ReaderSlideshowPrevious=previous", ofd);
		fputs("\nD70ReaderSlideshowNext=next", ofd);
		fputs("\nD70ReaderSlideshowBackToGallery=Back to Gallery\n", ofd);	//the last one should have a newline

		fclose(ofd);

		addToWebConfig(string("D70ReaderSelectionIETitle=D70Reader Gallery Selection"));
		addToWebConfig(string("D70ReaderSelectionTitle=D70Reader Gallery Selection"));
		addToWebConfig(string("D70ReaderSelectionThumbnailNote=click on thumbnails for gallery index"));
		addToWebConfig(string("D70ReaderSelectionGalleryPrefix=Gallery"));
		addToWebConfig(string("D70ReaderGalleryIETitle=D70Reader Gallery"));
		addToWebConfig(string("D70ReaderGalleryPrefix=Gallery"));
		addToWebConfig(string("D70ReaderGalleryBackIndex=Back to Gallery Index"));
		addToWebConfig(string("D70ReaderGalleryThumbnailNote=click on thumbnails for full image"));
		addToWebConfig(string("D70ReaderSlideshowIETitle=D70Reader Slideshow"));
		addToWebConfig(string("D70ReaderSlideshowTitle=D70Reader Image &amp; EXIF Data"));
		addToWebConfig(string("D70ReaderSlideshowExifOn=exif on"));
		addToWebConfig(string("D70ReaderSlideshowExifOff=exif off"));
		addToWebConfig(string("D70ReaderSlideshowPrevious=previous"));
		addToWebConfig(string("D70ReaderSlideshowNext=next"));
		addToWebConfig(string("D70ReaderSlideshowBackToGallery=Back to Gallery"));

#ifdef __UNIX_ONLY__
		chmod(strWebConfigFile.c_str(), S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
#endif
	}
	else
	{
		if((rfd = fopen(strWebConfigFile.c_str(), "r")) == NULL)
		{
			//cout << "\nFile not opened: " << strWebConfigFile << "\n\n";
			strGlobalCOut.append("\nFile not opened: ");
			strGlobalCOut.append(strWebConfigFile);
			strGlobalCOut.append("\n\n");
			return;
		}

		do
		{
			memset(buffer, (int) '\0', 1024);
			data_buffer_ptr = fgets(buffer, 1024, rfd);

			if(data_buffer_ptr == NULL)
				break;

			string strBuffer = string(buffer);
			strBuffer.resize(strBuffer.length() - 1);

			WebConfig element = WebConfig(strBuffer);
			listWebConfig.insert(listWebConfig.end(), element);
		}
		while(data_buffer_ptr != NULL);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void addToWebConfig(const string& strElement)
{
	WebConfig element = WebConfig(strElement);
	listWebConfig.insert(listWebConfig.end(), element);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
char *upperCase(char *local_ptr)
{
	int ctr1 = 0;
	int length = 0;
	char *temp_ptr = NULL;
	char *final_ptr = NULL;

	temp_ptr = strdup(local_ptr);
	final_ptr = temp_ptr;

	length = strlen(temp_ptr);

	for(ctr1 = 0; ctr1 < length; ctr1++)
	{
		*temp_ptr = toupper(*temp_ptr);
		temp_ptr++;
	}

	return(final_ptr);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputThumbnail(char *filename, FILE *rfd)
{
	char *local_argv[10];
	bool bFirstMakerNote = true;
	FILE *ofd = NULL;
	long position = 0l;
	long lMakerNoteStartOffset = 0l;
	unsigned long size = 0l;
	unsigned long ulOrientation = atol(GlobalOrientationPtr);

	struct exif_header
	{
		unsigned char ff;
		unsigned char soi;
		unsigned char pad;
		unsigned char marker;
		unsigned short length;
		unsigned char exif[6];
		unsigned char byte_order[2];
		unsigned short flag;
		unsigned int offset;
	};

	struct makernote_header
	{
		unsigned char nikon[5];
		unsigned char f1;
		unsigned char f2;
		unsigned char f3;
		unsigned char f4;
		unsigned char f5;
		unsigned char byte_order[2];
		unsigned char flag1;
		unsigned char flag2;
		unsigned char offset1;
		unsigned char offset2;
		unsigned char offset3;
		unsigned char offset4;
	};

	struct exif_header *exif_header_ptr = NULL;
	struct makernote_header *makernote_header_ptr = NULL;

	typedef list<IFDTypeOffset> LISTIFDTYPEOFFSET;
	LISTIFDTYPEOFFSET listIFDTypeOffsets;
	LISTIFDTYPEOFFSET::iterator itrIFDTypeOffsets;

	exif_header_ptr = (struct exif_header *) malloc(sizeof(struct exif_header));
	memset(exif_header_ptr, (int) 0xFF, sizeof(struct exif_header));

	// force this to NEF
	iImageType = 1;

	// force this to write data in MM order
	int old_reverse_byte_order = reverse_byte_order;

#ifdef APPLE
	reverse_byte_order = 0;
#else
	reverse_byte_order = 1;
#endif // APPLE

	exif_header_ptr->ff = 0xFF;
	exif_header_ptr->soi = 0xD8;
	exif_header_ptr->pad = 0xFF;
	exif_header_ptr->marker = 0xE1;
	exif_header_ptr->length = 1110;	// If I leave this as zero, virus scan fails!!!!
	memcpy(exif_header_ptr->exif,"Exif\0\0", 6);

	if(NativeORDER == IntelORDER)
	{
		memcpy(exif_header_ptr->byte_order,"MM", 2);

		exif_header_ptr->flag = 0x2A00;
		exif_header_ptr->offset = 0x08000000;
	}
	else
	{
		memcpy(exif_header_ptr->byte_order,"MM", 2);

		exif_header_ptr->flag = 0x002A;
		exif_header_ptr->offset = 0x00000008;
	}

	makernote_header_ptr = (struct makernote_header *) malloc(sizeof(struct makernote_header));
	memset(makernote_header_ptr, (int) 0xCC, sizeof(struct makernote_header));

	memcpy(makernote_header_ptr->nikon,"Nikon", 5);
	makernote_header_ptr->f1 = 0x00;
	makernote_header_ptr->f2 = 0x02;
	makernote_header_ptr->f3 = 0x10;
	makernote_header_ptr->f4 = 0x00;
	makernote_header_ptr->f5 = 0x00;

/*
	if(NativeORDER != IntelORDER)
	{
		memcpy(makernote_header_ptr->byte_order,"MM", 2);

		makernote_header_ptr->flag1 = 0x00;
		makernote_header_ptr->flag2 = 0x2A;
		makernote_header_ptr->offset1 = 0x00;
		makernote_header_ptr->offset2 = 0x00;
		makernote_header_ptr->offset3 = 0x00;
		makernote_header_ptr->offset4 = 0x08;
	}
	else
	{
		memcpy(makernote_header_ptr->byte_order,"MM", 2);

		makernote_header_ptr->flag1 = 0x2A;
		makernote_header_ptr->flag2 = 0x00;
		makernote_header_ptr->offset1 = 0x08;
		makernote_header_ptr->offset2 = 0x00;
		makernote_header_ptr->offset3 = 0x00;
		makernote_header_ptr->offset4 = 0x00;
	}
*/

	memcpy(makernote_header_ptr->byte_order,"MM", 2);

	makernote_header_ptr->flag1 = 0x00;
	makernote_header_ptr->flag2 = 0x2A;
	makernote_header_ptr->offset1 = 0x00;
	makernote_header_ptr->offset2 = 0x00;
	makernote_header_ptr->offset3 = 0x00;
	makernote_header_ptr->offset4 = 0x08;

	string strFilePath;
	string strFilePath2;
	string strFileName;

	strFilePath.append(getFilePath(filename));
	strFileName.append(getBaseName(filename));

	strFilePath2 = strFilePath;

#ifdef WIN_GUI
	strFilePath += string("d70readerw");
#else
	strFilePath += string("d70reader");
#endif

	bool bFound = isDirectoryFound(strFilePath);

	if(!bFound)
#ifdef WIN_GUI
	{
		_mkdir(strFilePath.c_str());
		pClassPtr->InsertNewDirectory(CString(strFilePath2.c_str()), CString("d70readerw"));
	}
#else
		my_mkdir((char *) strFilePath.c_str());
#endif

	if(iSystemType)
		strFilePath += string("\\thumbnail");
	else
		strFilePath += string("/thumbnail");

	bFound = isDirectoryFound(strFilePath);

	if(!bFound)
#ifdef WIN_GUI
	{
		_mkdir(strFilePath.c_str());
		pClassPtr->InsertNewDirectory(CString(strFilePath2.c_str()) + CString("d70readerw") + CString("\\"), CString("thumbnail"));
	}
#else
		my_mkdir((char *) strFilePath.c_str());
#endif

	if(iSystemType)
		strFilePath += string("\\");
	else
		strFilePath += string("/");

	string strOutputFileName;
	string strFinalFileName;
	string strTempFileName;

	//d70reader -j -od:p:\work\junk\ f:\share\d70\*.nef
	if(bOutputDirectory)
	{
		strTempFileName.append(strOutputDirectory);
		strTempFileName.append(strFileName);
		strTempFileName.append(".tmp.JPG");
		strOutputFileName = strTempFileName;

		strTempFileName.resize(0);
		strTempFileName.append(strOutputDirectory);
		strTempFileName.append(strFileName);
		strTempFileName.append(".JPG");
		strFinalFileName = strTempFileName;
	}
	else
	{
		strTempFileName.append(strFilePath);
		strTempFileName.append(strFileName);
		strTempFileName.append(".tmp.JPG");
		strOutputFileName = strTempFileName;

		strTempFileName.resize(0);
		strTempFileName.append(strFilePath);
		strTempFileName.append(strFileName);
		strTempFileName.append(".JPG");
		strFinalFileName = strTempFileName;
	}

	if((ofd = fopen(strOutputFileName.c_str(), "w+b")) == NULL)
	{
		string strTemp = string("\nFile not created: ") + strOutputFileName + string("\n\nOpen error\n\n");
		ThrowD70Exception(strTemp);
	}

	if((size = fwrite(exif_header_ptr, sizeof(struct exif_header), 1, ofd)) == 0)
	{
		string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n\n");
		ThrowD70Exception(strTemp);
	}

	if(!listIFDElements2.empty())
	{
		for(int ctr1 = 1; ctr1 < (itMakerNotes + 1); ctr1++)
		{
			unsigned short iDirCount = 0;

			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				if((itrIFDElements2->getIFDType() == ctr1) && (itrIFDElements2->getIFDType() != itThumbIFD))
					iDirCount++;
			}

			if(iDirCount)
			{
				long lPosition = ftell(ofd);

				IFDTypeOffset ifdelement = IFDTypeOffset(ctr1, lPosition);
				listIFDTypeOffsets.insert(listIFDTypeOffsets.end(), ifdelement);

				if(ctr1 == itMakerNotes)
				{
					if(bFirstMakerNote)
					{
						bFirstMakerNote = false;

						if((size = fwrite(makernote_header_ptr, sizeof(struct makernote_header), 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[2]: ") + strOutputFileName + string("\n\n");
							ThrowD70Exception(strTemp);
						}
					}
				}

				if(NativeORDER == IntelORDER)
					iDirCount = rev_word( iDirCount );

				if((size = fwrite(&iDirCount, sizeof(unsigned short), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[3]: ") + strOutputFileName + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}

				for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
				{
					if(itrIFDElements2->getIFDType() == ctr1)
					{
						position = ftell(ofd);
						itrIFDElements2->setTagWriteOffset(position);

						if(itrIFDElements2->getIFD().put(ofd) == -1)
						{
							string strTemp = string("\nFile not written[4]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}

					}
				}
			}	// end of iDirCount if
		}	// end of ctr1 loop

		// set major offsets
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			switch(itrIFDElements2->getTag())
			{
				case 0x8769:	// EXIF data tag
					for( itrIFDTypeOffsets = listIFDTypeOffsets.begin(); itrIFDTypeOffsets != listIFDTypeOffsets.end(); itrIFDTypeOffsets++)
					{
						if(itrIFDTypeOffsets->getIFDType() == itEXIF)
						{
							fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

							long lOffset = itrIFDTypeOffsets->getOffset();

							IFDENTRY ifde(	itrIFDElements2->getTag(), 
											itrIFDElements2->getType(),
											itrIFDElements2->getCount(),
											lOffset - fileTypeOffset());
							
							if(ifde.put(ofd) == -1)
							{
								string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
								ThrowD70Exception(strTemp);
							}
							break;
						}
					}
					break;
				case 0x927c:	// MakerNotes data tag
					for( itrIFDTypeOffsets = listIFDTypeOffsets.begin(); itrIFDTypeOffsets != listIFDTypeOffsets.end(); itrIFDTypeOffsets++)
					{
						if(itrIFDTypeOffsets->getIFDType() == itMakerNotes)
						{
							fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

							long lOffset = itrIFDTypeOffsets->getOffset();
							lMakerNoteStartOffset = lOffset;

							IFDENTRY ifde(	itrIFDElements2->getTag(), 
											itrIFDElements2->getType(),
											itrIFDElements2->getCount(),
											lOffset - fileTypeOffset());
							
							if(ifde.put(ofd) == -1)
							{
								string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
								ThrowD70Exception(strTemp);
							}
							break;
						}
					}
					break;
				default:
					break;
			}	// end of switch

		}	// end of offset update loop

		//write all greater than four count data for makernotes
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getIFDType() == itMakerNotes)
			{
				if(!itrIFDElements2->getIsLessThanFourBytes())
				{
					fseek(ofd, 0, SEEK_END);

					long lOffset = ftell(ofd);

					if(itrIFDElements2->getTag() == 0x0004)	//Quality
					{
						int iDataLength = TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount();

						string strQuality = string("BASIC");

						strQuality.resize(iDataLength);

						if((size = fwrite((void *) strQuality.c_str(), iDataLength, 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
					else
					{
						if((size = fwrite((void *) itrIFDElements2->getRawData().c_str(), TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount(), 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}

					fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

					IFDENTRY ifde(	itrIFDElements2->getTag(), 
									itrIFDElements2->getType(),
									itrIFDElements2->getCount(),
									lOffset - lMakerNoteStartOffset - ulMakerNotePreHeaderLength);
					
					if(ifde.put(ofd) == -1)
					{
						string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
						ThrowD70Exception(strTemp);
					}
				}
			}

		}

		fseek(ofd, 0, SEEK_END);
		long lMakerNoteEndOffset = ftell(ofd);

		// update MakerNote length
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getTag() == 0x927c)		//MakerNoteOffset
			{
				fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

				if(itrIFDElements2->getIFDAddress()->get(ofd) == -1)
				{
					string strTemp = string("\nFile not read[6]: ") + strOutputFileName + string("\nRead error\n");
					ThrowD70Exception(strTemp);
				}

				fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

				long litrOffset = itrIFDElements2->getOffset();

				IFDENTRY ifde(	itrIFDElements2->getTag(), 
								itrIFDElements2->getType(),
								lMakerNoteEndOffset - itrIFDElements2->getOffset(),
								itrIFDElements2->getOffset());
				
				if(ifde.put(ofd) == -1)
				{
					string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}

		}	// end of loop

		//write all greater than four count data for itIFD0
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getIFDType() == itIFD0)
			{
				if(!itrIFDElements2->getIsLessThanFourBytes())
				{
					fseek(ofd, 0, SEEK_END);

					long lOffset = ftell(ofd);

					int iDataLength = 0;

					if(itrIFDElements2->getTag() == 0x0131)	//Software
					{
						string strQuality = string("D70Reader - v").append(d70Version);

						iDataLength = strQuality.length() + 1;

						strQuality.resize(iDataLength);

						if((size = fwrite((void *) strQuality.c_str(), iDataLength, 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
					else
					{
						if((size = fwrite((void *) itrIFDElements2->getRawData().c_str(), TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount(), 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}

					fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

					if(itrIFDElements2->getTag() == 0x0131)	//Software
					{
						IFDENTRY ifde(	itrIFDElements2->getTag(), 
										itrIFDElements2->getType(),
										iDataLength,
										lOffset - fileTypeOffset());

						if(ifde.put(ofd) == -1)
						{
							string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
					else
					{
						IFDENTRY ifde(	itrIFDElements2->getTag(), 
										itrIFDElements2->getType(),
										itrIFDElements2->getCount(),
										lOffset - fileTypeOffset());

						if(ifde.put(ofd) == -1)
						{
							string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
				}
			}

		}

		//write all greater than four count data for itEXIF
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getIFDType() == itEXIF)
			{
				if(!itrIFDElements2->getIsLessThanFourBytes())
				{
					fseek(ofd, 0, SEEK_END);

					long lOffset = ftell(ofd);

					if((size = fwrite((void *) itrIFDElements2->getRawData().c_str(), TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount(), 1, ofd)) == 0)
					{
						string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
						ThrowD70Exception(strTemp);
					}

					fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

					IFDENTRY ifde(	itrIFDElements2->getTag(), 
									itrIFDElements2->getType(),
									itrIFDElements2->getCount(),
									lOffset - fileTypeOffset());
					
					if(ifde.put(ofd) == -1)
					{
						string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
						ThrowD70Exception(strTemp);
					}
				}
			}

		}

		// START: calculate and write the length of the exif data:

		fseek(ofd, 0, SEEK_END);

		long lEndOffset = ftell(ofd);
		unsigned short usEndOffset = (unsigned short) lEndOffset - 4;

		if(NativeORDER == IntelORDER)
			usEndOffset = rev_word(usEndOffset);

		fseek(ofd, 4, SEEK_SET);

		if((size = fwrite(&usEndOffset, sizeof(unsigned short), 1, ofd)) == 0)
		{
			string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
			ThrowD70Exception(strTemp);
		}

		// END: calculate and write the length of the exif data:

/*
		// JON JON JON JON

		char padBuffer[8];

		memset(padBuffer, '\0', 8);

		fseek(ofd, 0, SEEK_END);

		if((size = fwrite(padBuffer, 8, 1, ofd)) == 0)
		{
			string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
			ThrowD70Exception(strTemp);
		}

		// JON JON JON JON
*/

		string strModel = string(upperCase((char *) getDataValue(0x110, 1, true).c_str()));

		if((!strModel.compare(0, 10, "NIKON D100")) || (!strModel.compare(0, 9, "NIKON D2H")))
		{
			fseek(rfd, lThumbnailOffset + ulCalcMakerNoteHeaderOffset + 2, 0);
		}
		else
		{
			fseek(rfd, lThumbnailOffset + 2, SEEK_SET);
		}

		string strThumbNail;
		strThumbNail.resize(lThumbnailLength);

		fread((void *) strThumbNail.c_str(), lThumbnailLength - 2, 1, rfd);

		fseek(ofd, 0, SEEK_END);

		if((size = fwrite(strThumbNail.c_str(), lThumbnailLength, 1, ofd)) == 0)
		{
			string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
			ThrowD70Exception(strTemp);
		}

	}	// end of empty list check

	fclose(ofd);

	free(exif_header_ptr);
	free(makernote_header_ptr);

	// don't worry, if it is not there to delete
	if(remove(strFinalFileName.c_str()))
	{
		if(bTrace)
		{
			//printf("\nFile not deleted: %s", strFinalFileName.c_str());
			strGlobalCOut.append("\nFile not deleted: ");
			strGlobalCOut.append(strFinalFileName);

			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\n%s", "Remove error");
			ThrowD70ArgException(string(glb_buffer));
			//perror( "\nRemove error" );
		}
	}

#ifdef ALLOW_ROTATE
	if((ulOrientation == 6) || (ulOrientation == 8))
	{
		local_argv[0] = "jpgtran";
		local_argv[1] = "-rot";

		if(ulOrientation == 6)
			local_argv[2] = "90";
		else
			local_argv[2] = "270";

		local_argv[3] = "-copy";
		local_argv[4] = "all";
		local_argv[5] = (char *) strOutputFileName.c_str();
		local_argv[6] = (char *) strFinalFileName.c_str();
		local_argv[7] = NULL;

		jpegtran(7, local_argv);

		if(remove(strOutputFileName.c_str()))
		{
			//printf("\nFile not deleted: %s", strOutputFileName.c_str());
			strGlobalCOut.append("\nFile not deleted: ");
			strGlobalCOut.append(strOutputFileName);
		}
	}
#endif	//ALLOW_ROTATE

	if((ulOrientation != 6) && (ulOrientation != 8))
	{
		if(rename( strOutputFileName.c_str(), strFinalFileName.c_str()))
		{
			//cout << "\nFailed to rename file:  " << strOutputFileName;
			strGlobalCOut.append("\nFailed to rename file: ");
			strGlobalCOut.append(strOutputFileName);
		}
	}

	reverse_byte_order = old_reverse_byte_order;
	//cout << "\nCreated file: " << strFinalFileName;
	strGlobalCOut.append("\nCreated file: ");
	strGlobalCOut.append(strFinalFileName);
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& DecryptNikonData(string& strModel, string& strDataIn, string& strSerialNumber, string& strShutterCount, unsigned short usCount, unsigned short usOffset)
{
	bool bLocalTrace = false;
	unsigned short usCtr1 = 0;
	unsigned short usKey = 0;
	unsigned long ulSerialNumber = 0l;
	unsigned short usShutterCount = 0;
	unsigned char ci;
	unsigned char cj;
	unsigned char ck;
	unsigned char *pChar = NULL;
	unsigned char *pOffset = NULL;

	static string strDataOut;

	strDataOut.resize(0);

	pOffset = (unsigned char *) strDataIn.c_str();
	pOffset += usOffset;

	pChar = new unsigned char[usCount + 1];
	memset(pChar, (int) '\0', usCount + 1);
	memcpy(pChar, pOffset, usCount - usOffset);

	//string strModel = string(upperCase((char *) getDataValue(0x110, 1, true).c_str()));

/*
	if(!strModel.compare(0, 9, "NIKON D50"))
		ulSerialNumber = 0x22;	// Is nothing standard?!
	else
		ulSerialNumber = (unsigned long) atol(strSerialNumber.c_str());
*/

	if(!strModel.compare(0, 9, "NIKON D50"))
		ulSerialNumber = 0x22;	// Is nothing standard?!
	else if(!strModel.compare(0, 10, "NIKON D200") || !strModel.compare(0, 10, "NIKON D80") || !strModel.compare(0, 10, "NIKON D3"))
	{
		char * pChar = (char *) strSerialNumber.c_str();

		while (*pChar)
		{
			ulSerialNumber = ulSerialNumber * 10 + (isdigit(*pChar) ? *pChar - '0' : *pChar % 10);
			pChar++;
		}
	}
	else
		ulSerialNumber = (unsigned long) atol(strSerialNumber.c_str());

	if(bLocalTrace)
	{
		//printf("\n%d", ulSerialNumber);

		memset(glb_buffer, '\0', GLB_SIZE);
		sprintf(glb_buffer, "%ld", ulSerialNumber);
		strGlobalCOut.append(glb_buffer);
	}

	usShutterCount = (unsigned short) atoi(strShutterCount.c_str());

	for (usCtr1 = 0; usCtr1 < 4; ++usCtr1) 
	{
        usKey ^= (usShutterCount >> (usCtr1 * 8)) & 0xff;
    }

    ci = xlat[0][ulSerialNumber & 0xff];

	if(bLocalTrace)
	{
		//printf("\n%d", ci);
		strGlobalCOut.append("\n");
		memset(glb_buffer, '\0', GLB_SIZE);
		sprintf(glb_buffer, "%c", ci);
		strGlobalCOut.append(glb_buffer);
	}

    cj = xlat[1][usKey];
    ck = 0x60;

    for(usCtr1 = 0; usCtr1 < (usCount - usOffset); usCtr1++) 
	{
        cj = (cj + ci * ck) & 0xff;
        ck = (ck + 1) & 0xff;
        pChar[usCtr1] ^= cj;

		if(bLocalTrace)
		{
			//printf("\n%d - %d", usCtr1, pChar[usCtr1]);
			strGlobalCOut.append("\n");
			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "%d", usCtr1);
			strGlobalCOut.append(glb_buffer);

			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "%c", pChar[usCtr1]);
			strGlobalCOut.append(glb_buffer);
		}
    }

	strDataOut = string((char *) pChar, (usCount - usOffset));

	delete [] pChar;

    return strDataOut;
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
bool isRestoreEXIFTargetFileValid(char *filename)
{
	char *file_name_ptr = NULL;

	// open the file
	FILE *rfd = fopen( filename, "rb" );

	if( rfd == 0 )
		return false;

	// read header and validate
	TIFFHEADER tifh;
	tifh.get( rfd );

	if( tifh.isvalid() )
	{
		// cann't be NEF file
		fclose(rfd);
		return false;
	}

	fclose(rfd);
	return true;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void restoreEXIF(char *filename, char *targetName)
{
	//-re DSC_0001.NEF NOEXIF_DSC_0001.NEF.JPG

	FILE *jfd = NULL;
	struct stat buf;
	char *buf_ptr = NULL;
	long jposition = 0l;
	long db_position = 0l;
	long c0_position = 0l;
	long ljFileSize = 0l;

	bool bFirstMakerNote = true;
	FILE *ofd = NULL;
	long position = 0l;
	long lMakerNoteStartOffset = 0l;
	unsigned long size = 0l;
	//unsigned long ulOrientation = atol(GlobalOrientationPtr);

	struct exif_header
	{
		unsigned char ff;
		unsigned char soi;

		unsigned char f01;
		unsigned char f02;
		unsigned char f03;
		unsigned char f04;
		unsigned char f05;
		unsigned char f06;
		unsigned char f07;
		unsigned char f08;
		unsigned char f09;
		unsigned char f10;
		unsigned char f11;
		unsigned char f12;
		unsigned char f13;
		unsigned char f14;
		unsigned char f15;
		unsigned char f16;
		unsigned char f17;
		unsigned char f18;

		unsigned char pad;
		unsigned char marker;
		unsigned short length;
		unsigned char exif[6];
		unsigned char byte_order[2];
		unsigned char flag1;
		unsigned char flag2;
		//unsigned int offset;
		unsigned char offset1;
		unsigned char offset2;
		unsigned char offset3;
		unsigned char offset4;
	};

	struct makernote_header
	{
		unsigned char nikon[5];
		unsigned char f1;
		unsigned char f2;
		unsigned char f3;
		unsigned char f4;
		unsigned char f5;
		unsigned char byte_order[2];
		unsigned char flag1;
		unsigned char flag2;
		unsigned char offset1;
		unsigned char offset2;
		unsigned char offset3;
		unsigned char offset4;
	};

	struct exif_header *exif_header_ptr = NULL;
	struct makernote_header *makernote_header_ptr = NULL;

	typedef list<IFDTypeOffset> LISTIFDTYPEOFFSET;
	LISTIFDTYPEOFFSET listIFDTypeOffsets;
	LISTIFDTYPEOFFSET::iterator itrIFDTypeOffsets;

	exif_header_ptr = (struct exif_header *) malloc(sizeof(struct exif_header));
	memset(exif_header_ptr, (int) 0xFF, sizeof(struct exif_header));

	if(stat(strTargetEXIFFileName.c_str(), &buf) == (-1))
	{
		memset(glb_buffer, '\0', GLB_SIZE);
		sprintf(glb_buffer, "\n%s", strTargetEXIFFileName.c_str());
		ThrowD70ArgException(string(glb_buffer));
		//perror(strTargetEXIFFileName.c_str());
	}

	ljFileSize = buf.st_size;

	// force this to NEF
	iImageType = 1;

	// force this to write data in MM order
	int old_reverse_byte_order = reverse_byte_order;

#ifdef APPLE
	reverse_byte_order = 0;
#else
	reverse_byte_order = 1;
#endif // APPLE

	//FF D8 FF E0 00 10 4A 46 49 46 00 01 01 01 01 2C 01 2C 00 00

	exif_header_ptr->ff = 0xFF;
	exif_header_ptr->soi = 0xD8;

	exif_header_ptr->f01 = 0xFF;
	exif_header_ptr->f02 = 0xE0;
	exif_header_ptr->f03 = 0x00;
	exif_header_ptr->f04 = 0x10;
	exif_header_ptr->f05 = 0x4A;
	exif_header_ptr->f06 = 0x46;
	exif_header_ptr->f07 = 0x49;
	exif_header_ptr->f08 = 0x46;
	exif_header_ptr->f09 = 0x00;
	exif_header_ptr->f10 = 0x01;
	exif_header_ptr->f11 = 0x01;
	exif_header_ptr->f12 = 0x01;
	exif_header_ptr->f13 = 0x01;
	exif_header_ptr->f14 = 0x2C;
	exif_header_ptr->f15 = 0x01;
	exif_header_ptr->f16 = 0x2C;
	exif_header_ptr->f17 = 0x00;
	exif_header_ptr->f18 = 0x00;

	exif_header_ptr->pad = 0xFF;
	exif_header_ptr->marker = 0xE1;
	exif_header_ptr->length = 1110;	// If I leave this as zero, virus scan fails!!!!
	memcpy(exif_header_ptr->exif,"Exif\0\0", 6);

/*
	if(NativeORDER != IntelORDER)
	{
		memcpy(exif_header_ptr->byte_order,"MM", 2);

		exif_header_ptr->flag1 = 0x00;
		exif_header_ptr->flag2 = 0x2A;
		exif_header_ptr->offset1 = 0x00;
		exif_header_ptr->offset2 = 0x00;
		exif_header_ptr->offset3 = 0x00;
		exif_header_ptr->offset4 = 0x08;
	}
	else
	{
		memcpy(exif_header_ptr->byte_order,"MM", 2);

		exif_header_ptr->flag1 = 0x2A;
		exif_header_ptr->flag2 = 0x00;
		exif_header_ptr->offset1 = 0x08;
		exif_header_ptr->offset2 = 0x00;
		exif_header_ptr->offset3 = 0x00;
		exif_header_ptr->offset4 = 0x00;
	}
*/
	memcpy(exif_header_ptr->byte_order,"MM", 2);

	exif_header_ptr->flag1 = 0x00;
	exif_header_ptr->flag2 = 0x2A;
	exif_header_ptr->offset1 = 0x00;
	exif_header_ptr->offset2 = 0x00;
	exif_header_ptr->offset3 = 0x00;
	exif_header_ptr->offset4 = 0x08;

	makernote_header_ptr = (struct makernote_header *) malloc(sizeof(struct makernote_header));
	memset(makernote_header_ptr, (int) 0xCC, sizeof(struct makernote_header));

	memcpy(makernote_header_ptr->nikon,"Nikon", 5);
	makernote_header_ptr->f1 = 0x00;
	makernote_header_ptr->f2 = 0x02;
	makernote_header_ptr->f3 = 0x10;
	makernote_header_ptr->f4 = 0x00;
	makernote_header_ptr->f5 = 0x00;

/*
	if(NativeORDER != IntelORDER)
	{
		memcpy(makernote_header_ptr->byte_order,"MM", 2);

		makernote_header_ptr->flag1 = 0x00;
		makernote_header_ptr->flag2 = 0x2A;
		makernote_header_ptr->offset1 = 0x00;
		makernote_header_ptr->offset2 = 0x00;
		makernote_header_ptr->offset3 = 0x00;
		makernote_header_ptr->offset4 = 0x08;
	}
	else
	{
		memcpy(makernote_header_ptr->byte_order,"MM", 2);

		makernote_header_ptr->flag1 = 0x2A;
		makernote_header_ptr->flag2 = 0x00;
		makernote_header_ptr->offset1 = 0x08;
		makernote_header_ptr->offset2 = 0x00;
		makernote_header_ptr->offset3 = 0x00;
		makernote_header_ptr->offset4 = 0x00;
	}
*/

	memcpy(makernote_header_ptr->byte_order,"MM", 2);

	makernote_header_ptr->flag1 = 0x00;
	makernote_header_ptr->flag2 = 0x2A;
	makernote_header_ptr->offset1 = 0x00;
	makernote_header_ptr->offset2 = 0x00;
	makernote_header_ptr->offset3 = 0x00;
	makernote_header_ptr->offset4 = 0x08;

	string strFilePath;
	string strFilePath2;
	string strFileName;

	strFilePath.append(getFilePath((char *) strTargetEXIFFileName.c_str()));
	strFileName.append(getFileName((char *) strTargetEXIFFileName.c_str()));

	strFilePath2 = strFilePath;

	if(!targetName)
	{
#ifdef WIN_GUI
		strFilePath += string("d70readerw");
#else
		strFilePath += string("d70reader");
#endif

		bool bFound = isDirectoryFound(strFilePath);

		if(!bFound)
#ifdef WIN_GUI
		{
			_mkdir(strFilePath.c_str());
			pClassPtr->InsertNewDirectory(CString(strFilePath2.c_str()), CString("d70readerw"));
		}
#else
			my_mkdir((char *) strFilePath.c_str());
#endif

		if(iSystemType)
			strFilePath += string("\\restore_exif");
		else
			strFilePath += string("/restore_exif");

		bFound = isDirectoryFound(strFilePath);

		if(!bFound)
#ifdef WIN_GUI
		{
			_mkdir(strFilePath.c_str());
			pClassPtr->InsertNewDirectory(CString(strFilePath2.c_str()) + CString("d70readerw") + CString("\\"), CString("restore_exif"));
		}
#else
			my_mkdir((char *) strFilePath.c_str());
#endif

		if(iSystemType)
			strFilePath += string("\\");
		else
			strFilePath += string("/");
	}

	string strOutputFileName;
	string strFinalFileName;
	string strTempFileName;

	//d70reader -j -od:p:\work\junk\ f:\share\d70\*.nef
	if(bOutputDirectory)
	{
		strTempFileName.append(strOutputDirectory);
		strTempFileName.append(strFileName);
		strTempFileName.append(".tmp.JPG");
		strOutputFileName = strTempFileName;

		strTempFileName.resize(0);
		strTempFileName.append(strOutputDirectory);
		strTempFileName.append(strFileName);
		//strTempFileName.append(".EXIF.JPG");
		strFinalFileName = strTempFileName;
	}
	else
	{
		strTempFileName.append(strFilePath);
		strTempFileName.append(strFileName);
		strTempFileName.append(".tmp.JPG");
		strOutputFileName = strTempFileName;

		strTempFileName.resize(0);
		strTempFileName.append(strFilePath);
		strTempFileName.append(strFileName);
		//strTempFileName.append(".EXIF.JPG");
		strFinalFileName = strTempFileName;
	}

	if(targetName != NULL)
	{
		if((ofd = fopen(targetName, "w+b")) == NULL)
		{
			string strTemp = string("\nFile not created: ") + targetName + string("\nOpen error\n");
			ThrowD70Exception(strTemp);
		}
	}
	else
	{
		if((ofd = fopen(strOutputFileName.c_str(), "w+b")) == NULL)
		{
			string strTemp = string("\nFile not created: ") + strOutputFileName + string("\nOpen error\n");
			ThrowD70Exception(strTemp);
		}
	}

	if((size = fwrite(exif_header_ptr, sizeof(struct exif_header), 1, ofd)) == 0)
	{
		string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
		ThrowD70Exception(strTemp);
	}

	if(!listIFDElements2.empty())
	{
		for(int ctr1 = 1; ctr1 < (itMakerNotes + 1); ctr1++)
		{
			unsigned short iDirCount = 0;

			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				if((itrIFDElements2->getIFDType() == ctr1) && (itrIFDElements2->getIFDType() != itThumbIFD))
					iDirCount++;
			}

			if(iDirCount)
			{
				long lPosition = ftell(ofd);

				IFDTypeOffset ifdelement = IFDTypeOffset(ctr1, lPosition);
				listIFDTypeOffsets.insert(listIFDTypeOffsets.end(), ifdelement);

				if(ctr1 == itMakerNotes)
				{
					if(bFirstMakerNote)
					{
						bFirstMakerNote = false;

						if((size = fwrite(makernote_header_ptr, sizeof(struct makernote_header), 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[2]: ") + strOutputFileName + string("\n\n");
							ThrowD70Exception(strTemp);
						}
					}
				}

				if(NativeORDER == IntelORDER)
					iDirCount = rev_word( iDirCount );

				if((size = fwrite(&iDirCount, sizeof(unsigned short), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[3]: ") + strOutputFileName + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}

				for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
				{
					if(itrIFDElements2->getIFDType() == ctr1)
					{
						position = ftell(ofd);
						itrIFDElements2->setTagWriteOffset(position);

						if(itrIFDElements2->getIFD().put(ofd) == -1)
						{
							string strTemp = string("\nFile not written[4]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}

					}
				}
			}	// end of iDirCount if
		}	// end of ctr1 loop

		// set major offsets
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			switch(itrIFDElements2->getTag())
			{
				case 0x8769:
					for( itrIFDTypeOffsets = listIFDTypeOffsets.begin(); itrIFDTypeOffsets != listIFDTypeOffsets.end(); itrIFDTypeOffsets++)
					{
						if(itrIFDTypeOffsets->getIFDType() == itEXIF)
						{
							fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

							long lOffset = itrIFDTypeOffsets->getOffset();

							IFDENTRY ifde(	itrIFDElements2->getTag(), 
											itrIFDElements2->getType(),
											itrIFDElements2->getCount(),
											lOffset - fileTypeOffset() - 0x12);
							
							if(ifde.put(ofd) == -1)
							{
								string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
								ThrowD70Exception(strTemp);
							}
							break;
						}
					}
					break;
				case 0x927c:
					for( itrIFDTypeOffsets = listIFDTypeOffsets.begin(); itrIFDTypeOffsets != listIFDTypeOffsets.end(); itrIFDTypeOffsets++)
					{
						if(itrIFDTypeOffsets->getIFDType() == itMakerNotes)
						{
							fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

							long lOffset = itrIFDTypeOffsets->getOffset();
							lMakerNoteStartOffset = lOffset;

							IFDENTRY ifde(	itrIFDElements2->getTag(), 
											itrIFDElements2->getType(),
											itrIFDElements2->getCount(),
											lOffset - fileTypeOffset() - 0x12);
							
							if(ifde.put(ofd) == -1)
							{
								string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
								ThrowD70Exception(strTemp);
							}
							break;
						}
					}
					break;
				default:
					break;
			}	// end of switch

		}	// end of offset update loop

		//write all greater than four count data for makernotes
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getIFDType() == itMakerNotes)
			{
				if(!itrIFDElements2->getIsLessThanFourBytes())
				{
					fseek(ofd, 0, SEEK_END);

					long lOffset = ftell(ofd);

					if(itrIFDElements2->getTag() == 0x0004)	//Quality
					{
						int iDataLength = TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount();

						string strQuality;

						if(targetName != NULL)
						{
							strQuality = string("FINE");
						}
						else
						{
							strQuality = string("BASIC");
						}

						strQuality.resize(iDataLength);

						if((size = fwrite((void *) strQuality.c_str(), iDataLength, 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
					else
					{
						if((size = fwrite((void *) itrIFDElements2->getRawData().c_str(), TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount(), 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}

					fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

					IFDENTRY ifde(	itrIFDElements2->getTag(), 
									itrIFDElements2->getType(),
									itrIFDElements2->getCount(),
									lOffset - lMakerNoteStartOffset - ulMakerNotePreHeaderLength);
					
					if(ifde.put(ofd) == -1)
					{
						string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
						ThrowD70Exception(strTemp);
					}
				}
			}

		}

		fseek(ofd, 0, SEEK_END);
		long lMakerNoteEndOffset = ftell(ofd);

		// update MakerNote length
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getTag() == 0x927c)		//MakerNoteOffset
			{
				fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

				if(itrIFDElements2->getIFDAddress()->get(ofd) == -1)
				{
					string strTemp = string("\nFile not read[6]: ") + strOutputFileName + string("\nRead error\n");
					ThrowD70Exception(strTemp);
				}

				fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

				long litrOffset = itrIFDElements2->getOffset();

				IFDENTRY ifde(	itrIFDElements2->getTag(), 
								itrIFDElements2->getType(),
								lMakerNoteEndOffset - itrIFDElements2->getOffset(),
								itrIFDElements2->getOffset());
				
				if(ifde.put(ofd) == -1)
				{
					string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}

		}	// end of loop

		//write all greater than four count data for itIFD0
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getIFDType() == itIFD0)
			{
				if(!itrIFDElements2->getIsLessThanFourBytes())
				{
					fseek(ofd, 0, SEEK_END);

					long lOffset = ftell(ofd);

					int iDataLength = 0;

					if(itrIFDElements2->getTag() == 0x0131)	//Software
					{
						string strQuality = string("D70Reader - v").append(d70Version);

						iDataLength = strQuality.length() + 1;

						strQuality.resize(iDataLength);

						if((size = fwrite((void *) strQuality.c_str(), iDataLength, 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
					else
					{
						if((size = fwrite((void *) itrIFDElements2->getRawData().c_str(), TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount(), 1, ofd)) == 0)
						{
							string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}

					fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

					if(itrIFDElements2->getTag() == 0x0131)	//Software
					{
						IFDENTRY ifde(	itrIFDElements2->getTag(), 
										itrIFDElements2->getType(),
										iDataLength,
										lOffset - fileTypeOffset() - 0x12);

						if(ifde.put(ofd) == -1)
						{
							string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
					else
					{
						IFDENTRY ifde(	itrIFDElements2->getTag(), 
										itrIFDElements2->getType(),
										itrIFDElements2->getCount(),
										lOffset - fileTypeOffset() - 0x12);

						if(ifde.put(ofd) == -1)
						{
							string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
							ThrowD70Exception(strTemp);
						}
					}
				}
			}

		}

		//write all greater than four count data for itEXIF
		for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
		{
			if(itrIFDElements2->getIFDType() == itEXIF)
			{
				if(!itrIFDElements2->getIsLessThanFourBytes())
				{
					fseek(ofd, 0, SEEK_END);

					long lOffset = ftell(ofd);

					if((size = fwrite((void *) itrIFDElements2->getRawData().c_str(), TiffDataTypeSize(itrIFDElements2->getType()) * itrIFDElements2->getCount(), 1, ofd)) == 0)
					{
						string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
						ThrowD70Exception(strTemp);
					}

					fseek(ofd, itrIFDElements2->getTagWriteOffset(), SEEK_SET);

					IFDENTRY ifde(	itrIFDElements2->getTag(), 
									itrIFDElements2->getType(),
									itrIFDElements2->getCount(),
									lOffset - fileTypeOffset() - 0x12);
					
					if(ifde.put(ofd) == -1)
					{
						string strTemp = string("\nFile not written[6]: ") + strOutputFileName + string("\nWrite error\n");
						ThrowD70Exception(strTemp);
					}
				}
			}

		}

		fseek(ofd, 0, SEEK_END);

		long lEndOffset = ftell(ofd);
		long lTmp;
		lTmp = 0x16;
		unsigned short usEndOffset = (unsigned short) lEndOffset - lTmp;

		if(NativeORDER == IntelORDER)
			usEndOffset = rev_word(usEndOffset);

		fseek(ofd, 0x16, SEEK_SET);

		if((size = fwrite(&usEndOffset, sizeof(unsigned short), 1, ofd)) == 0)
		{
			string strTemp = string("\nFile not written[1]: ") + strOutputFileName + string("\nWrite error\n");
			ThrowD70Exception(strTemp);
		}

		if((jfd = fopen(strTargetEXIFFileName.c_str(), "rb")) == NULL)
		{
			//printf("\nFile not opened: %s\n\n", strTargetEXIFFileName.c_str());
			strGlobalCOut.append("\nFile not opened: ");
			strGlobalCOut.append(strTargetEXIFFileName);
			strGlobalCOut.append("\n\n");
			return;
		}

		db_position = getMarkerOffset(jfd, 0x00db);
		c0_position = getMarkerOffset(jfd, 0x00c0);

		if((c0_position != 0) && (c0_position < db_position))
			jposition = c0_position;
		else
			jposition = db_position;

		if(jposition)
		{
			fseek(jfd, jposition, 0);

			buf_ptr = (char *) malloc((ljFileSize - jposition + 1));
			memset(buf_ptr, (int) '\0', (ljFileSize - jposition + 1));
			fread(buf_ptr, (ljFileSize - jposition), 1, jfd);

			fseek(ofd, 0, SEEK_END);

			if((size = fwrite(buf_ptr, (ljFileSize - jposition), 1, ofd)) == 0)
			{
				//printf("\nFailed to write to file: %s", strOutputFileName.c_str());
				strGlobalCOut.append("\nFailed to write to file: ");
				strGlobalCOut.append(strOutputFileName);
				free(buf_ptr);
				return;
			}

			free(buf_ptr);
		}

		fclose(jfd);
	}	// end of empty list check

	fclose(ofd);

	free(exif_header_ptr);
	free(makernote_header_ptr);

	// don't worry, if it is not there to delete
	if(remove(strFinalFileName.c_str()))
	{
		if(bTrace)
		{
			//printf("\nFile not deleted: %s", strFinalFileName.c_str());
			strGlobalCOut.append("\nFile not deleted: ");
			strGlobalCOut.append(strFinalFileName);

			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\n%s", "Remove error");
			ThrowD70ArgException(string(glb_buffer));
			//perror( "\nRemove error" );
		}
	}

//	if((ulOrientation != 6) && (ulOrientation != 8))
//	{
		if(targetName == NULL)
		{
			if(rename( strOutputFileName.c_str(), strFinalFileName.c_str()))
			{
				//cout << "\nFailed to rename file:  " << strOutputFileName;
				strGlobalCOut.append("\nFailed to rename file: ");
				strGlobalCOut.append(strOutputFileName);
			}
		}
//	}

	reverse_byte_order = old_reverse_byte_order;

	if(targetName == NULL)
	{
		//cout << "\nCreated file: " << strFinalFileName;
		strGlobalCOut.append("\nCreated file: ");
		strGlobalCOut.append(strFinalFileName);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& getLocalTime()
{
	static string strDate;
	char buffer[32];
	struct tm *newtime;
	time_t aclock;

	time( &aclock );                 // Get time in seconds

	newtime = localtime( &aclock );  // Convert time to struct tm form

/*
struct tm {
        int tm_sec;     // seconds after the minute - [0,59] 
        int tm_min;     // minutes after the hour - [0,59] 
        int tm_hour;    // hours since midnight - [0,23] 
        int tm_mday;    // day of the month - [1,31] 
        int tm_mon;     // months since January - [0,11] 
        int tm_year;    // years since 1900 
        int tm_wday;    // days since Sunday - [0,6] 
        int tm_yday;    // days since January 1 - [0,365] 
        int tm_isdst;   // daylight savings time flag 
        };

ModifyDate=2004:11:23 16:41:22
DateTimeOriginal=2004:11:21 16:57:05
DateTimeDigitized=2004:11:21 16:57:05

*/

	memset(buffer, (int) '\0', 32);

	sprintf(buffer, "%4d:%02d:%02d %02d:%02d:%02d", 
					(newtime->tm_year + 1900), 
					(newtime->tm_mon + 1), 
					newtime->tm_mday,
					newtime->tm_hour,
					newtime->tm_min,
					newtime->tm_sec);

	strDate.resize(0);

	strDate = string(buffer);

	return strDate;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void checkForDateConfig()
{
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char buffer[1024];
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;

	string strWebConfigFile;

	string strLocalTime;

	strLocalTime = getLocalTime();

	if(iSystemType)
	{
		strWebConfigFile.append(file_sys_dir_ptr);
		strWebConfigFile.append("\\");
		strWebConfigFile.append("d70readerdate.conf");
	}
	else
	{
		strWebConfigFile.append(file_sys_dir_ptr);
		strWebConfigFile.append("/");
		strWebConfigFile.append("d70readerdate.conf");
	}

	if(stat(strWebConfigFile.c_str(), &buf) == (-1))
	{
		if((ofd = fopen(strWebConfigFile.c_str(), "w")) == NULL)
		{
			string strTemp = string("\nFile not created: ") + strWebConfigFile + string("\n\n");
			ThrowD70Exception(strTemp);
		}

		string strTempTime;

		strTempTime = string("ModifyDate=") + strLocalTime + string("\n");
		fputs(strTempTime.c_str(), ofd);

		strTempTime = string("DateTimeOriginal=") + strLocalTime + string("\n");
		fputs(strTempTime.c_str(), ofd);
		
		strTempTime = string("DateTimeDigitized=") + strLocalTime + string("\n");
		fputs(strTempTime.c_str(), ofd);

		fclose(ofd);

		strModifyDate = strLocalTime;
		strDateTimeOriginal = strLocalTime;
		strDateTimeDigitized = strLocalTime;


#ifdef __UNIX_ONLY__
		chmod(strWebConfigFile.c_str(), S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
#endif
	}
	else
	{
		if((rfd = fopen(strWebConfigFile.c_str(), "r")) == NULL)
		{
			//cout << "\nFile not opened: " << strWebConfigFile << "\n\n";
			strGlobalCOut.append("\nFile not opened: ");
			strGlobalCOut.append(strWebConfigFile);
			strGlobalCOut.append("\n\n");
			return;
		}

		do
		{
			memset(buffer, (int) '\0', 1024);
			data_buffer_ptr = fgets(buffer, 1024, rfd);

			if(data_buffer_ptr == NULL)
				break;

			string strBuffer = string(buffer);
			strBuffer.resize(strBuffer.length() - 1);

			if(!strBuffer.compare(0, 11, "ModifyDate="))
				strModifyDate = strBuffer.substr((strBuffer.find_first_of("=", 0) + 1),19);
			else if(!strBuffer.compare(0, 17, "DateTimeOriginal="))
				strDateTimeOriginal = strBuffer.substr((strBuffer.find_first_of("=", 0) + 1),19);
			else if(!strBuffer.compare(0, 18, "DateTimeDigitized="))
				strDateTimeDigitized = strBuffer.substr((strBuffer.find_first_of("=", 0) + 1),19);
		}
		while(data_buffer_ptr != NULL);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void fixDate(char *filename, FILE *rfd)
{
	bool bLocalTrace = false;
	FILE *ofd = NULL;
	unsigned long size = 0l;

	string strTempFile;
	string strFilePath;
	string strFileName;

	strFilePath.append(getFilePath(filename));
	strFileName.append(getFileName(filename));

	//d70reader -fd -od:p:\work\junk\ f:\share\d70\*.jpg
	if(bOutputDirectory)
		strTempFile = strOutputDirectory;
	else
		strTempFile = strFilePath;

#ifdef WIN_GUI
	strTempFile += string("d70readerw");
#else
	strTempFile += string("d70reader");
#endif

	bool bFound = isDirectoryFound(strTempFile);

	if(!bFound)
#ifdef WIN_GUI
	{
		_mkdir(strTempFile.c_str());
		pClassPtr->InsertNewDirectory(CString(strFilePath.c_str()), CString("d70readerw"));
	}
#else
		my_mkdir((char *) strTempFile.c_str());
#endif

	if(iSystemType)
		strTempFile += string("\\fix_date");
	else
		strTempFile += string("/fix_date");

	bFound = isDirectoryFound(strTempFile);

	if(!bFound)
#ifdef WIN_GUI
	{
		_mkdir(strTempFile.c_str());
		pClassPtr->InsertNewDirectory(CString(strFilePath.c_str()) + CString("d70readerw") + CString("\\"), CString("fix_date"));
	}
#else
		my_mkdir((char *) strTempFile.c_str());
#endif

	if(iSystemType)
		strTempFile += string("\\");
	else
		strTempFile += string("/");
	
	strTempFile.append(strFileName);

	if((ofd = fopen(strTempFile.c_str(), "w+b")) == NULL)
	{
		string strTemp = string("\nFile not created: ") + strTempFile + string("\nOpen error\n");
		ThrowD70Exception(strTemp);
	}

	fseek( rfd, 0l, SEEK_SET );

	unsigned char *pChar = new unsigned char[lFileSize];

	fread((void *) pChar, lFileSize, 1, rfd);

	if((size = fwrite((void *) pChar, lFileSize, 1, ofd)) == 0)
	{
		string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
		ThrowD70Exception(strTemp);
	}

	delete pChar;

	//start of ModifyDate
	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x132)
			{
				if(bLocalTrace)
				{
					//cout << "\nOffset: " << itrIFDElements->getValueWriteOffset();
					strGlobalCOut.append("\nOffset: ");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", itrIFDElements->getValueWriteOffset());
					strGlobalCOut.append(glb_buffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				string strBuffer;

				strBuffer.resize(itrIFDElements->getIFD().countval());

				fread((void *) strBuffer.c_str(), itrIFDElements->getIFD().countval(), 1, ofd);

				if(bLocalTrace)
				{
					//cout << "\nstrBuffer: " << strBuffer;
					strGlobalCOut.append("\nstrBuffer: ");
					strGlobalCOut.append(strBuffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				if((size = fwrite((void *) strModifyDate.c_str(), itrIFDElements->getIFD().countval(), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}
		}
	}
	//end of ModifyDate

	//start of DateTimeOriginal
	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x9003)
			{
				if(bLocalTrace)
				{
					//cout << "\nOffset: " << itrIFDElements->getValueWriteOffset();
					strGlobalCOut.append("\nOffset: ");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", itrIFDElements->getValueWriteOffset());
					strGlobalCOut.append(glb_buffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				string strBuffer;

				strBuffer.resize(itrIFDElements->getIFD().countval());

				fread((void *) strBuffer.c_str(), itrIFDElements->getIFD().countval(), 1, ofd);

				if(bLocalTrace)
				{
					//cout << "\nstrBuffer: " << strBuffer;
					strGlobalCOut.append("\nstrBuffer: ");
					strGlobalCOut.append(strBuffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				if((size = fwrite((void *) strDateTimeOriginal.c_str(), itrIFDElements->getIFD().countval(), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}
		}
	}
	//end of DateTimeOriginal

	//start of DateTimeDigitized
	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x9004)
			{
				if(bLocalTrace)
				{
					//cout << "\nOffset: " << itrIFDElements->getValueWriteOffset();
					strGlobalCOut.append("\nOffset: ");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", itrIFDElements->getValueWriteOffset());
					strGlobalCOut.append(glb_buffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				string strBuffer;

				strBuffer.resize(itrIFDElements->getIFD().countval());

				fread((void *) strBuffer.c_str(), itrIFDElements->getIFD().countval(), 1, ofd);

				if(bLocalTrace)
				{
					//cout << "\nstrBuffer: " << strBuffer;
					strGlobalCOut.append("\nstrBuffer: ");
					strGlobalCOut.append(strBuffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				if((size = fwrite((void *) strDateTimeDigitized.c_str(), itrIFDElements->getIFD().countval(), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}
		}
	}
	//end of DateTimeDigitized

	fclose(ofd);

	struct utimbuf utimeb;
	struct stat buf;
	int result;
	result = stat( filename, &buf );

	utimeb.actime = buf.st_atime;
	utimeb.modtime = buf.st_mtime;

	utime(strTempFile.c_str(), &utimeb);

	//cout << "\nCreated File: " << strTempFile;
	strGlobalCOut.append("\nCreated File: ");
	strGlobalCOut.append(strTempFile);

	if(bLocalTrace)
	{
		//cout << "\nstrFilePath: " << strFilePath;
		strGlobalCOut.append("\nstrFilePath: ");
		strGlobalCOut.append(strFilePath);

		//cout << "\nstrFileName: " << strFileName;
		strGlobalCOut.append("\nstrFileName: ");
		strGlobalCOut.append(strFileName);

		//cout << "\nstrTempFile: " << strTempFile;
		strGlobalCOut.append("\nstrTempFile: ");
		strGlobalCOut.append(strTempFile);

		//cout << '\n' << strModifyDate;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strModifyDate);

		//cout << '\n' << strDateTimeOriginal;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strDateTimeOriginal);

		//cout << '\n' << strDateTimeDigitized;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strDateTimeDigitized);

		//cout << "\n\nLocal Date: " << getLocalTime() << '\n';
		strGlobalCOut.append("\n\nLocal Date: ");
		strGlobalCOut.append(getLocalTime());
		strGlobalCOut.append("\n");
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
IFDElement& getIFDObject(unsigned short usTag, bool bMainList)
{
	IFDElement *ifde = NULL;

	if(bMainList)
	{
		if(!listIFDElements.empty())
		{
			for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
			{
				if(itrIFDElements->getTag() == usTag)
				{
					return *itrIFDElements;
					break;
				}
			}
		}
	}
	else
	{
		if(!listIFDElements2.empty())
		{
			for( itrIFDElements2 = listIFDElements2.begin(); itrIFDElements2 != listIFDElements2.end(); itrIFDElements2++)
			{
				if(itrIFDElements2->getTag() == usTag)
				{
					return *itrIFDElements2;
					break;
				}
			}
		}
	}

	return *ifde;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputJpegFine(char *filename)
{
	char *local_argv[10];
	unsigned long ulOrientation = atol(GlobalOrientationPtr);

	string strTempFile;
	string strFilePath;
	string strFilePath2;
	string strFileName;
	string strBaseName;

	strFilePath.append(getFilePath(filename));
	strFilePath2.append(getFilePath(filename));
	strFileName.append(getFileName(filename));
	strBaseName.append(getBaseName(filename));

	//cout << "\nProcessing File: " << filename;
	//cout << "\nPlease Wait.....";
	strGlobalCOut.append("\nPlease Wait.....");

	fflush(stdout);

	//d70reader -njf -od:p:\work\junk\ f:\share\d70\*.nef
	if(bOutputDirectory)
		strTempFile = strOutputDirectory;
	else
		strTempFile = strFilePath;

#ifdef WIN_GUI
	strTempFile += string("d70readerw");
#else
	strTempFile += string("d70reader");
#endif

	bool bFound = isDirectoryFound(strTempFile);

	if(!bFound)
#ifdef WIN_GUI
	{
		_mkdir(strTempFile.c_str());
		pClassPtr->InsertNewDirectory(CString(strFilePath2.c_str()), CString("d70readerw"));
	}
#else
		my_mkdir((char *) strTempFile.c_str());
#endif

	if(iSystemType)
		strTempFile += string("\\jpeg_fine");
	else
		strTempFile += string("/jpeg_fine");

	bFound = isDirectoryFound(strTempFile);

	if(!bFound)
#ifdef WIN_GUI
	{
		_mkdir(strTempFile.c_str());
		pClassPtr->InsertNewDirectory(CString(strFilePath2.c_str()) + CString("d70readerw") + CString("\\"), CString("jpeg_fine"));
	}
#else
		my_mkdir((char *) strTempFile.c_str());
#endif

	if(iSystemType)
		strTempFile.append("\\");
	else
		strTempFile.append("/");

	string strPPMName;
	strPPMName = strTempFile;
	strPPMName.append(strBaseName);
	strPPMName.append(".PPM");

	string strJpegName;
	strJpegName = strTempFile;
	strJpegName.append(strBaseName);
	strJpegName.append(".JPG");

	string strJpegTmpName;
	strJpegTmpName = strTempFile;
	strJpegTmpName.append(strBaseName);
	strJpegTmpName.append(".tmp.JPG");

	int iFlip = 0;

	if((ulOrientation == 6) || (ulOrientation == 8))
	{
		if(ulOrientation == 6)
			iFlip = 6;
		else
			iFlip = 5;
	}

	dcrawMain(filename, (char *) strPPMName.c_str(), iFlip, bMainDoNotClipColor, bMainUseCameraWB, bMainUseAutoWB, bMainUseCameraRGB, fMainBright, fMainRed, fMainBlue, iMainBlack, false);

	local_argv[0] = "cjpeg";
	local_argv[1] = (char *) strPPMName.c_str();
	local_argv[2] = (char *) strJpegTmpName.c_str();
	local_argv[3] = NULL;

	cjpegMain(3, local_argv);

	strTargetEXIFFileName = string(strJpegTmpName);

	restoreEXIF(filename, (char *) strJpegName.c_str());

	//cout << "\nCreated file: " << strJpegName;
	strGlobalCOut.append("\nCreated file: ");
	strGlobalCOut.append(strJpegName);

	// don't worry, if it is not there to delete
	if(remove(strJpegTmpName.c_str()))
	{
		if(bTrace)
		{
			//printf("\nFile not deleted: %s", strJpegTmpName.c_str());
			strGlobalCOut.append("\nFile not deleted: ");
			strGlobalCOut.append(strJpegTmpName);

			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\n%s", "Remove error");
			ThrowD70ArgException(string(glb_buffer));
			//perror( "\nRemove error" );
		}
	}

	// don't worry, if it is not there to delete
	if(remove(strPPMName.c_str()))
	{
		if(bTrace)
		{
			//printf("\nFile not deleted: %s", strPPMName.c_str());
			strGlobalCOut.append("\nFile not deleted: ");
			strGlobalCOut.append(strPPMName);

			memset(glb_buffer, '\0', GLB_SIZE);
			sprintf(glb_buffer, "\n%s", "Remove error");
			ThrowD70ArgException(string(glb_buffer));
			//perror( "\nRemove error" );
		}
	}
}



//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
string& IdentifyLensString(unsigned char byteB,
						   unsigned char byteC,
						   unsigned char byteD,
						   unsigned char byteE,
						   unsigned char byteF,
						   unsigned char byte10,
						   unsigned char byte11,
						   unsigned char lensType)
{
	static string strLensID;
	strLensID.resize(0);

	bool idmatch = false;
	bool ltypematch = false;

	int lensentry = 0;
	int foundlens = 0;

	for(; fmountlens[lensentry].lensname != NULL; lensentry++)
	{
        if((byteB == fmountlens[lensentry].lid)  &&
           (byteC == fmountlens[lensentry].stps) &&
           (byteD == fmountlens[lensentry].focs) &&
           (byteE == fmountlens[lensentry].focl) &&
           (byteF == fmountlens[lensentry].aps)  &&
           (byte10 == fmountlens[lensentry].apl) &&
           (byte11 == fmountlens[lensentry].lfw))
           {
				idmatch = true;
				foundlens = lensentry;

				if(lensType == fmountlens[lensentry].ltype)
					ltypematch = true;
           }
	}

	if(idmatch && ltypematch)
	{
		string manu=  fmountlens[foundlens].manuf;
		string lname= fmountlens[foundlens].lensname;

		strLensID = manu + string(" ") + lname;
	}   
	else
		strLensID = string("not identified");

	return strLensID;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& getTagValue(unsigned short usTag, int iType)
{
	bool bNotFound = true;
	static string strValue;
	strValue.resize(0);

	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if((itrIFDElements->getTag() == usTag) && (itrIFDElements->getIFDType() == iType))
			{
				bNotFound = false;
				strValue = itrIFDElements->getData();
				break;
			}
		}
	}

	if(!strValue.compare(string("e")))
		strValue = string("14");
	else if(!strValue.compare(string("a")))
		strValue = string("10");

	string strTemp = strValue;

	return strValue;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void QuickSortCString(vector<string>& vString, int left, int right, string& temp)
{
	if(left < right)
	{
		string pivot = vString[left];
		const int nSize = vString.size();

		int i = left, j = right + 1;

		do 
		{
			do i++;
			while(i < nSize && vString[i].compare(pivot) < 0);

			do j--;
			while(j < nSize && vString[j].compare(pivot) > 0);

			if(i < j)
			{
				temp=vString[i];
				vString[i]=vString[j];
				vString[j]=temp;
			}
		}
		while(i<j);

		temp=vString[left];
		vString[left]=vString[j];
		vString[j]=temp;

		QuickSortCString(vString, left, j-1, temp);
		QuickSortCString(vString, j+1, right, temp);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void getDOFData()
{
	int iUnits = 3;
	double HyperfocalDistance = 0.0f;
	//double dist = 0.0f;
	double nearFocus = 0.0f;
	double farFocus = 0.0f;
	double dFocalLength = 0.0f;
	bool bInfinity = false;
	double fstop = 0.0f;
	double units = 0.0f;
	double circleOfConfusionDiameter = 0.0203;
	char buffer[256];

	if(!dFocusDistance)
		return;

	//string strDistance = getDataValueRaw(0xFFFA, 0, true);
	//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", dFocusDistance, dFocusDistance/2.54, dFocusDistance/2.54/12);
	string strMake = getDataValue(0x10f, 1, true);
	string strFocalLength = getDataValueRaw(0x920A, 6, true);
	string strFStop = getDataValueRaw(0x829D, 6, true);

	dFocalLength = atof(strFocalLength.c_str());
	fstop = atof(strFStop.c_str());

	if(!dFocalLength)
		return;

	if(!fstop)
		return;

	if(!bNewCoC)
		strCicleOfConfusion = string("0.0203");
	else
		circleOfConfusionDiameter = atof(strCicleOfConfusion.c_str());

	if(iUnits == 0)		// feet
		units = 0.3048f;
	else if(iUnits == 1)	// inches
		units = 0.0254f;
	else if(iUnits == 2)	// meters
		units = 1.0f;
	else if(iUnits == 3)	// centimeters
		units = 0.01f;

	HyperfocalDistance = (dFocalLength * dFocalLength) / (fstop * circleOfConfusionDiameter) + dFocalLength;

	nearFocus = ((HyperfocalDistance - dFocalLength) * (dFocusDistance * 10.0)) / (HyperfocalDistance + (dFocusDistance * 10.0) - (2 * dFocalLength));

	// preventing division by zero when calculating far distance
	// when focusing at hyperfocal distance or beyond
	// preventing too large numbers or overflow by using
	// 0.00001 for minimum (hyperfocal - distance)
	if( (HyperfocalDistance - (dFocusDistance * 10.0)) <= 0.00001)
	{
		farFocus = 10000000.0; // set infinity at 10000m
		bInfinity = true;
	}
	else
	{
		farFocus = ((HyperfocalDistance - dFocalLength) * (dFocusDistance * 10.0)) / (HyperfocalDistance - (dFocusDistance * 10.0));
		bInfinity = false;
	}

	if(farFocus < nearFocus)
		farFocus = nearFocus;

	// convert to proper units
	HyperfocalDistance = HyperfocalDistance / 1000.0 / units;
	nearFocus = nearFocus / 1000.0 / units;
	farFocus  = farFocus / 1000.0 / units;

	//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", HyperfocalDistance/10.0, HyperfocalDistance/10.0/2.54, HyperfocalDistance/10.0/2.54/12.0);
	sprintf(buffer, "%.4f cm | %.4f in | %.4f ft", HyperfocalDistance, HyperfocalDistance/2.54, HyperfocalDistance/2.54/12.0);

	IFDENTRY ifde(0xFFEE, 2, 0, 0);

	IFDElement ifdelement(ifde, itNone);
	ifdelement.setProgramName(string("d70reader"));
	ifdelement.setProgramVersion(string(d70Version));
	ifdelement.setProgramDate(string(d70VersionDate));
	ifdelement.setFileTypeOffset(fileTypeOffset());
	ifdelement.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement.setFileType(iFileType);
	ifdelement.setASCILabelSkipLength(iASCILabelSkipLength);
	ifdelement.setDisplayData(buffer);

	listIFDElements.insert(listIFDElements.end(), ifdelement);
	
	//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", nearFocus/10.0, nearFocus/10.0/2.54, nearFocus/10.0/2.54/12.0);
	sprintf(buffer, "%.4f cm | %.4f in | %.4f ft", nearFocus, nearFocus/2.54, nearFocus/2.54/12.0);

	IFDENTRY ifde1(0xFFEC, 2, 0, 0);

	IFDElement ifdelement1(ifde1, itNone);
	ifdelement1.setProgramName(string("d70reader"));
	ifdelement1.setProgramVersion(string(d70Version));
	ifdelement1.setProgramDate(string(d70VersionDate));
	ifdelement1.setFileTypeOffset(fileTypeOffset());
	ifdelement1.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement1.setFileType(iFileType);
	ifdelement1.setASCILabelSkipLength(iASCILabelSkipLength);
	ifdelement1.setDisplayData(buffer);

	listIFDElements.insert(listIFDElements.end(), ifdelement1);

	if(bInfinity)
	{
		//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", farFocus/10.0, farFocus/10.0/2.54, farFocus/10.0/2.54/12.0);
		sprintf(buffer, "%s", "Infinity");
	}
	else
	{
		//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", farFocus/10.0, farFocus/10.0/2.54, farFocus/10.0/2.54/12.0);
		sprintf(buffer, "%.4f cm | %.4f in | %.4f ft", farFocus, farFocus/2.54, farFocus/2.54/12.0);
	}

	IFDENTRY ifde2(0xFFED, 2, 0, 0);

	IFDElement ifdelement2(ifde2, itNone);
	ifdelement2.setProgramName(string("d70reader"));
	ifdelement2.setProgramVersion(string(d70Version));
	ifdelement2.setProgramDate(string(d70VersionDate));
	ifdelement2.setFileTypeOffset(fileTypeOffset());
	ifdelement2.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement2.setFileType(iFileType);
	ifdelement2.setASCILabelSkipLength(iASCILabelSkipLength);
	ifdelement2.setDisplayData(buffer);

	listIFDElements.insert(listIFDElements.end(), ifdelement2);
	
	if(bInfinity)
	{
		//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", farFocus/10.0, farFocus/10.0/2.54, farFocus/10.0/2.54/12.0);
		sprintf(buffer, "%s", "Infinity");
	}
	else
	{
		//sprintf(buffer, "%.2f cm | %.2f in | %.2f ft", farFocus/10.0, farFocus/10.0/2.54, farFocus/10.0/2.54/12.0);
		sprintf(buffer, "%.4f cm | %.4f in | %.4f ft", farFocus - nearFocus, (farFocus - nearFocus)/2.54, (farFocus - nearFocus)/2.54/12.0);
	}

	IFDENTRY ifde3(0xFFEF, 2, 0, 0);

	IFDElement ifdelement3(ifde3, itNone);
	ifdelement3.setProgramName(string("d70reader"));
	ifdelement3.setProgramVersion(string(d70Version));
	ifdelement3.setProgramDate(string(d70VersionDate));
	ifdelement3.setFileTypeOffset(fileTypeOffset());
	ifdelement3.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement3.setFileType(iFileType);
	ifdelement3.setASCILabelSkipLength(iASCILabelSkipLength);
	ifdelement3.setDisplayData(buffer);

	listIFDElements.insert(listIFDElements.end(), ifdelement3);
	
	IFDENTRY ifde4(0xFFF0, 2, 0, 0);

	IFDElement ifdelement4(ifde4, itNone);
	ifdelement4.setProgramName(string("d70reader"));
	ifdelement4.setProgramVersion(string(d70Version));
	ifdelement4.setProgramDate(string(d70VersionDate));
	ifdelement4.setFileTypeOffset(fileTypeOffset());
	ifdelement4.setMakerNoteHdrOffset(ulMakerNoteHeaderOffset);
	ifdelement4.setFileType(iFileType);
	ifdelement4.setASCILabelSkipLength(iASCILabelSkipLength);
	ifdelement4.setDisplayData(strCicleOfConfusion.c_str());

	listIFDElements.insert(listIFDElements.end(), ifdelement4);
	
//  { 0, 65516u, "NearFocusLimitDOF_Type0          " },	//0xFFEC
//  { 0, 65517u, "FarFocusLimitDOF_Type0           " },	//0xFFED
//  { 0, 65518u, "HyperfocalDistance_Type0         " },	//0xFFEE
//  { 0, 65519u, "TotalDOF_Type0                   " },	//0xFFEF
//  { 0, 65520u, "CircleOfConfusion_Type0          " },	//0xFFF0
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
struct tm *getTmTime(string& strLocal)
{
	static struct tm when;
	struct tm *tm_ptr;

	time_t tmResult;
	time_t now;

	char *temp_ptr = NULL;

	char *year_token = NULL;
	char *month_token = NULL;
	char *day_token = NULL;
	char *hour_token = NULL;
	char *minute_token = NULL;
	char *second_token = NULL;

	//2005:07:10 23:18:41
	temp_ptr = strdup(strLocal.c_str());
	year_token = strtok(temp_ptr, ": ");
	month_token = strtok(NULL, ": ");
	day_token = strtok(NULL, ": ");
	hour_token = strtok(NULL, ": ");
	minute_token = strtok(NULL, ": ");
	second_token = strtok(NULL, ": ");

	time( &now );
	when = *localtime( &now );
	tm_ptr = &when;

	int iSec = atoi(second_token);
	int iMin = atoi(minute_token);
	int iHour = atoi(hour_token);
	int iMon = atoi(month_token);

	if((iSec - 1) != -1)
		when.tm_sec = iSec - 1;   /* seconds after the minute - [0,59] */
	else
		when.tm_sec = 59;

	if((iMin - 1) != -1)
		when.tm_min = iMin - 1;   /* minutes after the hour - [0,59] */
	else
		when.tm_min = 59;

	if((iHour - 1) != -1)
		when.tm_hour = iHour - 1;	/* hours since midnight - [0,23] */
	else
		when.tm_hour = 23;
	
	if((iMon - 1) != -1)
		when.tm_mon = iMon - 1;    /* months since January - [0,11] */
	else
		when.tm_mon = 11;

	when.tm_mday = atoi(day_token);			/* day of the month - [1,31] */
	when.tm_year = atoi(year_token) - 1900; /* years since 1900 */

	tmResult = mktime(&when);

	if(when.tm_isdst)
	{
		if((iHour - 1) != -1)
			when.tm_hour = iHour - 1;	/* hours since midnight - [0,23] */
		else
			when.tm_hour = 23;
	}

	return &when;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
string& setTmTime(time_t tmLocal)
{
	char buffer[24];
	struct tm *tm_ptr;
	static string strFinal;
	strFinal.resize(0);

	tm_ptr = localtime(&tmLocal);

	memset(buffer, (int) '\0', 24);

	//2005:07:10 23:18:41
	sprintf(buffer, 
			"%4d:%02d:%02d %02d:%02d:%02d", 
			tm_ptr->tm_year + 1900, 
			tm_ptr->tm_mon + 1,
			tm_ptr->tm_mday,
			tm_ptr->tm_hour + 1,
			tm_ptr->tm_min + 1,
			tm_ptr->tm_sec + 1);

	strFinal = string(buffer);

	return strFinal;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void fixDateWithOffset(char *filename, FILE *rfd)
{
	bool bLocalTrace = false;
	FILE *ofd = NULL;
	unsigned long size = 0l;
	struct tm *tmTime;
	time_t tmResult;

	string strTempFile;
	string strFilePath;
	string strFileName;
	string strNewModifyDate;
	string strNewDateTimeOriginal;
	string strNewDateTimeDigitized;

	strFilePath.append(getFilePath(filename));
	strFileName.append(getFileName(filename));

	//d70reader -fd -od:p:\work\junk\ f:\share\d70\*.jpg
	if(bOutputDirectory)
		strTempFile = strOutputDirectory;
	else
		strTempFile = strFilePath;

#ifdef WIN_GUI
	strTempFile += string("d70readerw");
#else
	strTempFile += string("d70reader");
#endif

	bool bFound = isDirectoryFound(strTempFile);

	if(!bFound)
		my_mkdir((char *) strTempFile.c_str());

	if(iSystemType)
		strTempFile += string("\\fix_date");
	else
		strTempFile += string("/fix_date");

	bFound = isDirectoryFound(strTempFile);

	if(!bFound)
		my_mkdir((char *) strTempFile.c_str());

	if(iSystemType)
		strTempFile += string("\\");
	else
		strTempFile += string("/");
	
	strTempFile.append(strFileName);

	if((ofd = fopen(strTempFile.c_str(), "w+b")) == NULL)
	{
		string strTemp = string("\nFile not created: ") + strTempFile + string("\nOpen error\n");
		ThrowD70Exception(strTemp);
	}

	fseek( rfd, 0l, SEEK_SET );

	unsigned char *pChar = new unsigned char[lFileSize];

	fread((void *) pChar, lFileSize, 1, rfd);

	if((size = fwrite((void *) pChar, lFileSize, 1, ofd)) == 0)
	{
		string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
		ThrowD70Exception(strTemp);
	}

	delete pChar;

	//start of ModifyDate
	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x132)
			{
				if(bLocalTrace)
				{
					//cout << "\nOffset: " << itrIFDElements->getValueWriteOffset();
					strGlobalCOut.append("\nOffset: ");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", itrIFDElements->getValueWriteOffset());
					strGlobalCOut.append(glb_buffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				string strBuffer;

				strBuffer.resize(itrIFDElements->getIFD().countval());

				fread((void *) strBuffer.c_str(), itrIFDElements->getIFD().countval(), 1, ofd);

				//2005:07:10 23:18:41
				tmTime = getTmTime(strBuffer);

				tmTime->tm_min += iFixDateOffset;

				tmResult = mktime(tmTime);

				strNewModifyDate = setTmTime(tmResult);

				if(bLocalTrace)
				{
					//cout << "\nstrBuffer: " << strBuffer;
					strGlobalCOut.append("\nstrBuffer: ");
					strGlobalCOut.append(strBuffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				if((size = fwrite((void *) strNewModifyDate.c_str(), itrIFDElements->getIFD().countval(), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}
		}
	}
	//end of ModifyDate

	//start of DateTimeOriginal
	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x9003)
			{
				if(bLocalTrace)
				{
					//cout << "\nOffset: " << itrIFDElements->getValueWriteOffset();
					strGlobalCOut.append("");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", itrIFDElements->getValueWriteOffset());
					strGlobalCOut.append(glb_buffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				string strBuffer;

				strBuffer.resize(itrIFDElements->getIFD().countval());

				fread((void *) strBuffer.c_str(), itrIFDElements->getIFD().countval(), 1, ofd);

				//2005:07:10 23:18:41
				tmTime = getTmTime(strBuffer);

				tmTime->tm_min += iFixDateOffset;

				tmResult = mktime(tmTime);

				strNewDateTimeOriginal = setTmTime(tmResult);

				if(bLocalTrace)
				{
					//cout << "\nstrBuffer: " << strBuffer;
					strGlobalCOut.append("\nstrBuffer: ");
					strGlobalCOut.append(strBuffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				if((size = fwrite((void *) strNewDateTimeOriginal.c_str(), itrIFDElements->getIFD().countval(), 1, ofd)) == 0)
				{
					string strTemp = string("File not written[1]: ") + strTempFile + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}
		}
	}
	//end of DateTimeOriginal

	//start of DateTimeDigitized
	if(!listIFDElements.empty())
	{
		for( itrIFDElements = listIFDElements.begin(); itrIFDElements != listIFDElements.end(); itrIFDElements++)
		{
			if(itrIFDElements->getTag() == 0x9004)
			{
				if(bLocalTrace)
				{
					//cout << "\nOffset: " << itrIFDElements->getValueWriteOffset();
					strGlobalCOut.append("\nOffset: ");

					memset(glb_buffer, '\0', GLB_SIZE);
					sprintf(glb_buffer, "%d", itrIFDElements->getValueWriteOffset());
					strGlobalCOut.append(glb_buffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				string strBuffer;

				strBuffer.resize(itrIFDElements->getIFD().countval());

				fread((void *) strBuffer.c_str(), itrIFDElements->getIFD().countval(), 1, ofd);

				//2005:07:10 23:18:41
				tmTime = getTmTime(strBuffer);

				tmTime->tm_min += iFixDateOffset;

				tmResult = mktime(tmTime);

				strNewDateTimeDigitized = setTmTime(tmResult);

				if(bLocalTrace)
				{
					//cout << "\nstrBuffer: " << strBuffer;
					strGlobalCOut.append("\nstrBuffer: ");
					strGlobalCOut.append(strBuffer);
				}

				fseek( ofd, itrIFDElements->getValueWriteOffset() + fileTypeOffset(), SEEK_SET );

				if((size = fwrite((void *) strNewDateTimeDigitized.c_str(), itrIFDElements->getIFD().countval(), 1, ofd)) == 0)
				{
					string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
					ThrowD70Exception(strTemp);
				}
			}
		}
	}
	//end of DateTimeDigitized

	fclose(ofd);

	struct utimbuf utimeb;
	struct stat buf;
	int result;
	result = stat( filename, &buf );

	utimeb.actime = buf.st_atime;
	utimeb.modtime = buf.st_mtime;

	utime(strTempFile.c_str(), &utimeb);

	//cout << "\nCreated File: " << strTempFile;
	strGlobalCOut.append("\nCreated File: ");
	strGlobalCOut.append(strTempFile);

	if(bLocalTrace)
	{
		//cout << "\nstrFilePath: " << strFilePath;
		strGlobalCOut.append("\nstrFilePath: ");
		strGlobalCOut.append(strFilePath);

		//cout << "\nstrFileName: " << strFileName;
		strGlobalCOut.append("\nstrFileName: ");
		strGlobalCOut.append(strFileName);

		//cout << "\nstrTempFile: " << strTempFile;
		strGlobalCOut.append("\nstrTempFile: ");
		strGlobalCOut.append(strTempFile);

		//cout << '\n' << strModifyDate;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strModifyDate);

		//cout << '\n' << strDateTimeOriginal;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strDateTimeOriginal);

		//cout << '\n' << strDateTimeDigitized;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strDateTimeDigitized);

		//cout << "\n\nLocal Date: " << getLocalTime() << '\n';
		strGlobalCOut.append("\n\nLocal Date: ");
		strGlobalCOut.append(getLocalTime());
		strGlobalCOut.append("\n");
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void renameImage(char *filename, FILE *rfd)
{
	bool bLocalTrace = false;
	FILE *ofd = NULL;
	unsigned long size = 0l;

	string strTempFile;
	string strFilePath;
	string strFileName;

	strFilePath.append(getFilePath(filename));
	//strFileName.append(getFileName(filename));

	//d70reader -fd -od:p:\work\junk\ f:\share\d70\*.jpg
	if(bOutputDirectory)
		strTempFile = strOutputDirectory;
	else
		strTempFile = strFilePath;

#ifdef WIN_GUI
	strTempFile += string("d70readerw");
#else
	strTempFile += string("d70reader");
#endif

	bool bFound = isDirectoryFound(strTempFile);

	if(!bFound)
		my_mkdir((char *) strTempFile.c_str());

	if(iSystemType)
		strTempFile += string("\\rename_image");
	else
		strTempFile += string("/rename_image");

	bFound = isDirectoryFound(strTempFile);

	if(!bFound)
		my_mkdir((char *) strTempFile.c_str());

	if(iSystemType)
		strTempFile += string("\\");
	else
		strTempFile += string("/");
	
	string strShutterCount = getDataValue(0x00a7, 9, true);

	if(!strShutterCount.length())
	{
		//cout << "\nFile does not contain Shutter Count: " << filename;
		strGlobalCOut.append("\nFile does not contain Shutter Count: ");
		strGlobalCOut.append(filename);
		return;
	}

	strFileName = strRenamePrefix + strShutterCount;

	if(!iFileType)
		strFileName.append(".JPG");
	else
		strFileName.append(".NEF");

	struct stat buf;

	strTempFile.append(strFileName);

	if(stat(strTempFile.c_str(), &buf) != (-1))
	{
		//cout << "\nTarget file already exists: " << strTempFile;
		strGlobalCOut.append("\nTarget file already exists: ");
		strGlobalCOut.append(strTempFile);
		return;
	}

	if((ofd = fopen(strTempFile.c_str(), "w+b")) == NULL)
	{
		string strTemp = string("\nFile not created: ") + strTempFile + string("\nOpen error\n");
		ThrowD70Exception(strTemp);
	}

	fseek( rfd, 0l, SEEK_SET );

	unsigned char *pChar = new unsigned char[lFileSize];

	fread((void *) pChar, lFileSize, 1, rfd);

	if((size = fwrite((void *) pChar, lFileSize, 1, ofd)) == 0)
	{
		string strTemp = string("\nFile not written[1]: ") + strTempFile + string("\nWrite error\n");
		ThrowD70Exception(strTemp);
	}

	delete pChar;

	fclose(ofd);

	struct utimbuf utimeb;
	int result;
	result = stat( filename, &buf );

	utimeb.actime = buf.st_atime;
	utimeb.modtime = buf.st_mtime;

	utime(strTempFile.c_str(), &utimeb);

	//cout << "\nCreated File: " << strTempFile;
	strGlobalCOut.append("\nCreated File: ");
	strGlobalCOut.append(strTempFile);

	if(bLocalTrace)
	{
		//cout << "\nstrFilePath: " << strFilePath;
		strGlobalCOut.append("\nstrFilePath: ");
		strGlobalCOut.append(strFilePath);

		//cout << "\nstrFileName: " << strFileName;
		strGlobalCOut.append("\nstrFileName: ");
		strGlobalCOut.append(strFileName);

		//cout << "\nstrTempFile: " << strTempFile;
		strGlobalCOut.append("\nstrTempFile: ");
		strGlobalCOut.append(strTempFile);

		//cout << '\n' << strModifyDate;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strModifyDate);

		//cout << '\n' << strDateTimeOriginal;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strDateTimeOriginal);

		//cout << '\n' << strDateTimeDigitized;
		strGlobalCOut.append("\n");
		strGlobalCOut.append(strDateTimeDigitized);

		//cout << "\n\nLocal Date: " << getLocalTime() << '\n';
		strGlobalCOut.append("\n\nLocal Date: ");
		strGlobalCOut.append(getLocalTime());
		strGlobalCOut.append("\n");
	}
}

#ifdef WIN_GUI
//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void resetFlags()
{
	bTrace = false;		// set this with -t option 
	bVerbos = false;		// set this with -v option
	bHtml = false;			// set this with -w option
	bXml = false;			// set this with -x option
	bThumbnail = false;	// set this with -j option
	bThumbnailOld = false;	// set this with -jold option
	bCsv = false;			// set this with -c option
	bJS = false;			// set this with -js option
	bWeb = false;			// set this with -web option
	bRemoveEXIF = false;	// set this with -r option
	bOutputPlain = true;	// turned off if certain switches are turned on
	bNewConfig = false;	// set this with -cf option
	bOutputDirectory = false;	// set this with -od option
	bRestoreEXIF = false;	// set this with -re option
	bFixDate = false;		// set this with -fd option
	bJpegFine = false;		// set this with -njf option
	bMainUseCameraWB = false;	// set this with -cwb option
	bMainUseAutoWB = false;	// set this with -cwb option
	bMainUseCameraRGB = false;	// set this with -crgb option
	bMainBright = false;	// set this with -br option
	bMainRed = false;		// set this with -red option
	bMainBlue = false;		// set this with -blue option
	bMainBlack = false;		// set this with -black option
	bMainDoNotClipColor = false;	// set this with -noclip option

	//listWantedTags.clear();
	//listWebConfig.clear();
	//listIFDElements.clear();
	//listIFDElements2.clear();

	int iList1 = listIFDElements.size();
	int iList2 = listIFDElements2.size();

}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void resetLists()
{
	if(GlobalOrientationPtr)
		free(GlobalOrientationPtr);

	listWantedTags.clear();
	listWebConfig.clear();
	listIFDElements.clear();
	listIFDElements2.clear();

	int iList1 = listIFDElements.size();
	int iList2 = listIFDElements2.size();

}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void rebuildConfig(CString strFilePtr, LISTWANTEDTAGS2* pList)
{
	CString strTemp;
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char *file_ptr = NULL;
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;
	LISTWANTEDTAGS2::iterator itrWantedTags2;
	unsigned short usTag = 0;
	int iType = 0;
	int iWriteLen = 0;
	WantedTags wtagelement;

	file_ptr = strFilePtr.GetBuffer(_MAX_PATH + _MAX_FNAME);
	remove(file_ptr);
	listWantedTags.clear();

	if(stat(file_ptr, &buf) == (-1))
	{
		if((ofd = fopen(file_ptr, "w")) == NULL)
		{
			ThrowD70Exception(string("File not created: ") + string(file_ptr));
		}


		for( itrWantedTags2 = pList->begin(); itrWantedTags2 != pList->end(); itrWantedTags2++)
		{
			usTag = itrWantedTags2->getTag();
			iType = itrWantedTags2->getIFDType();
			strTemp = CString(TiffTagName(usTag, iType)).SpanExcluding(" ");
			char *pTiffTagName = strTemp.GetBuffer(strTemp.GetLength());

			string strTagName(pTiffTagName);

			if(strTagName.compare(0, 7, "Unknown"))
			{
				data_buffer_ptr = (char *) malloc(strlen(strTemp.GetBuffer(strTemp.GetLength())) + 6);
				memset(data_buffer_ptr, (int) '\0', (strlen(strTemp.GetBuffer(strTemp.GetLength())) + 6));

				if(itrWantedTags2->isWanted())
				{
					sprintf(data_buffer_ptr, "%s=yes\n", strTemp.GetBuffer(strTemp.GetLength()));
					iWriteLen = 5;
				}
				else
				{
					sprintf(data_buffer_ptr, "%s=no\n", strTemp.GetBuffer(strTemp.GetLength()));
					iWriteLen = 4;
				}

				if((size = fwrite(data_buffer_ptr, (strlen(strTemp.GetBuffer(strTemp.GetLength())) + iWriteLen), 1, ofd)) == 0)
				{
					cout << "\nFile not written: " << file_ptr << "\n\n";
					free(data_buffer_ptr);
					return;
				}

				if(itrWantedTags2->isWanted())
					wtagelement = WantedTags(true, usTag, iType);
				else
					wtagelement = WantedTags(false, usTag, iType);

				listWantedTags.insert(listWantedTags.begin(), wtagelement);

				free(data_buffer_ptr);
			}
		}

		fclose(ofd);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void checkForConfig(CString strFilePtr)
{
	CString strTemp;
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char buffer[256];
	char *file_ptr = NULL;
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;
	struct TAGNAMES *pTiffTagData = NULL;

	file_ptr = strFilePtr.GetBuffer(_MAX_PATH + _MAX_FNAME);

	if(stat(file_ptr, &buf) == (-1))
	{
		if((ofd = fopen(file_ptr, "w")) == NULL)
		{
			ThrowD70Exception(string("File not created: ") + string(file_ptr));
		}

		for(int ctr0 = 0; ctr0 < 11; ctr0++)
		{
			for(ctr1 = 0; ctr1 < 0xffff; ctr1++)
			{
				pTiffTagData = TiffTagData(ctr1, ctr0);

				if(pTiffTagData != NULL)
				{
					strTemp = CString(pTiffTagData->name).SpanExcluding(" ");
					char *pTiffTagName = strTemp.GetBuffer(strTemp.GetLength());

					string strTagName(pTiffTagName);

					if(strTagName.compare(0, 7, "Unknown"))
					{
						data_buffer_ptr = (char *) malloc(strlen(strTemp.GetBuffer(strTemp.GetLength())) + 6);
						memset(data_buffer_ptr, (int) '\0', (strlen(strTemp.GetBuffer(strTemp.GetLength())) + 6));
						sprintf(data_buffer_ptr, "%s=yes\n", strTemp.GetBuffer(strTemp.GetLength()));

						if((size = fwrite(data_buffer_ptr, (strlen(strTemp.GetBuffer(strTemp.GetLength())) + 5), 1, ofd)) == 0)
						{
							cout << "\nFile not written: " << file_ptr << "\n\n";
							free(data_buffer_ptr);
							return;
						}

						WantedTags wtagelement = WantedTags(true, ctr1, pTiffTagData->type);
						listWantedTags.insert(listWantedTags.begin(), wtagelement);

						free(data_buffer_ptr);
					}
				}
			}
		}

		fclose(ofd);
	}
	else
	{
		if((rfd = fopen(file_ptr, "r")) == NULL)
		{
			cout << "\nFile not opened: " << file_ptr << "\n\n";
			return;
		}

		do
		{
			memset(buffer, (int) '\0', 256);
			data_buffer_ptr = fgets(buffer, 256, rfd);

			if(data_buffer_ptr == NULL)
				break;

			WantedTags wtagelement;

			strTemp = CString(buffer).SpanExcluding("=");
			char *pTiffTagName = strTemp.GetBuffer(strTemp.GetLength());
			pTiffTagData = TiffTagData(pTiffTagName);

			if(isTagYes(buffer))
				wtagelement = WantedTags(true, pTiffTagData->tag, pTiffTagData->type);
			else
				wtagelement = WantedTags(false, pTiffTagData->tag, pTiffTagData->type);
				//wtagelement = WantedTags(false, getTagFromDescription(buffer), TiffTypeNumber(getTagFromDescription(buffer)));

			listWantedTags.insert(listWantedTags.begin(), wtagelement);
		}
		while(data_buffer_ptr != NULL);

		fclose(rfd);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void checkForWebConfig(CString strFilePtr)
{
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char buffer[1024];
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;

	string strWebConfigFile = strFilePtr.GetBuffer(_MAX_PATH + _MAX_FNAME);

	if(stat(strWebConfigFile.c_str(), &buf) == (-1))
	{
		if((ofd = fopen(strWebConfigFile.c_str(), "w")) == NULL)
		{
			ThrowD70Exception(string("File not created: ") + string(strWebConfigFile.c_str()));
		}

		fputs("D70ReaderSelectionIETitle=D70Reader Gallery Selection", ofd);
		fputs("\nD70ReaderSelectionTitle=D70Reader Gallery Selection", ofd);
		fputs("\nD70ReaderSelectionThumbnailNote=click on thumbnails for gallery index", ofd);
		fputs("\nD70ReaderSelectionGalleryPrefix=Gallery", ofd);
		fputs("\nD70ReaderGalleryIETitle=D70Reader Gallery", ofd);
		fputs("\nD70ReaderGalleryPrefix=Gallery", ofd);
		fputs("\nD70ReaderGalleryBackIndex=Back to Gallery Index", ofd);
		fputs("\nD70ReaderGalleryThumbnailNote=click on thumbnails for full image", ofd);
		fputs("\nD70ReaderSlideshowIETitle=D70Reader Slideshow", ofd);
		fputs("\nD70ReaderSlideshowTitle=D70Reader Image &amp; EXIF Data", ofd);
		fputs("\nD70ReaderSlideshowExifOn=exif on", ofd);
		fputs("\nD70ReaderSlideshowExifOff=exif off", ofd);
		fputs("\nD70ReaderSlideshowPrevious=previous", ofd);
		fputs("\nD70ReaderSlideshowNext=next", ofd);
		fputs("\nD70ReaderSlideshowBackToGallery=Back to Gallery\n", ofd);	//the last one should have a newline

		fclose(ofd);

		addToWebConfig(string("D70ReaderSelectionIETitle=D70Reader Gallery Selection"));
		addToWebConfig(string("D70ReaderSelectionTitle=D70Reader Gallery Selection"));
		addToWebConfig(string("D70ReaderSelectionThumbnailNote=click on thumbnails for gallery index"));
		addToWebConfig(string("D70ReaderSelectionGalleryPrefix=Gallery"));
		addToWebConfig(string("D70ReaderGalleryIETitle=D70Reader Gallery"));
		addToWebConfig(string("D70ReaderGalleryPrefix=Gallery"));
		addToWebConfig(string("D70ReaderGalleryBackIndex=Back to Gallery Index"));
		addToWebConfig(string("D70ReaderGalleryThumbnailNote=click on thumbnails for full image"));
		addToWebConfig(string("D70ReaderSlideshowIETitle=D70Reader Slideshow"));
		addToWebConfig(string("D70ReaderSlideshowTitle=D70Reader Image &amp; EXIF Data"));
		addToWebConfig(string("D70ReaderSlideshowExifOn=exif on"));
		addToWebConfig(string("D70ReaderSlideshowExifOff=exif off"));
		addToWebConfig(string("D70ReaderSlideshowPrevious=previous"));
		addToWebConfig(string("D70ReaderSlideshowNext=next"));
		addToWebConfig(string("D70ReaderSlideshowBackToGallery=Back to Gallery"));
	}
	else
	{
		if((rfd = fopen(strWebConfigFile.c_str(), "r")) == NULL)
		{
			cout << "\nFile not opened: " << strWebConfigFile << "\n\n";
			return;
		}

		do
		{
			memset(buffer, (int) '\0', 1024);
			data_buffer_ptr = fgets(buffer, 1024, rfd);

			if(data_buffer_ptr == NULL)
				break;

			string strBuffer = string(buffer);
			strBuffer.resize(strBuffer.length() - 1);

			WebConfig element = WebConfig(strBuffer);
			listWebConfig.insert(listWebConfig.end(), element);
		}
		while(data_buffer_ptr != NULL);
	}
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void setDateDigitized(CString& strDateDigitized)
{
	strDateTimeDigitized = string(strDateDigitized.GetBuffer(strDateDigitized.GetLength()));

}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void setOriginalDate(CString& strOrigDate)
{
	strDateTimeOriginal = string(strOrigDate.GetBuffer(strOrigDate.GetLength()));
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void setModifyDate(CString& strModifyDate_in)
{
	strModifyDate = string(strModifyDate_in.GetBuffer(strModifyDate_in.GetLength()));
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void setJSFirstTime(bool bFlag, int iCount)
{
	bJSFirstTime = bFlag;
	js_ctr1 = 0;
	js_total_count = iCount;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void setCSVFirstTime(bool bFlag)
{
	bCsvFirstTime = bFlag;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void checkForDateConfig(CString strFilePtr)
{
	int ctr1 = 0;
	unsigned long size = 0l;
	struct stat buf;
	char buffer[1024];
	char *data_buffer_ptr = NULL;
	FILE *ofd = NULL;
	FILE *rfd = NULL;
	struct wanted_tags *local_header_ptr = NULL;
	struct all_wanted_tags *local_header_all_ptr = NULL;

	string strWebConfigFile = strFilePtr.GetBuffer(_MAX_PATH + _MAX_FNAME);

	string strLocalTime;

	strLocalTime = getLocalTime();

	if(stat(strWebConfigFile.c_str(), &buf) == (-1))
	{
		if((ofd = fopen(strWebConfigFile.c_str(), "w")) == NULL)
		{
			ThrowD70Exception(string("File not created: ") + string(strWebConfigFile.c_str()));
		}

		string strTempTime;

		strTempTime = string("ModifyDate=") + strLocalTime + string("\n");
		fputs(strTempTime.c_str(), ofd);

		strTempTime = string("DateTimeOriginal=") + strLocalTime + string("\n");
		fputs(strTempTime.c_str(), ofd);
		
		strTempTime = string("DateTimeDigitized=") + strLocalTime + string("\n");
		fputs(strTempTime.c_str(), ofd);

		fclose(ofd);

		strModifyDate = strLocalTime;
		strDateTimeOriginal = strLocalTime;
		strDateTimeDigitized = strLocalTime;
	}
	else
	{
		if((rfd = fopen(strWebConfigFile.c_str(), "r")) == NULL)
		{
			cout << "\nFile not opened: " << strWebConfigFile << "\n\n";
			return;
		}

		do
		{
			memset(buffer, (int) '\0', 1024);
			data_buffer_ptr = fgets(buffer, 1024, rfd);

			if(data_buffer_ptr == NULL)
				break;

			string strBuffer = string(buffer);
			strBuffer.resize(strBuffer.length() - 1);

			if(!strBuffer.compare(0, 11, "ModifyDate="))
				strModifyDate = strBuffer.substr((strBuffer.find_first_of("=", 0) + 1),19);
			else if(!strBuffer.compare(0, 17, "DateTimeOriginal="))
				strDateTimeOriginal = strBuffer.substr((strBuffer.find_first_of("=", 0) + 1),19);
			else if(!strBuffer.compare(0, 18, "DateTimeDigitized="))
				strDateTimeDigitized = strBuffer.substr((strBuffer.find_first_of("=", 0) + 1),19);
		}
		while(data_buffer_ptr != NULL);

		fclose(rfd);
	}
}

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
bool isTagWanted(unsigned short usTag)
{
	bool isFound = false;

	for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
	{
		if(itrWantedTags->getTag() == usTag)
			return itrWantedTags->isWanted();
	}

	return isFound;
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void outputJSCout(bool bFirstTime, int iIndex)
{
	int ctr1 = 0;
	char buffer[1024];
	bool bFirstTag = false;

#ifdef JON
	if(bTrace)
	{
		if(!listWantedTags.empty())
		{
			for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
			{
				if(itrWantedTags->isWanted())
				{
					cout << '\n' << "Tag: " << itrWantedTags->getTag();

					CString strTemp;
					strTemp = CString(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType())).SpanExcluding(" ");
					//cout << '\n' << strip(TiffTagName( itrWantedTags->getTag()));
					cout << '\n' << strTemp.GetBuffer(strTemp.GetLength());
				}
			}

			cout << '\n';
		}
	}
#endif

	if(bFirstTime)
	{
		cout << "aExifData = new Array()";
		cout << "\naExifDataHeader = new Array()";
		cout << "\naExifDataHeader[0] = new Array(";

		if(!listWantedTags.empty())
		{
			for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
			{
				if(itrWantedTags->isWanted())
				{
					if(bFirstTag)
						cout << ",";
					else
						bFirstTag = true;

					//if(itrWantedTags != listWantedTags.begin())
					//	cout << ",";

					CString strTemp;
					strTemp = CString(TiffTagName( itrWantedTags->getTag(), itrWantedTags->getIFDType())).SpanExcluding(" ");
					//sprintf(buffer, "\"%s\"", strip(TiffTagName( itrWantedTags->getTag())));
					sprintf(buffer, "\"%s\"", strTemp.GetBuffer(strTemp.GetLength()));
					cout <<  buffer;
				}
			}
		}

		cout << ")\n";

		bFirstTime = false;
	}

	sprintf(buffer, "aExifData[%d] = new Array(", iIndex);
	cout << buffer;

	bFirstTag = false;

	if(!listWantedTags.empty())
	{
		for( itrWantedTags = listWantedTags.begin(); itrWantedTags != listWantedTags.end(); itrWantedTags++)
		{
			if(itrWantedTags->isWanted())
			{
				if(bFirstTag)
					cout << ",";
				else
					bFirstTag = true;

				//if(itrWantedTags != listWantedTags.begin())
				//	cout << ",";

				string strTemp;
				strTemp = getDataValue(itrWantedTags->getTag(), itrWantedTags->getIFDType(), false);

				sprintf(buffer, "\"%s\"", strTemp.c_str() );
				cout <<  buffer;
			}
		}
	}

	cout << ")\n";
}

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
void removeTempDirectory()
{
	if(str_root_temp_dir != NULL)
	{
		my_rmdir(str_root_temp_dir);
		free(str_root_temp_dir);
	}
}

#endif
